(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(require('vue')) :
	typeof define === 'function' && define.amd ? define(['vue'], factory) :
	(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.Vue));
}(this, (function (Vue) { 'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

	var Vue__default = /*#__PURE__*/_interopDefaultLegacy(Vue);

	function unwrapExports (x) {
		return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
	}

	function createCommonjsModule(fn, module) {
		return module = { exports: {} }, fn(module, module.exports), module.exports;
	}

	var interopRequireDefault = createCommonjsModule(function (module) {
	function _interopRequireDefault(obj) {
	  return obj && obj.__esModule ? obj : {
	    "default": obj
	  };
	}

	module.exports = _interopRequireDefault;
	module.exports["default"] = module.exports, module.exports.__esModule = true;
	});

	var _interopRequireDefault = unwrapExports(interopRequireDefault);

	var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

	function randomExtend(minNum, maxNum) {
	  if (arguments.length === 1) {
	    return parseInt(Math.random() * minNum + 1, 10);
	  } else {
	    return parseInt(Math.random() * (maxNum - minNum + 1) + minNum, 10);
	  }
	}
	function debounce(delay, callback) {
	  let lastTime;
	  return function () {
	    clearTimeout(lastTime);
	    const [that, args] = [this, arguments];
	    lastTime = setTimeout(() => {
	      callback.apply(that, args);
	    }, delay);
	  };
	}
	function observerDomResize(dom, callback) {
	  const MutationObserver = window.MutationObserver || window.WebKitMutationObserver || window.MozMutationObserver;
	  const observer = new MutationObserver(callback);
	  observer.observe(dom, {
	    attributes: true,
	    attributeFilter: ['style'],
	    attributeOldValue: true
	  });
	  return observer;
	}
	function getPointDistance(pointOne, pointTwo) {
	  const minusX = Math.abs(pointOne[0] - pointTwo[0]);
	  const minusY = Math.abs(pointOne[1] - pointTwo[1]);
	  return Math.sqrt(minusX * minusX + minusY * minusY);
	}
	function uuid(hasHyphen) {
	  return (hasHyphen ? 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx' : 'xxxxxxxxxxxx4xxxyxxxxxxxxxxxxxxx').replace(/[xy]/g, function (c) {
	    const r = Math.random() * 16 | 0;
	    const v = c == 'x' ? r : r & 0x3 | 0x8;
	    return v.toString(16);
	  });
	}
	function deepMerge(target, merged) {
	  for (var key in merged) {
	    if (target[key] && typeof target[key] === 'object') {
	      deepMerge(target[key], merged[key]);
	      continue;
	    }

	    if (typeof merged[key] === 'object') {
	      target[key] = (deepClone)(merged[key], true);
	      continue;
	    }

	    target[key] = merged[key];
	  }

	  return target;
	}
	/**
	 * @description Clone an object or array
	 * @param {Object|Array} object Cloned object
	 * @param {Boolean} recursion   Whether to use recursive cloning
	 * @return {Object|Array} Clone object
	 */

	function deepClone(object) {
	  var recursion = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
	  if (!object) return object;
	  var parse = JSON.parse,
	      stringify = JSON.stringify;
	  if (!recursion) return parse(stringify(object));
	  var clonedObj = object instanceof Array ? [] : {};

	  if (object && typeof object === 'object') {
	    for (var key in object) {
	      if (object.hasOwnProperty(key)) {
	        if (object[key] && typeof object[key] === 'object') {
	          clonedObj[key] = deepClone(object[key], true);
	        } else {
	          clonedObj[key] = object[key];
	        }
	      }
	    }
	  }

	  return clonedObj;
	}
	/**
	 * @description Get the coordinates of the specified radian on the circle
	 * @param {Number} x      Circle x coordinate
	 * @param {Number} y      Circle y coordinate
	 * @param {Number} radius Circle radius
	 * @param {Number} radian Specfied radian
	 * @return {Array} Postion of point
	 */

	var sin = Math.sin,
	    cos = Math.cos;
	function getCircleRadianPoint(x, y, radius, radian) {
	  return [x + cos(radian) * radius, y + sin(radian) * radius];
	}

	function filterNonNumber(array) {
	  return array.filter(function (n) {
	    return typeof n === 'number';
	  });
	}

	function mulAdd(nums) {
	  nums = filterNonNumber(nums);
	  return nums.reduce(function (all, num) {
	    return all + num;
	  }, 0);
	}

	function getTwoPointDistance(pointOne, pointTwo) {
	  var minusX = Math.abs(pointOne[0] - pointTwo[0]);
	  var minusY = Math.abs(pointOne[1] - pointTwo[1]);
	  return Math.sqrt(minusX * minusX + minusY * minusY);
	}

	function getPolylineLength(points) {
	  var lineSegments = new Array(points.length - 1).fill(0).map(function (foo, i) {
	    return [points[i], points[i + 1]];
	  });
	  var lengths = lineSegments.map(function (item) {
	    return getTwoPointDistance.apply(void 0, (0, _toConsumableArray2["default"])(item));
	  });
	  return mulAdd(lengths);
	}

	var autoResize = {
	  data() {
	    return {
	      dom: '',
	      width: 0,
	      height: 0,
	      debounceInitWHFun: '',
	      domObserver: ''
	    };
	  },

	  methods: {
	    async autoResizeMixinInit() {
	      const {
	        initWH,
	        getDebounceInitWHFun,
	        bindDomResizeCallback,
	        afterAutoResizeMixinInit
	      } = this;
	      await initWH(false);
	      getDebounceInitWHFun();
	      bindDomResizeCallback();
	      if (typeof afterAutoResizeMixinInit === 'function') afterAutoResizeMixinInit();
	    },

	    initWH(resize = true) {
	      const {
	        $nextTick,
	        $refs,
	        ref,
	        onResize
	      } = this;
	      return new Promise(resolve => {
	        $nextTick(_ => {
	          const dom = this.dom = $refs[ref];
	          this.width = dom ? dom.clientWidth : 0;
	          this.height = dom ? dom.clientHeight : 0;

	          if (!dom) {
	            console.warn('DataV: Failed to get dom node, component rendering may be abnormal!');
	          } else if (!this.width || !this.height) {
	            console.warn('DataV: Component width or height is 0px, rendering abnormality may occur!');
	          }

	          if (typeof onResize === 'function' && resize) onResize();
	          resolve();
	        });
	      });
	    },

	    getDebounceInitWHFun() {
	      const {
	        initWH
	      } = this;
	      this.debounceInitWHFun = debounce(100, initWH);
	    },

	    bindDomResizeCallback() {
	      const {
	        dom,
	        debounceInitWHFun
	      } = this;
	      this.domObserver = observerDomResize(dom, debounceInitWHFun);
	      window.addEventListener('resize', debounceInitWHFun);
	    },

	    unbindDomResizeCallback() {
	      let {
	        domObserver,
	        debounceInitWHFun
	      } = this;
	      if (!domObserver) return;
	      domObserver.disconnect();
	      domObserver.takeRecords();
	      domObserver = null;
	      window.removeEventListener('resize', debounceInitWHFun);
	    }

	  },

	  mounted() {
	    const {
	      autoResizeMixinInit
	    } = this;
	    autoResizeMixinInit();
	  },

	  beforeDestroy() {
	    const {
	      unbindDomResizeCallback
	    } = this;
	    unbindDomResizeCallback();
	  }

	};

	//
	var script$v = {
	  name: 'DvFullScreenContainer',
	  mixins: [autoResize],

	  data() {
	    return {
	      ref: 'full-screen-container',
	      allWidth: 0,
	      scale: 0,
	      datavRoot: '',
	      ready: false
	    };
	  },

	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        initConfig,
	        setAppScale
	      } = this;
	      initConfig();
	      setAppScale();
	      this.ready = true;
	    },

	    initConfig() {
	      const {
	        dom
	      } = this;
	      const {
	        width,
	        height
	      } = screen;
	      this.allWidth = width;
	      dom.style.width = `${width}px`;
	      dom.style.height = `${height}px`;
	    },

	    setAppScale() {
	      const {
	        allWidth,
	        dom
	      } = this;
	      const currentWidth = document.body.clientWidth;
	      dom.style.transform = `scale(${currentWidth / allWidth})`;
	    },

	    onResize() {
	      const {
	        setAppScale
	      } = this;
	      setAppScale();
	    }

	  }
	};

	function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
	    if (typeof shadowMode !== 'boolean') {
	        createInjectorSSR = createInjector;
	        createInjector = shadowMode;
	        shadowMode = false;
	    }
	    // Vue.extend constructor export interop.
	    const options = typeof script === 'function' ? script.options : script;
	    // render functions
	    if (template && template.render) {
	        options.render = template.render;
	        options.staticRenderFns = template.staticRenderFns;
	        options._compiled = true;
	        // functional template
	        if (isFunctionalTemplate) {
	            options.functional = true;
	        }
	    }
	    // scopedId
	    if (scopeId) {
	        options._scopeId = scopeId;
	    }
	    let hook;
	    if (moduleIdentifier) {
	        // server build
	        hook = function (context) {
	            // 2.3 injection
	            context =
	                context || // cached call
	                    (this.$vnode && this.$vnode.ssrContext) || // stateful
	                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
	            // 2.2 with runInNewContext: true
	            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
	                context = __VUE_SSR_CONTEXT__;
	            }
	            // inject component styles
	            if (style) {
	                style.call(this, createInjectorSSR(context));
	            }
	            // register component module identifier for async chunk inference
	            if (context && context._registeredComponents) {
	                context._registeredComponents.add(moduleIdentifier);
	            }
	        };
	        // used by ssr in case component is cached and beforeCreate
	        // never gets called
	        options._ssrRegister = hook;
	    }
	    else if (style) {
	        hook = shadowMode
	            ? function (context) {
	                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
	            }
	            : function (context) {
	                style.call(this, createInjector(context));
	            };
	    }
	    if (hook) {
	        if (options.functional) {
	            // register for functional component in vue file
	            const originalRender = options.render;
	            options.render = function renderWithStyleInjection(h, context) {
	                hook.call(context);
	                return originalRender(h, context);
	            };
	        }
	        else {
	            // inject component registration as beforeCreate hook
	            const existing = options.beforeCreate;
	            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
	        }
	    }
	    return script;
	}

	const isOldIE = typeof navigator !== 'undefined' &&
	    /msie [6-9]\\b/.test(navigator.userAgent.toLowerCase());
	function createInjector(context) {
	    return (id, style) => addStyle(id, style);
	}
	let HEAD;
	const styles = {};
	function addStyle(id, css) {
	    const group = isOldIE ? css.media || 'default' : id;
	    const style = styles[group] || (styles[group] = { ids: new Set(), styles: [] });
	    if (!style.ids.has(id)) {
	        style.ids.add(id);
	        let code = css.source;
	        if (css.map) {
	            // https://developer.chrome.com/devtools/docs/javascript-debugging
	            // this makes source maps inside style tags work properly in Chrome
	            code += '\n/*# sourceURL=' + css.map.sources[0] + ' */';
	            // http://stackoverflow.com/a/26603875
	            code +=
	                '\n/*# sourceMappingURL=data:application/json;base64,' +
	                    btoa(unescape(encodeURIComponent(JSON.stringify(css.map)))) +
	                    ' */';
	        }
	        if (!style.element) {
	            style.element = document.createElement('style');
	            style.element.type = 'text/css';
	            if (css.media)
	                style.element.setAttribute('media', css.media);
	            if (HEAD === undefined) {
	                HEAD = document.head || document.getElementsByTagName('head')[0];
	            }
	            HEAD.appendChild(style.element);
	        }
	        if ('styleSheet' in style.element) {
	            style.styles.push(code);
	            style.element.styleSheet.cssText = style.styles
	                .filter(Boolean)
	                .join('\n');
	        }
	        else {
	            const index = style.ids.size - 1;
	            const textNode = document.createTextNode(code);
	            const nodes = style.element.childNodes;
	            if (nodes[index])
	                style.element.removeChild(nodes[index]);
	            if (nodes.length)
	                style.element.insertBefore(textNode, nodes[index]);
	            else
	                style.element.appendChild(textNode);
	        }
	    }
	}

	/* script */
	const __vue_script__$v = script$v;

	/* template */
	var __vue_render__$v = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    { ref: _vm.ref, attrs: { id: "dv-full-screen-container" } },
	    [_vm.ready ? [_vm._t("default")] : _vm._e()],
	    2
	  )
	};
	var __vue_staticRenderFns__$v = [];
	__vue_render__$v._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$v = function (inject) {
	    if (!inject) return
	    inject("data-v-03cf1fd3_0", { source: "#dv-full-screen-container {\n  position: fixed;\n  top: 0px;\n  left: 0px;\n  overflow: hidden;\n  transform-origin: left top;\n  z-index: 999;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,eAAe;EACf,QAAQ;EACR,SAAS;EACT,gBAAgB;EAChB,0BAA0B;EAC1B,YAAY;AACd","file":"main.vue","sourcesContent":["#dv-full-screen-container {\n  position: fixed;\n  top: 0px;\n  left: 0px;\n  overflow: hidden;\n  transform-origin: left top;\n  z-index: 999;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$v = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$v = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$v = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$v = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$v, staticRenderFns: __vue_staticRenderFns__$v },
	    __vue_inject_styles__$v,
	    __vue_script__$v,
	    __vue_scope_id__$v,
	    __vue_is_functional_template__$v,
	    __vue_module_identifier__$v,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function fullScreenContainer (Vue) {
	  Vue.component(__vue_component__$v.name, __vue_component__$v);
	}

	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	var script$u = {
	  name: 'DvLoading'
	};

	/* script */
	const __vue_script__$u = script$u;

	/* template */
	var __vue_render__$u = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { staticClass: "dv-loading" }, [
	    _c("svg", { attrs: { width: "50px", height: "50px" } }, [
	      _c(
	        "circle",
	        {
	          attrs: {
	            cx: "25",
	            cy: "25",
	            r: "20",
	            fill: "transparent",
	            "stroke-width": "3",
	            "stroke-dasharray": "31.415, 31.415",
	            stroke: "#02bcfe",
	            "stroke-linecap": "round"
	          }
	        },
	        [
	          _c("animateTransform", {
	            attrs: {
	              attributeName: "transform",
	              type: "rotate",
	              values: "0, 25 25;360, 25 25",
	              dur: "1.5s",
	              repeatCount: "indefinite"
	            }
	          }),
	          _vm._v(" "),
	          _c("animate", {
	            attrs: {
	              attributeName: "stroke",
	              values: "#02bcfe;#3be6cb;#02bcfe",
	              dur: "3s",
	              repeatCount: "indefinite"
	            }
	          })
	        ],
	        1
	      ),
	      _vm._v(" "),
	      _c(
	        "circle",
	        {
	          attrs: {
	            cx: "25",
	            cy: "25",
	            r: "10",
	            fill: "transparent",
	            "stroke-width": "3",
	            "stroke-dasharray": "15.7, 15.7",
	            stroke: "#3be6cb",
	            "stroke-linecap": "round"
	          }
	        },
	        [
	          _c("animateTransform", {
	            attrs: {
	              attributeName: "transform",
	              type: "rotate",
	              values: "360, 25 25;0, 25 25",
	              dur: "1.5s",
	              repeatCount: "indefinite"
	            }
	          }),
	          _vm._v(" "),
	          _c("animate", {
	            attrs: {
	              attributeName: "stroke",
	              values: "#3be6cb;#02bcfe;#3be6cb",
	              dur: "3s",
	              repeatCount: "indefinite"
	            }
	          })
	        ],
	        1
	      )
	    ]),
	    _vm._v(" "),
	    _c("div", { staticClass: "loading-tip" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$u = [];
	__vue_render__$u._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$u = function (inject) {
	    if (!inject) return
	    inject("data-v-3f2456be_0", { source: ".dv-loading {\n  width: 100%;\n  height: 100%;\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  align-items: center;\n}\n.dv-loading .loading-tip {\n  font-size: 15px;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,WAAW;EACX,YAAY;EACZ,aAAa;EACb,sBAAsB;EACtB,uBAAuB;EACvB,mBAAmB;AACrB;AACA;EACE,eAAe;AACjB","file":"main.vue","sourcesContent":[".dv-loading {\n  width: 100%;\n  height: 100%;\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  align-items: center;\n}\n.dv-loading .loading-tip {\n  font-size: 15px;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$u = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$u = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$u = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$u = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$u, staticRenderFns: __vue_staticRenderFns__$u },
	    __vue_inject_styles__$u,
	    __vue_script__$u,
	    __vue_scope_id__$u,
	    __vue_is_functional_template__$u,
	    __vue_module_identifier__$u,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function loading (Vue) {
	  Vue.component(__vue_component__$u.name, __vue_component__$u);
	}

	//
	var script$t = {
	  name: 'DvBorderBox1',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-1',
	      border: ['left-top', 'right-top', 'left-bottom', 'right-bottom'],
	      defaultColor: ['#4fd2dd', '#235fa7'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$t = script$t;

	/* template */
	var __vue_render__$t = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    { ref: _vm.ref, staticClass: "dv-border-box-1" },
	    [
	      _c(
	        "svg",
	        {
	          staticClass: "border",
	          attrs: { width: _vm.width, height: _vm.height }
	        },
	        [
	          _c("polygon", {
	            attrs: {
	              fill: _vm.backgroundColor,
	              points:
	                "10, 27 10, " +
	                (_vm.height - 27) +
	                " 13, " +
	                (_vm.height - 24) +
	                " 13, " +
	                (_vm.height - 21) +
	                " 24, " +
	                (_vm.height - 11) +
	                "\n    38, " +
	                (_vm.height - 11) +
	                " 41, " +
	                (_vm.height - 8) +
	                " 73, " +
	                (_vm.height - 8) +
	                " 75, " +
	                (_vm.height - 10) +
	                " 81, " +
	                (_vm.height - 10) +
	                "\n    85, " +
	                (_vm.height - 6) +
	                " " +
	                (_vm.width - 85) +
	                ", " +
	                (_vm.height - 6) +
	                " " +
	                (_vm.width - 81) +
	                ", " +
	                (_vm.height - 10) +
	                " " +
	                (_vm.width - 75) +
	                ", " +
	                (_vm.height - 10) +
	                "\n    " +
	                (_vm.width - 73) +
	                ", " +
	                (_vm.height - 8) +
	                " " +
	                (_vm.width - 41) +
	                ", " +
	                (_vm.height - 8) +
	                " " +
	                (_vm.width - 38) +
	                ", " +
	                (_vm.height - 11) +
	                "\n    " +
	                (_vm.width - 24) +
	                ", " +
	                (_vm.height - 11) +
	                " " +
	                (_vm.width - 13) +
	                ", " +
	                (_vm.height - 21) +
	                " " +
	                (_vm.width - 13) +
	                ", " +
	                (_vm.height - 24) +
	                "\n    " +
	                (_vm.width - 10) +
	                ", " +
	                (_vm.height - 27) +
	                " " +
	                (_vm.width - 10) +
	                ", 27 " +
	                (_vm.width - 13) +
	                ", 25 " +
	                (_vm.width - 13) +
	                ", 21\n    " +
	                (_vm.width - 24) +
	                ", 11 " +
	                (_vm.width - 38) +
	                ", 11 " +
	                (_vm.width - 41) +
	                ", 8 " +
	                (_vm.width - 73) +
	                ", 8 " +
	                (_vm.width - 75) +
	                ", 10\n    " +
	                (_vm.width - 81) +
	                ", 10 " +
	                (_vm.width - 85) +
	                ", 6 85, 6 81, 10 75, 10 73, 8 41, 8 38, 11 24, 11 13, 21 13, 24"
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _vm._l(_vm.border, function(item) {
	        return _c(
	          "svg",
	          {
	            key: item,
	            class: item + " border",
	            attrs: { width: "150px", height: "150px" }
	          },
	          [
	            _c(
	              "polygon",
	              {
	                attrs: {
	                  fill: _vm.mergedColor[0],
	                  points:
	                    "6,66 6,18 12,12 18,12 24,6 27,6 30,9 36,9 39,6 84,6 81,9 75,9 73.2,7 40.8,7 37.8,10.2 24,10.2 12,21 12,24 9,27 9,51 7.8,54 7.8,63"
	                }
	              },
	              [
	                _c("animate", {
	                  attrs: {
	                    attributeName: "fill",
	                    values:
	                      _vm.mergedColor[0] +
	                      ";" +
	                      _vm.mergedColor[1] +
	                      ";" +
	                      _vm.mergedColor[0],
	                    dur: "0.5s",
	                    begin: "0s",
	                    repeatCount: "indefinite"
	                  }
	                })
	              ]
	            ),
	            _vm._v(" "),
	            _c(
	              "polygon",
	              {
	                attrs: {
	                  fill: _vm.mergedColor[1],
	                  points:
	                    "27.599999999999998,4.8 38.4,4.8 35.4,7.8 30.599999999999998,7.8"
	                }
	              },
	              [
	                _c("animate", {
	                  attrs: {
	                    attributeName: "fill",
	                    values:
	                      _vm.mergedColor[1] +
	                      ";" +
	                      _vm.mergedColor[0] +
	                      ";" +
	                      _vm.mergedColor[1],
	                    dur: "0.5s",
	                    begin: "0s",
	                    repeatCount: "indefinite"
	                  }
	                })
	              ]
	            ),
	            _vm._v(" "),
	            _c(
	              "polygon",
	              {
	                attrs: {
	                  fill: _vm.mergedColor[0],
	                  points:
	                    "9,54 9,63 7.199999999999999,66 7.199999999999999,75 7.8,78 7.8,110 8.4,110 8.4,66 9.6,66 9.6,54"
	                }
	              },
	              [
	                _c("animate", {
	                  attrs: {
	                    attributeName: "fill",
	                    values:
	                      _vm.mergedColor[0] +
	                      ";" +
	                      _vm.mergedColor[1] +
	                      ";transparent",
	                    dur: "1s",
	                    begin: "0s",
	                    repeatCount: "indefinite"
	                  }
	                })
	              ]
	            )
	          ]
	        )
	      }),
	      _vm._v(" "),
	      _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	    ],
	    2
	  )
	};
	var __vue_staticRenderFns__$t = [];
	__vue_render__$t._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$t = function (inject) {
	    if (!inject) return
	    inject("data-v-2c456604_0", { source: ".dv-border-box-1 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-1 .border {\n  position: absolute;\n  display: block;\n}\n.dv-border-box-1 .right-top {\n  right: 0px;\n  transform: rotateY(180deg);\n}\n.dv-border-box-1 .left-bottom {\n  bottom: 0px;\n  transform: rotateX(180deg);\n}\n.dv-border-box-1 .right-bottom {\n  right: 0px;\n  bottom: 0px;\n  transform: rotateX(180deg) rotateY(180deg);\n}\n.dv-border-box-1 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,cAAc;AAChB;AACA;EACE,UAAU;EACV,0BAA0B;AAC5B;AACA;EACE,WAAW;EACX,0BAA0B;AAC5B;AACA;EACE,UAAU;EACV,WAAW;EACX,0CAA0C;AAC5C;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-1 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-1 .border {\n  position: absolute;\n  display: block;\n}\n.dv-border-box-1 .right-top {\n  right: 0px;\n  transform: rotateY(180deg);\n}\n.dv-border-box-1 .left-bottom {\n  bottom: 0px;\n  transform: rotateX(180deg);\n}\n.dv-border-box-1 .right-bottom {\n  right: 0px;\n  bottom: 0px;\n  transform: rotateX(180deg) rotateY(180deg);\n}\n.dv-border-box-1 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$t = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$t = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$t = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$t = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$t, staticRenderFns: __vue_staticRenderFns__$t },
	    __vue_inject_styles__$t,
	    __vue_script__$t,
	    __vue_scope_id__$t,
	    __vue_is_functional_template__$t,
	    __vue_module_identifier__$t,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox1 (Vue) {
	  Vue.component(__vue_component__$t.name, __vue_component__$t);
	}

	//
	var script$s = {
	  name: 'DvBorderBox2',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-2',
	      defaultColor: ['#fff', 'rgba(255, 255, 255, 0.6)'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$s = script$s;

	/* template */
	var __vue_render__$s = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-2" }, [
	    _c(
	      "svg",
	      {
	        staticClass: "dv-border-svg-container",
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c("polygon", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            points:
	              "\n      7, 7 " +
	              (_vm.width - 7) +
	              ", 7 " +
	              (_vm.width - 7) +
	              ", " +
	              (_vm.height - 7) +
	              " 7, " +
	              (_vm.height - 7) +
	              "\n    "
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points:
	              "2, 2 " +
	              (_vm.width - 2) +
	              " ,2 " +
	              (_vm.width - 2) +
	              ", " +
	              (_vm.height - 2) +
	              " 2, " +
	              (_vm.height - 2) +
	              " 2, 2"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points:
	              "6, 6 " +
	              (_vm.width - 6) +
	              ", 6 " +
	              (_vm.width - 6) +
	              ", " +
	              (_vm.height - 6) +
	              " 6, " +
	              (_vm.height - 6) +
	              " 6, 6"
	          }
	        }),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: { fill: _vm.mergedColor[0], cx: "11", cy: "11", r: "1" }
	        }),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: {
	            fill: _vm.mergedColor[0],
	            cx: _vm.width - 11,
	            cy: "11",
	            r: "1"
	          }
	        }),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: {
	            fill: _vm.mergedColor[0],
	            cx: _vm.width - 11,
	            cy: _vm.height - 11,
	            r: "1"
	          }
	        }),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: {
	            fill: _vm.mergedColor[0],
	            cx: "11",
	            cy: _vm.height - 11,
	            r: "1"
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$s = [];
	__vue_render__$s._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$s = function (inject) {
	    if (!inject) return
	    inject("data-v-1ffd8df7_0", { source: ".dv-border-box-2 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-2 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-2 .dv-border-svg-container > polyline {\n  fill: none;\n  stroke-width: 1;\n}\n.dv-border-box-2 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,QAAQ;EACR,SAAS;AACX;AACA;EACE,UAAU;EACV,eAAe;AACjB;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-2 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-2 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-2 .dv-border-svg-container > polyline {\n  fill: none;\n  stroke-width: 1;\n}\n.dv-border-box-2 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$s = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$s = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$s = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$s = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$s, staticRenderFns: __vue_staticRenderFns__$s },
	    __vue_inject_styles__$s,
	    __vue_script__$s,
	    __vue_scope_id__$s,
	    __vue_is_functional_template__$s,
	    __vue_module_identifier__$s,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox2 (Vue) {
	  Vue.component(__vue_component__$s.name, __vue_component__$s);
	}

	//
	var script$r = {
	  name: 'DvBorderBox3',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-3',
	      defaultColor: ['#2862b7', '#2862b7'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$r = script$r;

	/* template */
	var __vue_render__$r = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-3" }, [
	    _c(
	      "svg",
	      {
	        staticClass: "dv-border-svg-container",
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c("polygon", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            points:
	              "\n      23, 23 " +
	              (_vm.width - 24) +
	              ", 23 " +
	              (_vm.width - 24) +
	              ", " +
	              (_vm.height - 24) +
	              " 23, " +
	              (_vm.height - 24) +
	              "\n    "
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb3-line1",
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points:
	              "4, 4 " +
	              (_vm.width - 22) +
	              " ,4 " +
	              (_vm.width - 22) +
	              ", " +
	              (_vm.height - 22) +
	              " 4, " +
	              (_vm.height - 22) +
	              " 4, 4"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb3-line2",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points:
	              "10, 10 " +
	              (_vm.width - 16) +
	              ", 10 " +
	              (_vm.width - 16) +
	              ", " +
	              (_vm.height - 16) +
	              " 10, " +
	              (_vm.height - 16) +
	              " 10, 10"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb3-line2",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points:
	              "16, 16 " +
	              (_vm.width - 10) +
	              ", 16 " +
	              (_vm.width - 10) +
	              ", " +
	              (_vm.height - 10) +
	              " 16, " +
	              (_vm.height - 10) +
	              " 16, 16"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb3-line2",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points:
	              "22, 22 " +
	              (_vm.width - 4) +
	              ", 22 " +
	              (_vm.width - 4) +
	              ", " +
	              (_vm.height - 4) +
	              " 22, " +
	              (_vm.height - 4) +
	              " 22, 22"
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$r = [];
	__vue_render__$r._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$r = function (inject) {
	    if (!inject) return
	    inject("data-v-02a623fa_0", { source: ".dv-border-box-3 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-3 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-3 .dv-border-svg-container > polyline {\n  fill: none;\n}\n.dv-border-box-3 .dv-bb3-line1 {\n  stroke-width: 3;\n}\n.dv-border-box-3 .dv-bb3-line2 {\n  stroke-width: 1;\n}\n.dv-border-box-3 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,QAAQ;EACR,SAAS;AACX;AACA;EACE,UAAU;AACZ;AACA;EACE,eAAe;AACjB;AACA;EACE,eAAe;AACjB;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-3 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-3 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-3 .dv-border-svg-container > polyline {\n  fill: none;\n}\n.dv-border-box-3 .dv-bb3-line1 {\n  stroke-width: 3;\n}\n.dv-border-box-3 .dv-bb3-line2 {\n  stroke-width: 1;\n}\n.dv-border-box-3 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$r = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$r = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$r = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$r = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$r, staticRenderFns: __vue_staticRenderFns__$r },
	    __vue_inject_styles__$r,
	    __vue_script__$r,
	    __vue_scope_id__$r,
	    __vue_is_functional_template__$r,
	    __vue_module_identifier__$r,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox3 (Vue) {
	  Vue.component(__vue_component__$r.name, __vue_component__$r);
	}

	//
	var script$q = {
	  name: 'DvBorderBox4',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    reverse: {
	      type: Boolean,
	      default: false
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-4',
	      defaultColor: ['red', 'rgba(0,0,255,0.8)'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$q = script$q;

	/* template */
	var __vue_render__$q = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-4" }, [
	    _c(
	      "svg",
	      {
	        class: "dv-border-svg-container " + (_vm.reverse && "dv-reverse"),
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c("polygon", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            points:
	              "\n      " +
	              (_vm.width - 15) +
	              ", 22 170, 22 150, 7 40, 7 28, 21 32, 24\n      16, 42 16, " +
	              (_vm.height - 32) +
	              " 41, " +
	              (_vm.height - 7) +
	              " " +
	              (_vm.width - 15) +
	              ", " +
	              (_vm.height - 7) +
	              "\n    "
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-1",
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points:
	              "145, " +
	              (_vm.height - 5) +
	              " 40, " +
	              (_vm.height - 5) +
	              " 10, " +
	              (_vm.height - 35) +
	              "\n        10, 40 40, 5 150, 5 170, 20 " +
	              (_vm.width - 15) +
	              ", 20"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-2",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points:
	              "245, " +
	              (_vm.height - 1) +
	              " 36, " +
	              (_vm.height - 1) +
	              " 14, " +
	              (_vm.height - 23) +
	              "\n        14, " +
	              (_vm.height - 100)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-3",
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points: "7, " + (_vm.height - 40) + " 7, " + (_vm.height - 75)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-4",
	          attrs: { stroke: _vm.mergedColor[0], points: "28, 24 13, 41 13, 64" }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-5",
	          attrs: { stroke: _vm.mergedColor[0], points: "5, 45 5, 140" }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-6",
	          attrs: { stroke: _vm.mergedColor[1], points: "14, 75 14, 180" }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-7",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points: "55, 11 147, 11 167, 26 250, 26"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-8",
	          attrs: { stroke: _vm.mergedColor[1], points: "158, 5 173, 16" }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-9",
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points: "200, 17 " + (_vm.width - 10) + ", 17"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb4-line-10",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points: "385, 17 " + (_vm.width - 10) + ", 17"
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$q = [];
	__vue_render__$q._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$q = function (inject) {
	    if (!inject) return
	    inject("data-v-59a381f2_0", { source: ".dv-border-box-4 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-4 .dv-reverse {\n  transform: rotate(180deg);\n}\n.dv-border-box-4 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-4 .dv-border-svg-container > polyline {\n  fill: none;\n}\n.dv-border-box-4 .sw1 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .sw3 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n}\n.dv-border-box-4 .dv-bb4-line-1 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-2 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-3 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n}\n.dv-border-box-4 .dv-bb4-line-4 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n}\n.dv-border-box-4 .dv-bb4-line-5 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-6 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-7 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-8 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n}\n.dv-border-box-4 .dv-bb4-line-9 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n  stroke-dasharray: 100 250;\n}\n.dv-border-box-4 .dv-bb4-line-10 {\n  stroke-width: 1;\n  stroke-dasharray: 80 270;\n}\n.dv-border-box-4 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,yBAAyB;AAC3B;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,QAAQ;EACR,SAAS;AACX;AACA;EACE,UAAU;AACZ;AACA;EACE,eAAe;AACjB;AACA;EACE,iBAAiB;EACjB,qBAAqB;AACvB;AACA;EACE,eAAe;AACjB;AACA;EACE,eAAe;AACjB;AACA;EACE,iBAAiB;EACjB,qBAAqB;AACvB;AACA;EACE,iBAAiB;EACjB,qBAAqB;AACvB;AACA;EACE,eAAe;AACjB;AACA;EACE,eAAe;AACjB;AACA;EACE,eAAe;AACjB;AACA;EACE,iBAAiB;EACjB,qBAAqB;AACvB;AACA;EACE,iBAAiB;EACjB,qBAAqB;EACrB,yBAAyB;AAC3B;AACA;EACE,eAAe;EACf,wBAAwB;AAC1B;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-4 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-4 .dv-reverse {\n  transform: rotate(180deg);\n}\n.dv-border-box-4 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-4 .dv-border-svg-container > polyline {\n  fill: none;\n}\n.dv-border-box-4 .sw1 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .sw3 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n}\n.dv-border-box-4 .dv-bb4-line-1 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-2 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-3 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n}\n.dv-border-box-4 .dv-bb4-line-4 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n}\n.dv-border-box-4 .dv-bb4-line-5 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-6 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-7 {\n  stroke-width: 1;\n}\n.dv-border-box-4 .dv-bb4-line-8 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n}\n.dv-border-box-4 .dv-bb4-line-9 {\n  stroke-width: 3px;\n  stroke-linecap: round;\n  stroke-dasharray: 100 250;\n}\n.dv-border-box-4 .dv-bb4-line-10 {\n  stroke-width: 1;\n  stroke-dasharray: 80 270;\n}\n.dv-border-box-4 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$q = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$q = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$q = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$q = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$q, staticRenderFns: __vue_staticRenderFns__$q },
	    __vue_inject_styles__$q,
	    __vue_script__$q,
	    __vue_scope_id__$q,
	    __vue_is_functional_template__$q,
	    __vue_module_identifier__$q,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox4 (Vue) {
	  Vue.component(__vue_component__$q.name, __vue_component__$q);
	}

	//
	var script$p = {
	  name: 'DvBorderBox5',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    reverse: {
	      type: Boolean,
	      default: false
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-5',
	      defaultColor: ['rgba(255, 255, 255, 0.35)', 'rgba(255, 255, 255, 0.20)'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$p = script$p;

	/* template */
	var __vue_render__$p = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-5" }, [
	    _c(
	      "svg",
	      {
	        class: "dv-border-svg-container  " + (_vm.reverse && "dv-reverse"),
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c("polygon", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            points:
	              "\n      10, 22 " +
	              (_vm.width - 22) +
	              ", 22 " +
	              (_vm.width - 22) +
	              ", " +
	              (_vm.height - 86) +
	              " " +
	              (_vm.width - 84) +
	              ", " +
	              (_vm.height - 24) +
	              " 10, " +
	              (_vm.height - 24) +
	              "\n    "
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb5-line-1",
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points:
	              "8, 5 " +
	              (_vm.width - 5) +
	              ", 5 " +
	              (_vm.width - 5) +
	              ", " +
	              (_vm.height - 100) +
	              "\n        " +
	              (_vm.width - 100) +
	              ", " +
	              (_vm.height - 5) +
	              " 8, " +
	              (_vm.height - 5) +
	              " 8, 5"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb5-line-2",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points:
	              "3, 5 " +
	              (_vm.width - 20) +
	              ", 5 " +
	              (_vm.width - 20) +
	              ", " +
	              (_vm.height - 60) +
	              "\n        " +
	              (_vm.width - 74) +
	              ", " +
	              (_vm.height - 5) +
	              " 3, " +
	              (_vm.height - 5) +
	              " 3, 5"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb5-line-3",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points: "50, 13 " + (_vm.width - 35) + ", 13"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb5-line-4",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points: "15, 20 " + (_vm.width - 35) + ", 20"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb5-line-5",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points:
	              "15, " +
	              (_vm.height - 20) +
	              " " +
	              (_vm.width - 110) +
	              ", " +
	              (_vm.height - 20)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          staticClass: "dv-bb5-line-6",
	          attrs: {
	            stroke: _vm.mergedColor[1],
	            points:
	              "15, " +
	              (_vm.height - 13) +
	              " " +
	              (_vm.width - 110) +
	              ", " +
	              (_vm.height - 13)
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$p = [];
	__vue_render__$p._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$p = function (inject) {
	    if (!inject) return
	    inject("data-v-9b8fe3be_0", { source: ".dv-border-box-5 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-5 .dv-reverse {\n  transform: rotate(180deg);\n}\n.dv-border-box-5 .dv-border-svg-container {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-5 .dv-border-svg-container > polyline {\n  fill: none;\n}\n.dv-border-box-5 .dv-bb5-line-1,\n.dv-border-box-5 .dv-bb5-line-2 {\n  stroke-width: 1;\n}\n.dv-border-box-5 .dv-bb5-line-3,\n.dv-border-box-5 .dv-bb5-line-6 {\n  stroke-width: 5;\n}\n.dv-border-box-5 .dv-bb5-line-4,\n.dv-border-box-5 .dv-bb5-line-5 {\n  stroke-width: 2;\n}\n.dv-border-box-5 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,yBAAyB;AAC3B;AACA;EACE,kBAAkB;EAClB,QAAQ;EACR,SAAS;EACT,WAAW;EACX,YAAY;AACd;AACA;EACE,UAAU;AACZ;AACA;;EAEE,eAAe;AACjB;AACA;;EAEE,eAAe;AACjB;AACA;;EAEE,eAAe;AACjB;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-5 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-5 .dv-reverse {\n  transform: rotate(180deg);\n}\n.dv-border-box-5 .dv-border-svg-container {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-5 .dv-border-svg-container > polyline {\n  fill: none;\n}\n.dv-border-box-5 .dv-bb5-line-1,\n.dv-border-box-5 .dv-bb5-line-2 {\n  stroke-width: 1;\n}\n.dv-border-box-5 .dv-bb5-line-3,\n.dv-border-box-5 .dv-bb5-line-6 {\n  stroke-width: 5;\n}\n.dv-border-box-5 .dv-bb5-line-4,\n.dv-border-box-5 .dv-bb5-line-5 {\n  stroke-width: 2;\n}\n.dv-border-box-5 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$p = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$p = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$p = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$p = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$p, staticRenderFns: __vue_staticRenderFns__$p },
	    __vue_inject_styles__$p,
	    __vue_script__$p,
	    __vue_scope_id__$p,
	    __vue_is_functional_template__$p,
	    __vue_module_identifier__$p,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox5 (Vue) {
	  Vue.component(__vue_component__$p.name, __vue_component__$p);
	}

	//
	var script$o = {
	  name: 'DvBorderBox6',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-6',
	      defaultColor: ['rgba(255, 255, 255, 0.35)', 'gray'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$o = script$o;

	/* template */
	var __vue_render__$o = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-6" }, [
	    _c(
	      "svg",
	      {
	        staticClass: "dv-border-svg-container",
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c("polygon", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            points:
	              "\n      9, 7 " +
	              (_vm.width - 9) +
	              ", 7 " +
	              (_vm.width - 9) +
	              ", " +
	              (_vm.height - 7) +
	              " 9, " +
	              (_vm.height - 7) +
	              "\n    "
	          }
	        }),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: { fill: _vm.mergedColor[1], cx: "5", cy: "5", r: "2" }
	        }),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: {
	            fill: _vm.mergedColor[1],
	            cx: _vm.width - 5,
	            cy: "5",
	            r: "2"
	          }
	        }),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: {
	            fill: _vm.mergedColor[1],
	            cx: _vm.width - 5,
	            cy: _vm.height - 5,
	            r: "2"
	          }
	        }),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: {
	            fill: _vm.mergedColor[1],
	            cx: "5",
	            cy: _vm.height - 5,
	            r: "2"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points: "10, 4 " + (_vm.width - 10) + ", 4"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points:
	              "10, " +
	              (_vm.height - 4) +
	              " " +
	              (_vm.width - 10) +
	              ", " +
	              (_vm.height - 4)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points: "5, 70 5, " + (_vm.height - 70)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points:
	              _vm.width -
	              5 +
	              ", 70 " +
	              (_vm.width - 5) +
	              ", " +
	              (_vm.height - 70)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: { stroke: _vm.mergedColor[0], points: "3, 10, 3, 50" }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: { stroke: _vm.mergedColor[0], points: "7, 30 7, 80" }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points: _vm.width - 3 + ", 10 " + (_vm.width - 3) + ", 50"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points: _vm.width - 7 + ", 30 " + (_vm.width - 7) + ", 80"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points: "3, " + (_vm.height - 10) + " 3, " + (_vm.height - 50)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points: "7, " + (_vm.height - 30) + " 7, " + (_vm.height - 80)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points:
	              _vm.width -
	              3 +
	              ", " +
	              (_vm.height - 10) +
	              " " +
	              (_vm.width - 3) +
	              ", " +
	              (_vm.height - 50)
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            points:
	              _vm.width -
	              7 +
	              ", " +
	              (_vm.height - 30) +
	              " " +
	              (_vm.width - 7) +
	              ", " +
	              (_vm.height - 80)
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$o = [];
	__vue_render__$o._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$o = function (inject) {
	    if (!inject) return
	    inject("data-v-2219093b_0", { source: ".dv-border-box-6 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-6 .dv-border-svg-container {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-6 .dv-border-svg-container > polyline {\n  fill: none;\n  stroke-width: 1;\n}\n.dv-border-box-6 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,QAAQ;EACR,SAAS;EACT,WAAW;EACX,YAAY;AACd;AACA;EACE,UAAU;EACV,eAAe;AACjB;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-6 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-6 .dv-border-svg-container {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-6 .dv-border-svg-container > polyline {\n  fill: none;\n  stroke-width: 1;\n}\n.dv-border-box-6 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$o = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$o = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$o = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$o = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$o, staticRenderFns: __vue_staticRenderFns__$o },
	    __vue_inject_styles__$o,
	    __vue_script__$o,
	    __vue_scope_id__$o,
	    __vue_is_functional_template__$o,
	    __vue_module_identifier__$o,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox6 (Vue) {
	  Vue.component(__vue_component__$o.name, __vue_component__$o);
	}

	//
	var script$n = {
	  name: 'DvBorderBox7',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-7',
	      defaultColor: ['rgba(128,128,128,0.3)', 'rgba(128,128,128,0.5)'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$n = script$n;

	/* template */
	var __vue_render__$n = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    {
	      ref: _vm.ref,
	      staticClass: "dv-border-box-7",
	      style:
	        "box-shadow: inset 0 0 40px " +
	        _vm.mergedColor[0] +
	        "; border: 1px solid " +
	        _vm.mergedColor[0] +
	        "; background-color: " +
	        _vm.backgroundColor
	    },
	    [
	      _c(
	        "svg",
	        {
	          staticClass: "dv-border-svg-container",
	          attrs: { width: _vm.width, height: _vm.height }
	        },
	        [
	          _c("polyline", {
	            staticClass: "dv-bb7-line-width-2",
	            attrs: { stroke: _vm.mergedColor[0], points: "0, 25 0, 0 25, 0" }
	          }),
	          _vm._v(" "),
	          _c("polyline", {
	            staticClass: "dv-bb7-line-width-2",
	            attrs: {
	              stroke: _vm.mergedColor[0],
	              points:
	                _vm.width -
	                25 +
	                ", 0 " +
	                _vm.width +
	                ", 0 " +
	                _vm.width +
	                ", 25"
	            }
	          }),
	          _vm._v(" "),
	          _c("polyline", {
	            staticClass: "dv-bb7-line-width-2",
	            attrs: {
	              stroke: _vm.mergedColor[0],
	              points:
	                _vm.width -
	                25 +
	                ", " +
	                _vm.height +
	                " " +
	                _vm.width +
	                ", " +
	                _vm.height +
	                " " +
	                _vm.width +
	                ", " +
	                (_vm.height - 25)
	            }
	          }),
	          _vm._v(" "),
	          _c("polyline", {
	            staticClass: "dv-bb7-line-width-2",
	            attrs: {
	              stroke: _vm.mergedColor[0],
	              points:
	                "0, " +
	                (_vm.height - 25) +
	                " 0, " +
	                _vm.height +
	                " 25, " +
	                _vm.height
	            }
	          }),
	          _vm._v(" "),
	          _c("polyline", {
	            staticClass: "dv-bb7-line-width-5",
	            attrs: { stroke: _vm.mergedColor[1], points: "0, 10 0, 0 10, 0" }
	          }),
	          _vm._v(" "),
	          _c("polyline", {
	            staticClass: "dv-bb7-line-width-5",
	            attrs: {
	              stroke: _vm.mergedColor[1],
	              points:
	                _vm.width -
	                10 +
	                ", 0 " +
	                _vm.width +
	                ", 0 " +
	                _vm.width +
	                ", 10"
	            }
	          }),
	          _vm._v(" "),
	          _c("polyline", {
	            staticClass: "dv-bb7-line-width-5",
	            attrs: {
	              stroke: _vm.mergedColor[1],
	              points:
	                _vm.width -
	                10 +
	                ", " +
	                _vm.height +
	                " " +
	                _vm.width +
	                ", " +
	                _vm.height +
	                " " +
	                _vm.width +
	                ", " +
	                (_vm.height - 10)
	            }
	          }),
	          _vm._v(" "),
	          _c("polyline", {
	            staticClass: "dv-bb7-line-width-5",
	            attrs: {
	              stroke: _vm.mergedColor[1],
	              points:
	                "0, " +
	                (_vm.height - 10) +
	                " 0, " +
	                _vm.height +
	                " 10, " +
	                _vm.height
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	    ]
	  )
	};
	var __vue_staticRenderFns__$n = [];
	__vue_render__$n._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$n = function (inject) {
	    if (!inject) return
	    inject("data-v-d30c5810_0", { source: ".dv-border-box-7 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-7 .dv-border-svg-container {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-7 .dv-border-svg-container > polyline {\n  fill: none;\n  stroke-linecap: round;\n}\n.dv-border-box-7 .dv-bb7-line-width-2 {\n  stroke-width: 2;\n}\n.dv-border-box-7 .dv-bb7-line-width-5 {\n  stroke-width: 5;\n}\n.dv-border-box-7 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,QAAQ;EACR,SAAS;EACT,WAAW;EACX,YAAY;AACd;AACA;EACE,UAAU;EACV,qBAAqB;AACvB;AACA;EACE,eAAe;AACjB;AACA;EACE,eAAe;AACjB;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-7 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-7 .dv-border-svg-container {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-7 .dv-border-svg-container > polyline {\n  fill: none;\n  stroke-linecap: round;\n}\n.dv-border-box-7 .dv-bb7-line-width-2 {\n  stroke-width: 2;\n}\n.dv-border-box-7 .dv-bb7-line-width-5 {\n  stroke-width: 5;\n}\n.dv-border-box-7 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$n = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$n = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$n = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$n = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$n, staticRenderFns: __vue_staticRenderFns__$n },
	    __vue_inject_styles__$n,
	    __vue_script__$n,
	    __vue_scope_id__$n,
	    __vue_is_functional_template__$n,
	    __vue_module_identifier__$n,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox7 (Vue) {
	  Vue.component(__vue_component__$n.name, __vue_component__$n);
	}

	//
	var script$m = {
	  name: 'DvBorderBox8',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    dur: {
	      type: Number,
	      default: 3
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    },
	    reverse: {
	      type: Boolean,
	      default: false
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'border-box-8',
	      path: `border-box-8-path-${id}`,
	      gradient: `border-box-8-gradient-${id}`,
	      mask: `border-box-8-mask-${id}`,
	      defaultColor: ['#235fa7', '#4fd2dd'],
	      mergedColor: []
	    };
	  },

	  computed: {
	    length() {
	      const {
	        width,
	        height
	      } = this;
	      return (width + height - 5) * 2;
	    },

	    pathD() {
	      const {
	        reverse,
	        width,
	        height
	      } = this;
	      if (reverse) return `M 2.5, 2.5 L 2.5, ${height - 2.5} L ${width - 2.5}, ${height - 2.5} L ${width - 2.5}, 2.5 L 2.5, 2.5`;
	      return `M2.5, 2.5 L${width - 2.5}, 2.5 L${width - 2.5}, ${height - 2.5} L2.5, ${height - 2.5} L2.5, 2.5`;
	    }

	  },
	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$m = script$m;

	/* template */
	var __vue_render__$m = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-8" }, [
	    _c(
	      "svg",
	      {
	        staticClass: "dv-border-svg-container",
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c(
	          "defs",
	          [
	            _c("path", {
	              attrs: { id: _vm.path, d: _vm.pathD, fill: "transparent" }
	            }),
	            _vm._v(" "),
	            _c(
	              "radialGradient",
	              { attrs: { id: _vm.gradient, cx: "50%", cy: "50%", r: "50%" } },
	              [
	                _c("stop", {
	                  attrs: {
	                    offset: "0%",
	                    "stop-color": "#fff",
	                    "stop-opacity": "1"
	                  }
	                }),
	                _vm._v(" "),
	                _c("stop", {
	                  attrs: {
	                    offset: "100%",
	                    "stop-color": "#fff",
	                    "stop-opacity": "0"
	                  }
	                })
	              ],
	              1
	            ),
	            _vm._v(" "),
	            _c("mask", { attrs: { id: _vm.mask } }, [
	              _c(
	                "circle",
	                {
	                  attrs: {
	                    cx: "0",
	                    cy: "0",
	                    r: "150",
	                    fill: "url(#" + _vm.gradient + ")"
	                  }
	                },
	                [
	                  _c("animateMotion", {
	                    attrs: {
	                      dur: _vm.dur + "s",
	                      path: _vm.pathD,
	                      rotate: "auto",
	                      repeatCount: "indefinite"
	                    }
	                  })
	                ],
	                1
	              )
	            ])
	          ],
	          1
	        ),
	        _vm._v(" "),
	        _c("polygon", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            points:
	              "5, 5 " +
	              (_vm.width - 5) +
	              ", 5 " +
	              (_vm.width - 5) +
	              " " +
	              (_vm.height - 5) +
	              " 5, " +
	              (_vm.height - 5)
	          }
	        }),
	        _vm._v(" "),
	        _c("use", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            "stroke-width": "1",
	            "xlink:href": "#" + _vm.path
	          }
	        }),
	        _vm._v(" "),
	        _c(
	          "use",
	          {
	            attrs: {
	              stroke: _vm.mergedColor[1],
	              "stroke-width": "3",
	              "xlink:href": "#" + _vm.path,
	              mask: "url(#" + _vm.mask + ")"
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: "stroke-dasharray",
	                from: "0, " + _vm.length,
	                to: _vm.length + ", 0",
	                dur: _vm.dur + "s",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        )
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$m = [];
	__vue_render__$m._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$m = function (inject) {
	    if (!inject) return
	    inject("data-v-182b13d4_0", { source: ".dv-border-box-8 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-8 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  left: 0px;\n  top: 0px;\n}\n.dv-border-box-8 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,SAAS;EACT,QAAQ;AACV;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-8 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-8 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  left: 0px;\n  top: 0px;\n}\n.dv-border-box-8 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$m = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$m = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$m = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$m = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$m, staticRenderFns: __vue_staticRenderFns__$m },
	    __vue_inject_styles__$m,
	    __vue_script__$m,
	    __vue_scope_id__$m,
	    __vue_is_functional_template__$m,
	    __vue_module_identifier__$m,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox8 (Vue) {
	  Vue.component(__vue_component__$m.name, __vue_component__$m);
	}

	//
	var script$l = {
	  name: 'DvBorderBox9',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'border-box-9',
	      gradientId: `border-box-9-gradient-${id}`,
	      maskId: `border-box-9-mask-${id}`,
	      defaultColor: ['#11eefd', '#0078d2'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$l = script$l;

	/* template */
	var __vue_render__$l = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-9" }, [
	    _c(
	      "svg",
	      {
	        staticClass: "dv-border-svg-container",
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c(
	          "defs",
	          [
	            _c(
	              "linearGradient",
	              {
	                attrs: {
	                  id: _vm.gradientId,
	                  x1: "0%",
	                  y1: "0%",
	                  x2: "100%",
	                  y2: "100%"
	                }
	              },
	              [
	                _c("animate", {
	                  attrs: {
	                    attributeName: "x1",
	                    values: "0%;100%;0%",
	                    dur: "10s",
	                    begin: "0s",
	                    repeatCount: "indefinite"
	                  }
	                }),
	                _vm._v(" "),
	                _c("animate", {
	                  attrs: {
	                    attributeName: "x2",
	                    values: "100%;0%;100%",
	                    dur: "10s",
	                    begin: "0s",
	                    repeatCount: "indefinite"
	                  }
	                }),
	                _vm._v(" "),
	                _c(
	                  "stop",
	                  { attrs: { offset: "0%", "stop-color": _vm.mergedColor[0] } },
	                  [
	                    _c("animate", {
	                      attrs: {
	                        attributeName: "stop-color",
	                        values:
	                          _vm.mergedColor[0] +
	                          ";" +
	                          _vm.mergedColor[1] +
	                          ";" +
	                          _vm.mergedColor[0],
	                        dur: "10s",
	                        begin: "0s",
	                        repeatCount: "indefinite"
	                      }
	                    })
	                  ]
	                ),
	                _vm._v(" "),
	                _c(
	                  "stop",
	                  {
	                    attrs: { offset: "100%", "stop-color": _vm.mergedColor[1] }
	                  },
	                  [
	                    _c("animate", {
	                      attrs: {
	                        attributeName: "stop-color",
	                        values:
	                          _vm.mergedColor[1] +
	                          ";" +
	                          _vm.mergedColor[0] +
	                          ";" +
	                          _vm.mergedColor[1],
	                        dur: "10s",
	                        begin: "0s",
	                        repeatCount: "indefinite"
	                      }
	                    })
	                  ]
	                )
	              ],
	              1
	            ),
	            _vm._v(" "),
	            _c("mask", { attrs: { id: _vm.maskId } }, [
	              _c("polyline", {
	                attrs: {
	                  stroke: "#fff",
	                  "stroke-width": "3",
	                  fill: "transparent",
	                  points:
	                    "8, " +
	                    _vm.height * 0.4 +
	                    " 8, 3, " +
	                    (_vm.width * 0.4 + 7) +
	                    ", 3"
	                }
	              }),
	              _vm._v(" "),
	              _c("polyline", {
	                attrs: {
	                  fill: "#fff",
	                  points:
	                    "8, " +
	                    _vm.height * 0.15 +
	                    " 8, 3, " +
	                    (_vm.width * 0.1 + 7) +
	                    ", 3\n            " +
	                    _vm.width * 0.1 +
	                    ", 8 14, 8 14, " +
	                    (_vm.height * 0.15 - 7) +
	                    "\n          "
	                }
	              }),
	              _vm._v(" "),
	              _c("polyline", {
	                attrs: {
	                  stroke: "#fff",
	                  "stroke-width": "3",
	                  fill: "transparent",
	                  points:
	                    _vm.width * 0.5 +
	                    ", 3 " +
	                    (_vm.width - 3) +
	                    ", 3, " +
	                    (_vm.width - 3) +
	                    ", " +
	                    _vm.height * 0.25
	                }
	              }),
	              _vm._v(" "),
	              _c("polyline", {
	                attrs: {
	                  fill: "#fff",
	                  points:
	                    "\n            " +
	                    _vm.width * 0.52 +
	                    ", 3 " +
	                    _vm.width * 0.58 +
	                    ", 3\n            " +
	                    (_vm.width * 0.58 - 7) +
	                    ", 9 " +
	                    (_vm.width * 0.52 + 7) +
	                    ", 9\n          "
	                }
	              }),
	              _vm._v(" "),
	              _c("polyline", {
	                attrs: {
	                  fill: "#fff",
	                  points:
	                    "\n            " +
	                    _vm.width * 0.9 +
	                    ", 3 " +
	                    (_vm.width - 3) +
	                    ", 3 " +
	                    (_vm.width - 3) +
	                    ", " +
	                    _vm.height * 0.1 +
	                    "\n            " +
	                    (_vm.width - 9) +
	                    ", " +
	                    (_vm.height * 0.1 - 7) +
	                    " " +
	                    (_vm.width - 9) +
	                    ", 9 " +
	                    (_vm.width * 0.9 + 7) +
	                    ", 9\n          "
	                }
	              }),
	              _vm._v(" "),
	              _c("polyline", {
	                attrs: {
	                  stroke: "#fff",
	                  "stroke-width": "3",
	                  fill: "transparent",
	                  points:
	                    "8, " +
	                    _vm.height * 0.5 +
	                    " 8, " +
	                    (_vm.height - 3) +
	                    " " +
	                    (_vm.width * 0.3 + 7) +
	                    ", " +
	                    (_vm.height - 3)
	                }
	              }),
	              _vm._v(" "),
	              _c("polyline", {
	                attrs: {
	                  fill: "#fff",
	                  points:
	                    "\n            8, " +
	                    _vm.height * 0.55 +
	                    " 8, " +
	                    _vm.height * 0.7 +
	                    "\n            2, " +
	                    (_vm.height * 0.7 - 7) +
	                    " 2, " +
	                    (_vm.height * 0.55 + 7) +
	                    "\n          "
	                }
	              }),
	              _vm._v(" "),
	              _c("polyline", {
	                attrs: {
	                  stroke: "#fff",
	                  "stroke-width": "3",
	                  fill: "transparent",
	                  points:
	                    _vm.width * 0.35 +
	                    ", " +
	                    (_vm.height - 3) +
	                    " " +
	                    (_vm.width - 3) +
	                    ", " +
	                    (_vm.height - 3) +
	                    " " +
	                    (_vm.width - 3) +
	                    ", " +
	                    _vm.height * 0.35
	                }
	              }),
	              _vm._v(" "),
	              _c("polyline", {
	                attrs: {
	                  fill: "#fff",
	                  points:
	                    "\n            " +
	                    _vm.width * 0.92 +
	                    ", " +
	                    (_vm.height - 3) +
	                    " " +
	                    (_vm.width - 3) +
	                    ", " +
	                    (_vm.height - 3) +
	                    " " +
	                    (_vm.width - 3) +
	                    ", " +
	                    _vm.height * 0.8 +
	                    "\n            " +
	                    (_vm.width - 9) +
	                    ", " +
	                    (_vm.height * 0.8 + 7) +
	                    " " +
	                    (_vm.width - 9) +
	                    ", " +
	                    (_vm.height - 9) +
	                    " " +
	                    (_vm.width * 0.92 + 7) +
	                    ", " +
	                    (_vm.height - 9) +
	                    "\n          "
	                }
	              })
	            ])
	          ],
	          1
	        ),
	        _vm._v(" "),
	        _c("polygon", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            points:
	              "\n      15, 9 " +
	              (_vm.width * 0.1 + 1) +
	              ", 9 " +
	              (_vm.width * 0.1 + 4) +
	              ", 6 " +
	              (_vm.width * 0.52 + 2) +
	              ", 6\n      " +
	              (_vm.width * 0.52 + 6) +
	              ", 10 " +
	              (_vm.width * 0.58 - 7) +
	              ", 10 " +
	              (_vm.width * 0.58 - 2) +
	              ", 6\n      " +
	              (_vm.width * 0.9 + 2) +
	              ", 6 " +
	              (_vm.width * 0.9 + 6) +
	              ", 10 " +
	              (_vm.width - 10) +
	              ", 10 " +
	              (_vm.width - 10) +
	              ", " +
	              (_vm.height * 0.1 - 6) +
	              "\n      " +
	              (_vm.width - 6) +
	              ", " +
	              (_vm.height * 0.1 - 1) +
	              " " +
	              (_vm.width - 6) +
	              ", " +
	              (_vm.height * 0.8 + 1) +
	              " " +
	              (_vm.width - 10) +
	              ", " +
	              (_vm.height * 0.8 + 6) +
	              "\n      " +
	              (_vm.width - 10) +
	              ", " +
	              (_vm.height - 10) +
	              " " +
	              (_vm.width * 0.92 + 7) +
	              ", " +
	              (_vm.height - 10) +
	              "  " +
	              (_vm.width * 0.92 + 2) +
	              ", " +
	              (_vm.height - 6) +
	              "\n      11, " +
	              (_vm.height - 6) +
	              " 11, " +
	              (_vm.height * 0.15 - 2) +
	              " 15, " +
	              (_vm.height * 0.15 - 7) +
	              "\n    "
	          }
	        }),
	        _vm._v(" "),
	        _c("rect", {
	          attrs: {
	            x: "0",
	            y: "0",
	            width: _vm.width,
	            height: _vm.height,
	            fill: "url(#" + _vm.gradientId + ")",
	            mask: "url(#" + _vm.maskId + ")"
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$l = [];
	__vue_render__$l._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$l = function (inject) {
	    if (!inject) return
	    inject("data-v-4d3ffff3_0", { source: ".dv-border-box-9 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-9 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  left: 0px;\n  top: 0px;\n}\n.dv-border-box-9 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,SAAS;EACT,QAAQ;AACV;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-9 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-9 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  left: 0px;\n  top: 0px;\n}\n.dv-border-box-9 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$l = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$l = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$l = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$l = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$l, staticRenderFns: __vue_staticRenderFns__$l },
	    __vue_inject_styles__$l,
	    __vue_script__$l,
	    __vue_scope_id__$l,
	    __vue_is_functional_template__$l,
	    __vue_module_identifier__$l,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox9 (Vue) {
	  Vue.component(__vue_component__$l.name, __vue_component__$l);
	}

	//
	var script$k = {
	  name: 'DvBorderBox10',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-10',
	      border: ['left-top', 'right-top', 'left-bottom', 'right-bottom'],
	      defaultColor: ['#1d48c4', '#d3e1f8'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$k = script$k;

	/* template */
	var __vue_render__$k = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    {
	      ref: _vm.ref,
	      staticClass: "dv-border-box-10",
	      style: "box-shadow: inset 0 0 25px 3px " + _vm.mergedColor[0]
	    },
	    [
	      _c(
	        "svg",
	        {
	          staticClass: "dv-border-svg-container",
	          attrs: { width: _vm.width, height: _vm.height }
	        },
	        [
	          _c("polygon", {
	            attrs: {
	              fill: _vm.backgroundColor,
	              points:
	                "\n      4, 0 " +
	                (_vm.width - 4) +
	                ", 0 " +
	                _vm.width +
	                ", 4 " +
	                _vm.width +
	                ", " +
	                (_vm.height - 4) +
	                " " +
	                (_vm.width - 4) +
	                ", " +
	                _vm.height +
	                "\n      4, " +
	                _vm.height +
	                " 0, " +
	                (_vm.height - 4) +
	                " 0, 4\n    "
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _vm._l(_vm.border, function(item) {
	        return _c(
	          "svg",
	          {
	            key: item,
	            class: item + " dv-border-svg-container",
	            attrs: { width: "150px", height: "150px" }
	          },
	          [
	            _c("polygon", {
	              attrs: {
	                fill: _vm.mergedColor[1],
	                points: "40, 0 5, 0 0, 5 0, 16 3, 19 3, 7 7, 3 35, 3"
	              }
	            })
	          ]
	        )
	      }),
	      _vm._v(" "),
	      _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	    ],
	    2
	  )
	};
	var __vue_staticRenderFns__$k = [];
	__vue_render__$k._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$k = function (inject) {
	    if (!inject) return
	    inject("data-v-1adea5f4_0", { source: ".dv-border-box-10 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  border-radius: 6px;\n}\n.dv-border-box-10 .dv-border-svg-container {\n  position: absolute;\n  display: block;\n}\n.dv-border-box-10 .right-top {\n  right: 0px;\n  transform: rotateY(180deg);\n}\n.dv-border-box-10 .left-bottom {\n  bottom: 0px;\n  transform: rotateX(180deg);\n}\n.dv-border-box-10 .right-bottom {\n  right: 0px;\n  bottom: 0px;\n  transform: rotateX(180deg) rotateY(180deg);\n}\n.dv-border-box-10 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,kBAAkB;AACpB;AACA;EACE,kBAAkB;EAClB,cAAc;AAChB;AACA;EACE,UAAU;EACV,0BAA0B;AAC5B;AACA;EACE,WAAW;EACX,0BAA0B;AAC5B;AACA;EACE,UAAU;EACV,WAAW;EACX,0CAA0C;AAC5C;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-10 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  border-radius: 6px;\n}\n.dv-border-box-10 .dv-border-svg-container {\n  position: absolute;\n  display: block;\n}\n.dv-border-box-10 .right-top {\n  right: 0px;\n  transform: rotateY(180deg);\n}\n.dv-border-box-10 .left-bottom {\n  bottom: 0px;\n  transform: rotateX(180deg);\n}\n.dv-border-box-10 .right-bottom {\n  right: 0px;\n  bottom: 0px;\n  transform: rotateX(180deg) rotateY(180deg);\n}\n.dv-border-box-10 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$k = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$k = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$k = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$k = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$k, staticRenderFns: __vue_staticRenderFns__$k },
	    __vue_inject_styles__$k,
	    __vue_script__$k,
	    __vue_scope_id__$k,
	    __vue_is_functional_template__$k,
	    __vue_module_identifier__$k,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox10 (Vue) {
	  Vue.component(__vue_component__$k.name, __vue_component__$k);
	}

	_interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

	var _keywords = new Map([['transparent', 'rgba(0,0,0,0)'], ['black', '#000000'], ['silver', '#C0C0C0'], ['gray', '#808080'], ['white', '#FFFFFF'], ['maroon', '#800000'], ['red', '#FF0000'], ['purple', '#800080'], ['fuchsia', '#FF00FF'], ['green', '#008000'], ['lime', '#00FF00'], ['olive', '#808000'], ['yellow', '#FFFF00'], ['navy', '#000080'], ['blue', '#0000FF'], ['teal', '#008080'], ['aqua', '#00FFFF'], ['aliceblue', '#f0f8ff'], ['antiquewhite', '#faebd7'], ['aquamarine', '#7fffd4'], ['azure', '#f0ffff'], ['beige', '#f5f5dc'], ['bisque', '#ffe4c4'], ['blanchedalmond', '#ffebcd'], ['blueviolet', '#8a2be2'], ['brown', '#a52a2a'], ['burlywood', '#deb887'], ['cadetblue', '#5f9ea0'], ['chartreuse', '#7fff00'], ['chocolate', '#d2691e'], ['coral', '#ff7f50'], ['cornflowerblue', '#6495ed'], ['cornsilk', '#fff8dc'], ['crimson', '#dc143c'], ['cyan', '#00ffff'], ['darkblue', '#00008b'], ['darkcyan', '#008b8b'], ['darkgoldenrod', '#b8860b'], ['darkgray', '#a9a9a9'], ['darkgreen', '#006400'], ['darkgrey', '#a9a9a9'], ['darkkhaki', '#bdb76b'], ['darkmagenta', '#8b008b'], ['darkolivegreen', '#556b2f'], ['darkorange', '#ff8c00'], ['darkorchid', '#9932cc'], ['darkred', '#8b0000'], ['darksalmon', '#e9967a'], ['darkseagreen', '#8fbc8f'], ['darkslateblue', '#483d8b'], ['darkslategray', '#2f4f4f'], ['darkslategrey', '#2f4f4f'], ['darkturquoise', '#00ced1'], ['darkviolet', '#9400d3'], ['deeppink', '#ff1493'], ['deepskyblue', '#00bfff'], ['dimgray', '#696969'], ['dimgrey', '#696969'], ['dodgerblue', '#1e90ff'], ['firebrick', '#b22222'], ['floralwhite', '#fffaf0'], ['forestgreen', '#228b22'], ['gainsboro', '#dcdcdc'], ['ghostwhite', '#f8f8ff'], ['gold', '#ffd700'], ['goldenrod', '#daa520'], ['greenyellow', '#adff2f'], ['grey', '#808080'], ['honeydew', '#f0fff0'], ['hotpink', '#ff69b4'], ['indianred', '#cd5c5c'], ['indigo', '#4b0082'], ['ivory', '#fffff0'], ['khaki', '#f0e68c'], ['lavender', '#e6e6fa'], ['lavenderblush', '#fff0f5'], ['lawngreen', '#7cfc00'], ['lemonchiffon', '#fffacd'], ['lightblue', '#add8e6'], ['lightcoral', '#f08080'], ['lightcyan', '#e0ffff'], ['lightgoldenrodyellow', '#fafad2'], ['lightgray', '#d3d3d3'], ['lightgreen', '#90ee90'], ['lightgrey', '#d3d3d3'], ['lightpink', '#ffb6c1'], ['lightsalmon', '#ffa07a'], ['lightseagreen', '#20b2aa'], ['lightskyblue', '#87cefa'], ['lightslategray', '#778899'], ['lightslategrey', '#778899'], ['lightsteelblue', '#b0c4de'], ['lightyellow', '#ffffe0'], ['limegreen', '#32cd32'], ['linen', '#faf0e6'], ['magenta', '#ff00ff'], ['mediumaquamarine', '#66cdaa'], ['mediumblue', '#0000cd'], ['mediumorchid', '#ba55d3'], ['mediumpurple', '#9370db'], ['mediumseagreen', '#3cb371'], ['mediumslateblue', '#7b68ee'], ['mediumspringgreen', '#00fa9a'], ['mediumturquoise', '#48d1cc'], ['mediumvioletred', '#c71585'], ['midnightblue', '#191970'], ['mintcream', '#f5fffa'], ['mistyrose', '#ffe4e1'], ['moccasin', '#ffe4b5'], ['navajowhite', '#ffdead'], ['oldlace', '#fdf5e6'], ['olivedrab', '#6b8e23'], ['orange', '#ffa500'], ['orangered', '#ff4500'], ['orchid', '#da70d6'], ['palegoldenrod', '#eee8aa'], ['palegreen', '#98fb98'], ['paleturquoise', '#afeeee'], ['palevioletred', '#db7093'], ['papayawhip', '#ffefd5'], ['peachpuff', '#ffdab9'], ['peru', '#cd853f'], ['pink', '#ffc0cb'], ['plum', '#dda0dd'], ['powderblue', '#b0e0e6'], ['rosybrown', '#bc8f8f'], ['royalblue', '#4169e1'], ['saddlebrown', '#8b4513'], ['salmon', '#fa8072'], ['sandybrown', '#f4a460'], ['seagreen', '#2e8b57'], ['seashell', '#fff5ee'], ['sienna', '#a0522d'], ['skyblue', '#87ceeb'], ['slateblue', '#6a5acd'], ['slategray', '#708090'], ['slategrey', '#708090'], ['snow', '#fffafa'], ['springgreen', '#00ff7f'], ['steelblue', '#4682b4'], ['tan', '#d2b48c'], ['thistle', '#d8bfd8'], ['tomato', '#ff6347'], ['turquoise', '#40e0d0'], ['violet', '#ee82ee'], ['wheat', '#f5deb3'], ['whitesmoke', '#f5f5f5'], ['yellowgreen', '#9acd32']]);

	var hexReg = /^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/;
	var rgbReg = /^(rgb|rgba|RGB|RGBA)/;
	/**
	 * @description Color validator
	 * @param {String} color Hex|Rgb|Rgba color or color keyword
	 * @return {String|Boolean} Valid color Or false
	 */

	function validator(color) {
	  var isHex = hexReg.test(color);
	  var isRgb = rgbReg.test(color);
	  if (isHex || isRgb) return color;
	  color = getColorByKeyword(color);

	  if (!color) {
	    console.error('Color: Invalid color!');
	    return false;
	  }

	  return color;
	}
	/**
	 * @description Get color by keyword
	 * @param {String} keyword Color keyword like red, green and etc.
	 * @return {String|Boolean} Hex or rgba color (Invalid keyword will return false)
	 */

	function getColorByKeyword(keyword) {
	  if (!keyword) {
	    console.error('getColorByKeywords: Missing parameters!');
	    return false;
	  }

	  if (!_keywords.has(keyword)) return false;
	  return _keywords.get(keyword);
	}
	/**
	 * @description Get the Rgb value of the color
	 * @param {String} color Hex|Rgb|Rgba color or color keyword
	 * @return {Array<Number>|Boolean} Rgb value of the color (Invalid input will return false)
	 */

	function getRgbValue(color) {
	  if (!color) {
	    console.error('getRgbValue: Missing parameters!');
	    return false;
	  }

	  color = validator(color);
	  if (!color) return false;
	  var isHex = hexReg.test(color);
	  var isRgb = rgbReg.test(color);
	  var lowerColor = color.toLowerCase();
	  if (isHex) return getRgbValueFromHex(lowerColor);
	  if (isRgb) return getRgbValueFromRgb(lowerColor);
	}
	/**
	 * @description Get the rgb value of the hex color
	 * @param {String} color Hex color
	 * @return {Array<Number>} Rgb value of the color
	 */

	function getRgbValueFromHex(color) {
	  color = color.replace('#', '');
	  if (color.length === 3) color = Array.from(color).map(function (hexNum) {
	    return hexNum + hexNum;
	  }).join('');
	  color = color.split('');
	  return new Array(3).fill(0).map(function (t, i) {
	    return parseInt("0x".concat(color[i * 2]).concat(color[i * 2 + 1]));
	  });
	}
	/**
	 * @description Get the rgb value of the rgb/rgba color
	 * @param {String} color Hex color
	 * @return {Array} Rgb value of the color
	 */

	function getRgbValueFromRgb(color) {
	  return color.replace(/rgb\(|rgba\(|\)/g, '').split(',').slice(0, 3).map(function (n) {
	    return parseInt(n);
	  });
	}
	/**
	 * @description Get Color from Rgb|Rgba value
	 * @param {Array<Number>} value Rgb|Rgba color value
	 * @return {String|Boolean} Rgb|Rgba color (Invalid input will return false)
	 */

	function getColorFromRgbValue(value) {
	  if (!value) {
	    console.error('getColorFromRgbValue: Missing parameters!');
	    return false;
	  }

	  var valueLength = value.length;

	  if (valueLength !== 3 && valueLength !== 4) {
	    console.error('getColorFromRgbValue: Value is illegal!');
	    return false;
	  }

	  var color = valueLength === 3 ? 'rgb(' : 'rgba(';
	  color += value.join(',') + ')';
	  return color;
	}
	/**
	 * @description Adjust color opacity
	 * @param {String} color   Hex|Rgb|Rgba color or color keyword
	 * @param {Number} Percent of opacity
	 * @return {String|Boolean} Rgba color (Invalid input will return false)
	 */

	function fade(color) {
	  var percent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 100;

	  if (!color) {
	    console.error('fade: Missing parameters!');
	    return false;
	  }

	  var rgbValue = getRgbValue(color);
	  if (!rgbValue) return false;
	  var rgbaValue = [].concat((0, toConsumableArray["default"])(rgbValue), [percent / 100]);
	  return getColorFromRgbValue(rgbaValue);
	}

	//
	var script$j = {
	  name: 'DvBorderBox11',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    titleWidth: {
	      type: Number,
	      default: 250
	    },
	    title: {
	      type: String,
	      default: ''
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'border-box-11',
	      filterId: `border-box-11-filterId-${id}`,
	      defaultColor: ['#8aaafb', '#1f33a2'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    },

	    fade
	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$j = script$j;

	/* template */
	var __vue_render__$j = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-11" }, [
	    _c(
	      "svg",
	      {
	        staticClass: "dv-border-svg-container",
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c("defs", [
	          _c(
	            "filter",
	            {
	              attrs: {
	                id: _vm.filterId,
	                height: "150%",
	                width: "150%",
	                x: "-25%",
	                y: "-25%"
	              }
	            },
	            [
	              _c("feMorphology", {
	                attrs: {
	                  operator: "dilate",
	                  radius: "2",
	                  in: "SourceAlpha",
	                  result: "thicken"
	                }
	              }),
	              _vm._v(" "),
	              _c("feGaussianBlur", {
	                attrs: { in: "thicken", stdDeviation: "3", result: "blurred" }
	              }),
	              _vm._v(" "),
	              _c("feFlood", {
	                attrs: {
	                  "flood-color": _vm.mergedColor[1],
	                  result: "glowColor"
	                }
	              }),
	              _vm._v(" "),
	              _c("feComposite", {
	                attrs: {
	                  in: "glowColor",
	                  in2: "blurred",
	                  operator: "in",
	                  result: "softGlowColored"
	                }
	              }),
	              _vm._v(" "),
	              _c(
	                "feMerge",
	                [
	                  _c("feMergeNode", { attrs: { in: "softGlowColored" } }),
	                  _vm._v(" "),
	                  _c("feMergeNode", { attrs: { in: "SourceGraphic" } })
	                ],
	                1
	              )
	            ],
	            1
	          )
	        ]),
	        _vm._v(" "),
	        _c("polygon", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            points:
	              "\n      20, 32 " +
	              (_vm.width * 0.5 - _vm.titleWidth / 2) +
	              ", 32 " +
	              (_vm.width * 0.5 - _vm.titleWidth / 2 + 20) +
	              ", 53\n      " +
	              (_vm.width * 0.5 + _vm.titleWidth / 2 - 20) +
	              ", 53 " +
	              (_vm.width * 0.5 + _vm.titleWidth / 2) +
	              ", 32\n      " +
	              (_vm.width - 20) +
	              ", 32 " +
	              (_vm.width - 8) +
	              ", 48 " +
	              (_vm.width - 8) +
	              ", " +
	              (_vm.height - 25) +
	              " " +
	              (_vm.width - 20) +
	              ", " +
	              (_vm.height - 8) +
	              "\n      20, " +
	              (_vm.height - 8) +
	              " 8, " +
	              (_vm.height - 25) +
	              " 8, 50\n    "
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            filter: "url(#" + _vm.filterId + ")",
	            points:
	              "\n        " +
	              (_vm.width - _vm.titleWidth) / 2 +
	              ", 30\n        20, 30 7, 50 7, " +
	              (50 + (_vm.height - 167) / 2) +
	              "\n        13, " +
	              (55 + (_vm.height - 167) / 2) +
	              " 13, " +
	              (135 + (_vm.height - 167) / 2) +
	              "\n        7, " +
	              (140 + (_vm.height - 167) / 2) +
	              " 7, " +
	              (_vm.height - 27) +
	              "\n        20, " +
	              (_vm.height - 7) +
	              " " +
	              (_vm.width - 20) +
	              ", " +
	              (_vm.height - 7) +
	              " " +
	              (_vm.width - 7) +
	              ", " +
	              (_vm.height - 27) +
	              "\n        " +
	              (_vm.width - 7) +
	              ", " +
	              (140 + (_vm.height - 167) / 2) +
	              " " +
	              (_vm.width - 13) +
	              ", " +
	              (135 + (_vm.height - 167) / 2) +
	              "\n        " +
	              (_vm.width - 13) +
	              ", " +
	              (55 + (_vm.height - 167) / 2) +
	              " " +
	              (_vm.width - 7) +
	              ", " +
	              (50 + (_vm.height - 167) / 2) +
	              "\n        " +
	              (_vm.width - 7) +
	              ", 50 " +
	              (_vm.width - 20) +
	              ", 30 " +
	              (_vm.width + _vm.titleWidth) / 2 +
	              ", 30\n        " +
	              ((_vm.width + _vm.titleWidth) / 2 - 20) +
	              ", 7 " +
	              ((_vm.width - _vm.titleWidth) / 2 + 20) +
	              ", 7\n        " +
	              (_vm.width - _vm.titleWidth) / 2 +
	              ", 30 " +
	              ((_vm.width - _vm.titleWidth) / 2 + 20) +
	              ", 52\n        " +
	              ((_vm.width + _vm.titleWidth) / 2 - 20) +
	              ", 52 " +
	              (_vm.width + _vm.titleWidth) / 2 +
	              ", 30\n      "
	          }
	        }),
	        _vm._v(" "),
	        _c("polygon", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            fill: "transparent",
	            points:
	              "\n        " +
	              ((_vm.width + _vm.titleWidth) / 2 - 5) +
	              ", 30 " +
	              ((_vm.width + _vm.titleWidth) / 2 - 21) +
	              ", 11\n        " +
	              ((_vm.width + _vm.titleWidth) / 2 - 27) +
	              ", 11 " +
	              ((_vm.width + _vm.titleWidth) / 2 - 8) +
	              ", 34\n      "
	          }
	        }),
	        _vm._v(" "),
	        _c("polygon", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            fill: "transparent",
	            points:
	              "\n        " +
	              ((_vm.width - _vm.titleWidth) / 2 + 5) +
	              ", 30 " +
	              ((_vm.width - _vm.titleWidth) / 2 + 22) +
	              ", 49\n        " +
	              ((_vm.width - _vm.titleWidth) / 2 + 28) +
	              ", 49 " +
	              ((_vm.width - _vm.titleWidth) / 2 + 8) +
	              ", 26\n      "
	          }
	        }),
	        _vm._v(" "),
	        _c("polygon", {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            fill: _vm.fade(_vm.mergedColor[1] || _vm.defaultColor[1], 30),
	            filter: "url(#" + _vm.filterId + ")",
	            points:
	              "\n        " +
	              ((_vm.width + _vm.titleWidth) / 2 - 11) +
	              ", 37 " +
	              ((_vm.width + _vm.titleWidth) / 2 - 32) +
	              ", 11\n        " +
	              ((_vm.width - _vm.titleWidth) / 2 + 23) +
	              ", 11 " +
	              ((_vm.width - _vm.titleWidth) / 2 + 11) +
	              ", 23\n        " +
	              ((_vm.width - _vm.titleWidth) / 2 + 33) +
	              ", 49 " +
	              ((_vm.width + _vm.titleWidth) / 2 - 22) +
	              ", 49\n      "
	          }
	        }),
	        _vm._v(" "),
	        _c(
	          "polygon",
	          {
	            attrs: {
	              filter: "url(#" + _vm.filterId + ")",
	              fill: _vm.mergedColor[0],
	              opacity: "1",
	              points:
	                "\n        " +
	                ((_vm.width - _vm.titleWidth) / 2 - 10) +
	                ", 37 " +
	                ((_vm.width - _vm.titleWidth) / 2 - 31) +
	                ", 37\n        " +
	                ((_vm.width - _vm.titleWidth) / 2 - 25) +
	                ", 46 " +
	                ((_vm.width - _vm.titleWidth) / 2 - 4) +
	                ", 46\n      "
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: "opacity",
	                values: "1;0.7;1",
	                dur: "2s",
	                begin: "0s",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        ),
	        _vm._v(" "),
	        _c(
	          "polygon",
	          {
	            attrs: {
	              filter: "url(#" + _vm.filterId + ")",
	              fill: _vm.mergedColor[0],
	              opacity: "0.7",
	              points:
	                "\n        " +
	                ((_vm.width - _vm.titleWidth) / 2 - 40) +
	                ", 37 " +
	                ((_vm.width - _vm.titleWidth) / 2 - 61) +
	                ", 37\n        " +
	                ((_vm.width - _vm.titleWidth) / 2 - 55) +
	                ", 46 " +
	                ((_vm.width - _vm.titleWidth) / 2 - 34) +
	                ", 46\n      "
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: "opacity",
	                values: "0.7;0.4;0.7",
	                dur: "2s",
	                begin: "0s",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        ),
	        _vm._v(" "),
	        _c(
	          "polygon",
	          {
	            attrs: {
	              filter: "url(#" + _vm.filterId + ")",
	              fill: _vm.mergedColor[0],
	              opacity: "0.5",
	              points:
	                "\n        " +
	                ((_vm.width - _vm.titleWidth) / 2 - 70) +
	                ", 37 " +
	                ((_vm.width - _vm.titleWidth) / 2 - 91) +
	                ", 37\n        " +
	                ((_vm.width - _vm.titleWidth) / 2 - 85) +
	                ", 46 " +
	                ((_vm.width - _vm.titleWidth) / 2 - 64) +
	                ", 46\n      "
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: "opacity",
	                values: "0.5;0.2;0.5",
	                dur: "2s",
	                begin: "0s",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        ),
	        _vm._v(" "),
	        _c(
	          "polygon",
	          {
	            attrs: {
	              filter: "url(#" + _vm.filterId + ")",
	              fill: _vm.mergedColor[0],
	              opacity: "1",
	              points:
	                "\n        " +
	                ((_vm.width + _vm.titleWidth) / 2 + 30) +
	                ", 37 " +
	                ((_vm.width + _vm.titleWidth) / 2 + 9) +
	                ", 37\n        " +
	                ((_vm.width + _vm.titleWidth) / 2 + 3) +
	                ", 46 " +
	                ((_vm.width + _vm.titleWidth) / 2 + 24) +
	                ", 46\n      "
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: "opacity",
	                values: "1;0.7;1",
	                dur: "2s",
	                begin: "0s",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        ),
	        _vm._v(" "),
	        _c(
	          "polygon",
	          {
	            attrs: {
	              filter: "url(#" + _vm.filterId + ")",
	              fill: _vm.mergedColor[0],
	              opacity: "0.7",
	              points:
	                "\n        " +
	                ((_vm.width + _vm.titleWidth) / 2 + 60) +
	                ", 37 " +
	                ((_vm.width + _vm.titleWidth) / 2 + 39) +
	                ", 37\n        " +
	                ((_vm.width + _vm.titleWidth) / 2 + 33) +
	                ", 46 " +
	                ((_vm.width + _vm.titleWidth) / 2 + 54) +
	                ", 46\n      "
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: "opacity",
	                values: "0.7;0.4;0.7",
	                dur: "2s",
	                begin: "0s",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        ),
	        _vm._v(" "),
	        _c(
	          "polygon",
	          {
	            attrs: {
	              filter: "url(#" + _vm.filterId + ")",
	              fill: _vm.mergedColor[0],
	              opacity: "0.5",
	              points:
	                "\n        " +
	                ((_vm.width + _vm.titleWidth) / 2 + 90) +
	                ", 37 " +
	                ((_vm.width + _vm.titleWidth) / 2 + 69) +
	                ", 37\n        " +
	                ((_vm.width + _vm.titleWidth) / 2 + 63) +
	                ", 46 " +
	                ((_vm.width + _vm.titleWidth) / 2 + 84) +
	                ", 46\n      "
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: "opacity",
	                values: "0.5;0.2;0.5",
	                dur: "2s",
	                begin: "0s",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        ),
	        _vm._v(" "),
	        _c(
	          "text",
	          {
	            staticClass: "dv-border-box-11-title",
	            attrs: {
	              x: "" + _vm.width / 2,
	              y: "32",
	              fill: "#fff",
	              "font-size": "18",
	              "text-anchor": "middle",
	              "dominant-baseline": "middle"
	            }
	          },
	          [_vm._v("\n      " + _vm._s(_vm.title) + "\n    ")]
	        ),
	        _vm._v(" "),
	        _c("polygon", {
	          attrs: {
	            fill: _vm.mergedColor[0],
	            filter: "url(#" + _vm.filterId + ")",
	            points:
	              "\n        7, " +
	              (53 + (_vm.height - 167) / 2) +
	              " 11, " +
	              (57 + (_vm.height - 167) / 2) +
	              "\n        11, " +
	              (133 + (_vm.height - 167) / 2) +
	              " 7, " +
	              (137 + (_vm.height - 167) / 2) +
	              "\n      "
	          }
	        }),
	        _vm._v(" "),
	        _c("polygon", {
	          attrs: {
	            fill: _vm.mergedColor[0],
	            filter: "url(#" + _vm.filterId + ")",
	            points:
	              "\n        " +
	              (_vm.width - 7) +
	              ", " +
	              (53 + (_vm.height - 167) / 2) +
	              " " +
	              (_vm.width - 11) +
	              ", " +
	              (57 + (_vm.height - 167) / 2) +
	              "\n        " +
	              (_vm.width - 11) +
	              ", " +
	              (133 + (_vm.height - 167) / 2) +
	              " " +
	              (_vm.width - 7) +
	              ", " +
	              (137 + (_vm.height - 167) / 2) +
	              "\n      "
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$j = [];
	__vue_render__$j._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$j = function (inject) {
	    if (!inject) return
	    inject("data-v-a6b73ba8_0", { source: ".dv-border-box-11 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-11 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-11 .dv-border-svg-container > polyline {\n  fill: none;\n  stroke-width: 1;\n}\n.dv-border-box-11 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,QAAQ;EACR,SAAS;AACX;AACA;EACE,UAAU;EACV,eAAe;AACjB;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-11 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-11 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-11 .dv-border-svg-container > polyline {\n  fill: none;\n  stroke-width: 1;\n}\n.dv-border-box-11 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$j = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$j = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$j = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$j = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$j, staticRenderFns: __vue_staticRenderFns__$j },
	    __vue_inject_styles__$j,
	    __vue_script__$j,
	    __vue_scope_id__$j,
	    __vue_is_functional_template__$j,
	    __vue_module_identifier__$j,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox11 (Vue) {
	  Vue.component(__vue_component__$j.name, __vue_component__$j);
	}

	//
	var script$i = {
	  name: 'DvBorderBox12',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'border-box-12',
	      filterId: `borderr-box-12-filterId-${id}`,
	      defaultColor: ['#2e6099', '#7ce7fd'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    },

	    fade
	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$i = script$i;

	/* template */
	var __vue_render__$i = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-12" }, [
	    _c(
	      "svg",
	      {
	        staticClass: "dv-border-svg-container",
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c("defs", [
	          _c(
	            "filter",
	            {
	              attrs: {
	                id: _vm.filterId,
	                height: "150%",
	                width: "150%",
	                x: "-25%",
	                y: "-25%"
	              }
	            },
	            [
	              _c("feMorphology", {
	                attrs: {
	                  operator: "dilate",
	                  radius: "1",
	                  in: "SourceAlpha",
	                  result: "thicken"
	                }
	              }),
	              _vm._v(" "),
	              _c("feGaussianBlur", {
	                attrs: { in: "thicken", stdDeviation: "2", result: "blurred" }
	              }),
	              _vm._v(" "),
	              _c(
	                "feFlood",
	                {
	                  attrs: {
	                    "flood-color": _vm.fade(
	                      _vm.mergedColor[1] || _vm.defaultColor[1],
	                      70
	                    ),
	                    result: "glowColor"
	                  }
	                },
	                [
	                  _c("animate", {
	                    attrs: {
	                      attributeName: "flood-color",
	                      values:
	                        "\n              " +
	                        _vm.fade(
	                          _vm.mergedColor[1] || _vm.defaultColor[1],
	                          70
	                        ) +
	                        ";\n              " +
	                        _vm.fade(
	                          _vm.mergedColor[1] || _vm.defaultColor[1],
	                          30
	                        ) +
	                        ";\n              " +
	                        _vm.fade(
	                          _vm.mergedColor[1] || _vm.defaultColor[1],
	                          70
	                        ) +
	                        ";\n            ",
	                      dur: "3s",
	                      begin: "0s",
	                      repeatCount: "indefinite"
	                    }
	                  })
	                ]
	              ),
	              _vm._v(" "),
	              _c("feComposite", {
	                attrs: {
	                  in: "glowColor",
	                  in2: "blurred",
	                  operator: "in",
	                  result: "softGlowColored"
	                }
	              }),
	              _vm._v(" "),
	              _c(
	                "feMerge",
	                [
	                  _c("feMergeNode", { attrs: { in: "softGlowColored" } }),
	                  _vm._v(" "),
	                  _c("feMergeNode", { attrs: { in: "SourceGraphic" } })
	                ],
	                1
	              )
	            ],
	            1
	          )
	        ]),
	        _vm._v(" "),
	        _vm.width && _vm.height
	          ? _c("path", {
	              attrs: {
	                fill: _vm.backgroundColor,
	                "stroke-width": "2",
	                stroke: _vm.mergedColor[0],
	                d:
	                  "\n        M15 5 L " +
	                  (_vm.width - 15) +
	                  " 5 Q " +
	                  (_vm.width - 5) +
	                  " 5, " +
	                  (_vm.width - 5) +
	                  " 15\n        L " +
	                  (_vm.width - 5) +
	                  " " +
	                  (_vm.height - 15) +
	                  " Q " +
	                  (_vm.width - 5) +
	                  " " +
	                  (_vm.height - 5) +
	                  ", " +
	                  (_vm.width - 15) +
	                  " " +
	                  (_vm.height - 5) +
	                  "\n        L 15, " +
	                  (_vm.height - 5) +
	                  " Q 5 " +
	                  (_vm.height - 5) +
	                  " 5 " +
	                  (_vm.height - 15) +
	                  " L 5 15\n        Q 5 5 15 5\n      "
	              }
	            })
	          : _vm._e(),
	        _vm._v(" "),
	        _c("path", {
	          attrs: {
	            "stroke-width": "2",
	            fill: "transparent",
	            "stroke-linecap": "round",
	            filter: "url(#" + _vm.filterId + ")",
	            stroke: _vm.mergedColor[1],
	            d: "M 20 5 L 15 5 Q 5 5 5 15 L 5 20"
	          }
	        }),
	        _vm._v(" "),
	        _c("path", {
	          attrs: {
	            "stroke-width": "2",
	            fill: "transparent",
	            "stroke-linecap": "round",
	            filter: "url(#" + _vm.filterId + ")",
	            stroke: _vm.mergedColor[1],
	            d:
	              "M " +
	              (_vm.width - 20) +
	              " 5 L " +
	              (_vm.width - 15) +
	              " 5 Q " +
	              (_vm.width - 5) +
	              " 5 " +
	              (_vm.width - 5) +
	              " 15 L " +
	              (_vm.width - 5) +
	              " 20"
	          }
	        }),
	        _vm._v(" "),
	        _c("path", {
	          attrs: {
	            "stroke-width": "2",
	            fill: "transparent",
	            "stroke-linecap": "round",
	            filter: "url(#" + _vm.filterId + ")",
	            stroke: _vm.mergedColor[1],
	            d:
	              "\n        M " +
	              (_vm.width - 20) +
	              " " +
	              (_vm.height - 5) +
	              " L " +
	              (_vm.width - 15) +
	              " " +
	              (_vm.height - 5) +
	              "\n        Q " +
	              (_vm.width - 5) +
	              " " +
	              (_vm.height - 5) +
	              " " +
	              (_vm.width - 5) +
	              " " +
	              (_vm.height - 15) +
	              "\n        L " +
	              (_vm.width - 5) +
	              " " +
	              (_vm.height - 20) +
	              "\n      "
	          }
	        }),
	        _vm._v(" "),
	        _c("path", {
	          attrs: {
	            "stroke-width": "2",
	            fill: "transparent",
	            "stroke-linecap": "round",
	            filter: "url(#" + _vm.filterId + ")",
	            stroke: _vm.mergedColor[1],
	            d:
	              "\n        M 20 " +
	              (_vm.height - 5) +
	              " L 15 " +
	              (_vm.height - 5) +
	              "\n        Q 5 " +
	              (_vm.height - 5) +
	              " 5 " +
	              (_vm.height - 15) +
	              "\n        L 5 " +
	              (_vm.height - 20) +
	              "\n      "
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$i = [];
	__vue_render__$i._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$i = function (inject) {
	    if (!inject) return
	    inject("data-v-398dc3f6_0", { source: ".dv-border-box-12 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-12 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-12 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,QAAQ;EACR,SAAS;AACX;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-12 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-12 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-12 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$i = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$i = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$i = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$i = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$i, staticRenderFns: __vue_staticRenderFns__$i },
	    __vue_inject_styles__$i,
	    __vue_script__$i,
	    __vue_scope_id__$i,
	    __vue_is_functional_template__$i,
	    __vue_module_identifier__$i,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox12 (Vue) {
	  Vue.component(__vue_component__$i.name, __vue_component__$i);
	}

	//
	var script$h = {
	  name: 'DvBorderBox13',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    backgroundColor: {
	      type: String,
	      default: 'transparent'
	    }
	  },

	  data() {
	    return {
	      ref: 'border-box-13',
	      defaultColor: ['#6586ec', '#2cf7fe'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$h = script$h;

	/* template */
	var __vue_render__$h = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-border-box-13" }, [
	    _c(
	      "svg",
	      {
	        staticClass: "dv-border-svg-container",
	        attrs: { width: _vm.width, height: _vm.height }
	      },
	      [
	        _c("path", {
	          attrs: {
	            fill: _vm.backgroundColor,
	            stroke: _vm.mergedColor[0],
	            d:
	              "\n        M 5 20 L 5 10 L 12 3  L 60 3 L 68 10\n        L " +
	              (_vm.width - 20) +
	              " 10 L " +
	              (_vm.width - 5) +
	              " 25\n        L " +
	              (_vm.width - 5) +
	              " " +
	              (_vm.height - 5) +
	              " L 20 " +
	              (_vm.height - 5) +
	              "\n        L 5 " +
	              (_vm.height - 20) +
	              " L 5 20\n      "
	          }
	        }),
	        _vm._v(" "),
	        _c("path", {
	          attrs: {
	            fill: "transparent",
	            "stroke-width": "3",
	            "stroke-linecap": "round",
	            "stroke-dasharray": "10, 5",
	            stroke: _vm.mergedColor[0],
	            d: "M 16 9 L 61 9"
	          }
	        }),
	        _vm._v(" "),
	        _c("path", {
	          attrs: {
	            fill: "transparent",
	            stroke: _vm.mergedColor[1],
	            d: "M 5 20 L 5 10 L 12 3  L 60 3 L 68 10"
	          }
	        }),
	        _vm._v(" "),
	        _c("path", {
	          attrs: {
	            fill: "transparent",
	            stroke: _vm.mergedColor[1],
	            d:
	              "M " +
	              (_vm.width - 5) +
	              " " +
	              (_vm.height - 30) +
	              " L " +
	              (_vm.width - 5) +
	              " " +
	              (_vm.height - 5) +
	              " L " +
	              (_vm.width - 30) +
	              " " +
	              (_vm.height - 5)
	          }
	        })
	      ]
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "border-box-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$h = [];
	__vue_render__$h._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$h = function (inject) {
	    if (!inject) return
	    inject("data-v-47a0c184_0", { source: ".dv-border-box-13 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-13 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-13 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,QAAQ;EACR,SAAS;AACX;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-border-box-13 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-border-box-13 .dv-border-svg-container {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  top: 0px;\n  left: 0px;\n}\n.dv-border-box-13 .border-box-content {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$h = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$h = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$h = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$h = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$h, staticRenderFns: __vue_staticRenderFns__$h },
	    __vue_inject_styles__$h,
	    __vue_script__$h,
	    __vue_scope_id__$h,
	    __vue_is_functional_template__$h,
	    __vue_module_identifier__$h,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function borderBox13 (Vue) {
	  Vue.component(__vue_component__$h.name, __vue_component__$h);
	}

	//
	var script$g = {
	  name: 'DvDecoration1',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    }
	  },

	  data() {
	    const pointSideLength = 2.5;
	    return {
	      ref: 'decoration-1',
	      svgWH: [200, 50],
	      svgScale: [1, 1],
	      rowNum: 4,
	      rowPoints: 20,
	      pointSideLength,
	      halfPointSideLength: pointSideLength / 2,
	      points: [],
	      rects: [],
	      defaultColor: ['#fff', '#0de7c2'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    calcSVGData() {
	      const {
	        calcPointsPosition,
	        calcRectsPosition,
	        calcScale
	      } = this;
	      calcPointsPosition();
	      calcRectsPosition();
	      calcScale();
	    },

	    calcPointsPosition() {
	      const {
	        svgWH,
	        rowNum,
	        rowPoints
	      } = this;
	      const [w, h] = svgWH;
	      const horizontalGap = w / (rowPoints + 1);
	      const verticalGap = h / (rowNum + 1);
	      let points = new Array(rowNum).fill(0).map((foo, i) => new Array(rowPoints).fill(0).map((foo, j) => [horizontalGap * (j + 1), verticalGap * (i + 1)]));
	      this.points = points.reduce((all, item) => [...all, ...item], []);
	    },

	    calcRectsPosition() {
	      const {
	        points,
	        rowPoints
	      } = this;
	      const rect1 = points[rowPoints * 2 - 1];
	      const rect2 = points[rowPoints * 2 - 3];
	      this.rects = [rect1, rect2];
	    },

	    calcScale() {
	      const {
	        width,
	        height,
	        svgWH
	      } = this;
	      const [w, h] = svgWH;
	      this.svgScale = [width / w, height / h];
	    },

	    onResize() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$g = script$g;

	/* template */
	var __vue_render__$g = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-1" }, [
	    _c(
	      "svg",
	      {
	        style:
	          "transform:scale(" + _vm.svgScale[0] + "," + _vm.svgScale[1] + ");",
	        attrs: { width: _vm.svgWH[0] + "px", height: _vm.svgWH[1] + "px" }
	      },
	      [
	        _vm._l(_vm.points, function(point, i) {
	          return [
	            Math.random() > 0.6
	              ? _c(
	                  "rect",
	                  {
	                    key: i,
	                    attrs: {
	                      fill: _vm.mergedColor[0],
	                      x: point[0] - _vm.halfPointSideLength,
	                      y: point[1] - _vm.halfPointSideLength,
	                      width: _vm.pointSideLength,
	                      height: _vm.pointSideLength
	                    }
	                  },
	                  [
	                    Math.random() > 0.6
	                      ? _c("animate", {
	                          attrs: {
	                            attributeName: "fill",
	                            values: _vm.mergedColor[0] + ";transparent",
	                            dur: "1s",
	                            begin: Math.random() * 2,
	                            repeatCount: "indefinite"
	                          }
	                        })
	                      : _vm._e()
	                  ]
	                )
	              : _vm._e()
	          ]
	        }),
	        _vm._v(" "),
	        _vm.rects[0]
	          ? _c(
	              "rect",
	              {
	                attrs: {
	                  fill: _vm.mergedColor[1],
	                  x: _vm.rects[0][0] - _vm.pointSideLength,
	                  y: _vm.rects[0][1] - _vm.pointSideLength,
	                  width: _vm.pointSideLength * 2,
	                  height: _vm.pointSideLength * 2
	                }
	              },
	              [
	                _c("animate", {
	                  attrs: {
	                    attributeName: "width",
	                    values: "0;" + _vm.pointSideLength * 2,
	                    dur: "2s",
	                    repeatCount: "indefinite"
	                  }
	                }),
	                _vm._v(" "),
	                _c("animate", {
	                  attrs: {
	                    attributeName: "height",
	                    values: "0;" + _vm.pointSideLength * 2,
	                    dur: "2s",
	                    repeatCount: "indefinite"
	                  }
	                }),
	                _vm._v(" "),
	                _c("animate", {
	                  attrs: {
	                    attributeName: "x",
	                    values:
	                      _vm.rects[0][0] +
	                      ";" +
	                      (_vm.rects[0][0] - _vm.pointSideLength),
	                    dur: "2s",
	                    repeatCount: "indefinite"
	                  }
	                }),
	                _vm._v(" "),
	                _c("animate", {
	                  attrs: {
	                    attributeName: "y",
	                    values:
	                      _vm.rects[0][1] +
	                      ";" +
	                      (_vm.rects[0][1] - _vm.pointSideLength),
	                    dur: "2s",
	                    repeatCount: "indefinite"
	                  }
	                })
	              ]
	            )
	          : _vm._e(),
	        _vm._v(" "),
	        _vm.rects[1]
	          ? _c(
	              "rect",
	              {
	                attrs: {
	                  fill: _vm.mergedColor[1],
	                  x: _vm.rects[1][0] - 40,
	                  y: _vm.rects[1][1] - _vm.pointSideLength,
	                  width: 40,
	                  height: _vm.pointSideLength * 2
	                }
	              },
	              [
	                _c("animate", {
	                  attrs: {
	                    attributeName: "width",
	                    values: "0;40;0",
	                    dur: "2s",
	                    repeatCount: "indefinite"
	                  }
	                }),
	                _vm._v(" "),
	                _c("animate", {
	                  attrs: {
	                    attributeName: "x",
	                    values:
	                      _vm.rects[1][0] +
	                      ";" +
	                      (_vm.rects[1][0] - 40) +
	                      ";" +
	                      _vm.rects[1][0],
	                    dur: "2s",
	                    repeatCount: "indefinite"
	                  }
	                })
	              ]
	            )
	          : _vm._e()
	      ],
	      2
	    )
	  ])
	};
	var __vue_staticRenderFns__$g = [];
	__vue_render__$g._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$g = function (inject) {
	    if (!inject) return
	    inject("data-v-c82b15b6_0", { source: ".dv-decoration-1 {\n  width: 100%;\n  height: 100%;\n}\n.dv-decoration-1 svg {\n  transform-origin: left top;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,WAAW;EACX,YAAY;AACd;AACA;EACE,0BAA0B;AAC5B","file":"main.vue","sourcesContent":[".dv-decoration-1 {\n  width: 100%;\n  height: 100%;\n}\n.dv-decoration-1 svg {\n  transform-origin: left top;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$g = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$g = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$g = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$g = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$g, staticRenderFns: __vue_staticRenderFns__$g },
	    __vue_inject_styles__$g,
	    __vue_script__$g,
	    __vue_scope_id__$g,
	    __vue_is_functional_template__$g,
	    __vue_module_identifier__$g,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration1 (Vue) {
	  Vue.component(__vue_component__$g.name, __vue_component__$g);
	}

	//
	var script$f = {
	  name: 'DvDecoration2',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    reverse: {
	      type: Boolean,
	      default: false
	    },
	    dur: {
	      type: Number,
	      default: 6
	    }
	  },

	  data() {
	    return {
	      ref: 'decoration-2',
	      x: 0,
	      y: 0,
	      w: 0,
	      h: 0,
	      defaultColor: ['#3faacb', '#fff'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    },

	    reverse() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    calcSVGData() {
	      const {
	        reverse,
	        width,
	        height
	      } = this;

	      if (reverse) {
	        this.w = 1;
	        this.h = height;
	        this.x = width / 2;
	        this.y = 0;
	      } else {
	        this.w = width;
	        this.h = 1;
	        this.x = 0;
	        this.y = height / 2;
	      }
	    },

	    onResize() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$f = script$f;

	/* template */
	var __vue_render__$f = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-2" }, [
	    _c(
	      "svg",
	      { attrs: { width: _vm.width + "px", height: _vm.height + "px" } },
	      [
	        _c(
	          "rect",
	          {
	            attrs: {
	              x: _vm.x,
	              y: _vm.y,
	              width: _vm.w,
	              height: _vm.h,
	              fill: _vm.mergedColor[0]
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: _vm.reverse ? "height" : "width",
	                from: "0",
	                to: _vm.reverse ? _vm.height : _vm.width,
	                dur: _vm.dur + "s",
	                calcMode: "spline",
	                keyTimes: "0;1",
	                keySplines: ".42,0,.58,1",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        ),
	        _vm._v(" "),
	        _c(
	          "rect",
	          {
	            attrs: {
	              x: _vm.x,
	              y: _vm.y,
	              width: "1",
	              height: "1",
	              fill: _vm.mergedColor[1]
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: _vm.reverse ? "y" : "x",
	                from: "0",
	                to: _vm.reverse ? _vm.height : _vm.width,
	                dur: _vm.dur + "s",
	                calcMode: "spline",
	                keyTimes: "0;1",
	                keySplines: "0.42,0,0.58,1",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        )
	      ]
	    )
	  ])
	};
	var __vue_staticRenderFns__$f = [];
	__vue_render__$f._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$f = function (inject) {
	    if (!inject) return
	    inject("data-v-b70fee02_0", { source: ".dv-decoration-2 {\n  display: flex;\n  width: 100%;\n  height: 100%;\n  justify-content: center;\n  align-items: center;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,aAAa;EACb,WAAW;EACX,YAAY;EACZ,uBAAuB;EACvB,mBAAmB;AACrB","file":"main.vue","sourcesContent":[".dv-decoration-2 {\n  display: flex;\n  width: 100%;\n  height: 100%;\n  justify-content: center;\n  align-items: center;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$f = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$f = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$f = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$f = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$f, staticRenderFns: __vue_staticRenderFns__$f },
	    __vue_inject_styles__$f,
	    __vue_script__$f,
	    __vue_scope_id__$f,
	    __vue_is_functional_template__$f,
	    __vue_module_identifier__$f,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration2 (Vue) {
	  Vue.component(__vue_component__$f.name, __vue_component__$f);
	}

	//
	var script$e = {
	  name: 'DvDecoration3',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    }
	  },

	  data() {
	    const pointSideLength = 7;
	    return {
	      ref: 'decoration-3',
	      svgWH: [300, 35],
	      svgScale: [1, 1],
	      rowNum: 2,
	      rowPoints: 25,
	      pointSideLength,
	      halfPointSideLength: pointSideLength / 2,
	      points: [],
	      defaultColor: ['#7acaec', 'transparent'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    calcSVGData() {
	      const {
	        calcPointsPosition,
	        calcScale
	      } = this;
	      calcPointsPosition();
	      calcScale();
	    },

	    calcPointsPosition() {
	      const {
	        svgWH,
	        rowNum,
	        rowPoints
	      } = this;
	      const [w, h] = svgWH;
	      const horizontalGap = w / (rowPoints + 1);
	      const verticalGap = h / (rowNum + 1);
	      let points = new Array(rowNum).fill(0).map((foo, i) => new Array(rowPoints).fill(0).map((foo, j) => [horizontalGap * (j + 1), verticalGap * (i + 1)]));
	      this.points = points.reduce((all, item) => [...all, ...item], []);
	    },

	    calcScale() {
	      const {
	        width,
	        height,
	        svgWH
	      } = this;
	      const [w, h] = svgWH;
	      this.svgScale = [width / w, height / h];
	    },

	    onResize() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$e = script$e;

	/* template */
	var __vue_render__$e = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-3" }, [
	    _c(
	      "svg",
	      {
	        style:
	          "transform:scale(" + _vm.svgScale[0] + "," + _vm.svgScale[1] + ");",
	        attrs: { width: _vm.svgWH[0] + "px", height: _vm.svgWH[1] + "px" }
	      },
	      [
	        _vm._l(_vm.points, function(point, i) {
	          return _c(
	            "rect",
	            {
	              key: i,
	              attrs: {
	                fill: _vm.mergedColor[0],
	                x: point[0] - _vm.halfPointSideLength,
	                y: point[1] - _vm.halfPointSideLength,
	                width: _vm.pointSideLength,
	                height: _vm.pointSideLength
	              }
	            },
	            [
	              Math.random() > 0.6
	                ? _c("animate", {
	                    attrs: {
	                      attributeName: "fill",
	                      values: "" + _vm.mergedColor.join(";"),
	                      dur: Math.random() + 1 + "s",
	                      begin: Math.random() * 2,
	                      repeatCount: "indefinite"
	                    }
	                  })
	                : _vm._e()
	            ]
	          )
	        })
	      ],
	      2
	    )
	  ])
	};
	var __vue_staticRenderFns__$e = [];
	__vue_render__$e._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$e = function (inject) {
	    if (!inject) return
	    inject("data-v-67895534_0", { source: ".dv-decoration-3 {\n  width: 100%;\n  height: 100%;\n}\n.dv-decoration-3 svg {\n  transform-origin: left top;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,WAAW;EACX,YAAY;AACd;AACA;EACE,0BAA0B;AAC5B","file":"main.vue","sourcesContent":[".dv-decoration-3 {\n  width: 100%;\n  height: 100%;\n}\n.dv-decoration-3 svg {\n  transform-origin: left top;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$e = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$e = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$e = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$e = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$e, staticRenderFns: __vue_staticRenderFns__$e },
	    __vue_inject_styles__$e,
	    __vue_script__$e,
	    __vue_scope_id__$e,
	    __vue_is_functional_template__$e,
	    __vue_module_identifier__$e,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration3 (Vue) {
	  Vue.component(__vue_component__$e.name, __vue_component__$e);
	}

	//
	var script$d = {
	  name: 'DvDecoration4',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    reverse: {
	      type: Boolean,
	      default: false
	    },
	    dur: {
	      type: Number,
	      default: 3
	    }
	  },

	  data() {
	    return {
	      ref: 'decoration-4',
	      defaultColor: ['rgba(255, 255, 255, 0.3)', 'rgba(255, 255, 255, 0.3)'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$d = script$d;

	/* template */
	var __vue_render__$d = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-4" }, [
	    _c(
	      "div",
	      {
	        class: "container " + (_vm.reverse ? "reverse" : "normal"),
	        style: _vm.reverse
	          ? "width:" +
	            _vm.width +
	            "px;height:5px;animation-duration:" +
	            _vm.dur +
	            "s"
	          : "width:5px;height:" +
	            _vm.height +
	            "px;animation-duration:" +
	            _vm.dur +
	            "s"
	      },
	      [
	        _c(
	          "svg",
	          {
	            attrs: {
	              width: _vm.reverse ? _vm.width : 5,
	              height: _vm.reverse ? 5 : _vm.height
	            }
	          },
	          [
	            _c("polyline", {
	              attrs: {
	                stroke: _vm.mergedColor[0],
	                points: _vm.reverse
	                  ? "0, 2.5 " + _vm.width + ", 2.5"
	                  : "2.5, 0 2.5, " + _vm.height
	              }
	            }),
	            _vm._v(" "),
	            _c("polyline", {
	              staticClass: "bold-line",
	              attrs: {
	                stroke: _vm.mergedColor[1],
	                "stroke-width": "3",
	                "stroke-dasharray": "20, 80",
	                "stroke-dashoffset": "-30",
	                points: _vm.reverse
	                  ? "0, 2.5 " + _vm.width + ", 2.5"
	                  : "2.5, 0 2.5, " + _vm.height
	              }
	            })
	          ]
	        )
	      ]
	    )
	  ])
	};
	var __vue_staticRenderFns__$d = [];
	__vue_render__$d._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$d = function (inject) {
	    if (!inject) return
	    inject("data-v-70ae0c19_0", { source: ".dv-decoration-4 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-decoration-4 .container {\n  display: flex;\n  overflow: hidden;\n  position: absolute;\n  flex: 1;\n}\n.dv-decoration-4 .normal {\n  animation: ani-height ease-in-out infinite;\n  left: 50%;\n  margin-left: -2px;\n}\n.dv-decoration-4 .reverse {\n  animation: ani-width ease-in-out infinite;\n  top: 50%;\n  margin-top: -2px;\n}\n@keyframes ani-height {\n0% {\n    height: 0%;\n}\n70% {\n    height: 100%;\n}\n100% {\n    height: 100%;\n}\n}\n@keyframes ani-width {\n0% {\n    width: 0%;\n}\n70% {\n    width: 100%;\n}\n100% {\n    width: 100%;\n}\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;AACd;AACA;EACE,aAAa;EACb,gBAAgB;EAChB,kBAAkB;EAClB,OAAO;AACT;AACA;EACE,0CAA0C;EAC1C,SAAS;EACT,iBAAiB;AACnB;AACA;EACE,yCAAyC;EACzC,QAAQ;EACR,gBAAgB;AAClB;AACA;AACE;IACE,UAAU;AACZ;AACA;IACE,YAAY;AACd;AACA;IACE,YAAY;AACd;AACF;AACA;AACE;IACE,SAAS;AACX;AACA;IACE,WAAW;AACb;AACA;IACE,WAAW;AACb;AACF","file":"main.vue","sourcesContent":[".dv-decoration-4 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n}\n.dv-decoration-4 .container {\n  display: flex;\n  overflow: hidden;\n  position: absolute;\n  flex: 1;\n}\n.dv-decoration-4 .normal {\n  animation: ani-height ease-in-out infinite;\n  left: 50%;\n  margin-left: -2px;\n}\n.dv-decoration-4 .reverse {\n  animation: ani-width ease-in-out infinite;\n  top: 50%;\n  margin-top: -2px;\n}\n@keyframes ani-height {\n  0% {\n    height: 0%;\n  }\n  70% {\n    height: 100%;\n  }\n  100% {\n    height: 100%;\n  }\n}\n@keyframes ani-width {\n  0% {\n    width: 0%;\n  }\n  70% {\n    width: 100%;\n  }\n  100% {\n    width: 100%;\n  }\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$d = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$d = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$d = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$d = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$d, staticRenderFns: __vue_staticRenderFns__$d },
	    __vue_inject_styles__$d,
	    __vue_script__$d,
	    __vue_scope_id__$d,
	    __vue_is_functional_template__$d,
	    __vue_module_identifier__$d,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration4 (Vue) {
	  Vue.component(__vue_component__$d.name, __vue_component__$d);
	}

	//
	var script$c = {
	  name: 'DvDecoration5',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    dur: {
	      type: Number,
	      default: 1.2
	    }
	  },

	  data() {
	    return {
	      ref: 'decoration-5',
	      line1Points: '',
	      line2Points: '',
	      line1Length: 0,
	      line2Length: 0,
	      defaultColor: ['#3f96a5', '#3f96a5'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    calcSVGData() {
	      const {
	        width,
	        height
	      } = this;
	      let line1Points = [[0, height * 0.2], [width * 0.18, height * 0.2], [width * 0.2, height * 0.4], [width * 0.25, height * 0.4], [width * 0.27, height * 0.6], [width * 0.72, height * 0.6], [width * 0.75, height * 0.4], [width * 0.8, height * 0.4], [width * 0.82, height * 0.2], [width, height * 0.2]];
	      let line2Points = [[width * 0.3, height * 0.8], [width * 0.7, height * 0.8]];
	      const line1Length = getPolylineLength(line1Points);
	      const line2Length = getPolylineLength(line2Points);
	      line1Points = line1Points.map(point => point.join(',')).join(' ');
	      line2Points = line2Points.map(point => point.join(',')).join(' ');
	      this.line1Points = line1Points;
	      this.line2Points = line2Points;
	      this.line1Length = line1Length;
	      this.line2Length = line2Length;
	    },

	    onResize() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$c = script$c;

	/* template */
	var __vue_render__$c = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-5" }, [
	    _c("svg", { attrs: { width: _vm.width, height: _vm.height } }, [
	      _c(
	        "polyline",
	        {
	          attrs: {
	            fill: "transparent",
	            stroke: _vm.mergedColor[0],
	            "stroke-width": "3",
	            points: _vm.line1Points
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              attributeName: "stroke-dasharray",
	              attributeType: "XML",
	              from: "0, " + _vm.line1Length / 2 + ", 0, " + _vm.line1Length / 2,
	              to: "0, 0, " + _vm.line1Length + ", 0",
	              dur: _vm.dur + "s",
	              begin: "0s",
	              calcMode: "spline",
	              keyTimes: "0;1",
	              keySplines: "0.4,1,0.49,0.98",
	              repeatCount: "indefinite"
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _c(
	        "polyline",
	        {
	          attrs: {
	            fill: "transparent",
	            stroke: _vm.mergedColor[1],
	            "stroke-width": "2",
	            points: _vm.line2Points
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              attributeName: "stroke-dasharray",
	              attributeType: "XML",
	              from: "0, " + _vm.line2Length / 2 + ", 0, " + _vm.line2Length / 2,
	              to: "0, 0, " + _vm.line2Length + ", 0",
	              dur: _vm.dur + "s",
	              begin: "0s",
	              calcMode: "spline",
	              keyTimes: "0;1",
	              keySplines: ".4,1,.49,.98",
	              repeatCount: "indefinite"
	            }
	          })
	        ]
	      )
	    ])
	  ])
	};
	var __vue_staticRenderFns__$c = [];
	__vue_render__$c._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$c = function (inject) {
	    if (!inject) return
	    inject("data-v-6b95f92e_0", { source: ".dv-decoration-5 {\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-decoration-5 {\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$c = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$c = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$c = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$c = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$c, staticRenderFns: __vue_staticRenderFns__$c },
	    __vue_inject_styles__$c,
	    __vue_script__$c,
	    __vue_scope_id__$c,
	    __vue_is_functional_template__$c,
	    __vue_module_identifier__$c,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration5 (Vue) {
	  Vue.component(__vue_component__$c.name, __vue_component__$c);
	}

	//
	var script$b = {
	  name: 'DvDecoration6',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    }
	  },

	  data() {
	    const rectWidth = 7;
	    return {
	      ref: 'decoration-6',
	      svgWH: [300, 35],
	      svgScale: [1, 1],
	      rowNum: 1,
	      rowPoints: 40,
	      rectWidth,
	      halfRectWidth: rectWidth / 2,
	      points: [],
	      heights: [],
	      minHeights: [],
	      randoms: [],
	      defaultColor: ['#7acaec', '#7acaec'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    calcSVGData() {
	      const {
	        calcPointsPosition,
	        calcScale
	      } = this;
	      calcPointsPosition();
	      calcScale();
	    },

	    calcPointsPosition() {
	      const {
	        svgWH,
	        rowNum,
	        rowPoints
	      } = this;
	      const [w, h] = svgWH;
	      const horizontalGap = w / (rowPoints + 1);
	      const verticalGap = h / (rowNum + 1);
	      let points = new Array(rowNum).fill(0).map((foo, i) => new Array(rowPoints).fill(0).map((foo, j) => [horizontalGap * (j + 1), verticalGap * (i + 1)]));
	      this.points = points.reduce((all, item) => [...all, ...item], []);
	      const heights = this.heights = new Array(rowNum * rowPoints).fill(0).map(foo => Math.random() > 0.8 ? randomExtend(0.7 * h, h) : randomExtend(0.2 * h, 0.5 * h));
	      this.minHeights = new Array(rowNum * rowPoints).fill(0).map((foo, i) => heights[i] * Math.random());
	      this.randoms = new Array(rowNum * rowPoints).fill(0).map(foo => Math.random() + 1.5);
	    },

	    calcScale() {
	      const {
	        width,
	        height,
	        svgWH
	      } = this;
	      const [w, h] = svgWH;
	      this.svgScale = [width / w, height / h];
	    },

	    onResize() {
	      const {
	        calcSVGData
	      } = this;
	      calcSVGData();
	    },

	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$b = script$b;

	/* template */
	var __vue_render__$b = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-6" }, [
	    _c(
	      "svg",
	      {
	        style:
	          "transform:scale(" + _vm.svgScale[0] + "," + _vm.svgScale[1] + ");",
	        attrs: { width: _vm.svgWH[0] + "px", height: _vm.svgWH[1] + "px" }
	      },
	      [
	        _vm._l(_vm.points, function(point, i) {
	          return _c(
	            "rect",
	            {
	              key: i,
	              attrs: {
	                fill: _vm.mergedColor[Math.random() > 0.5 ? 0 : 1],
	                x: point[0] - _vm.halfRectWidth,
	                y: point[1] - _vm.heights[i] / 2,
	                width: _vm.rectWidth,
	                height: _vm.heights[i]
	              }
	            },
	            [
	              _c("animate", {
	                attrs: {
	                  attributeName: "y",
	                  values:
	                    point[1] -
	                    _vm.minHeights[i] / 2 +
	                    ";" +
	                    (point[1] - _vm.heights[i] / 2) +
	                    ";" +
	                    (point[1] - _vm.minHeights[i] / 2),
	                  dur: _vm.randoms[i] + "s",
	                  keyTimes: "0;0.5;1",
	                  calcMode: "spline",
	                  keySplines: "0.42,0,0.58,1;0.42,0,0.58,1",
	                  begin: "0s",
	                  repeatCount: "indefinite"
	                }
	              }),
	              _vm._v(" "),
	              _c("animate", {
	                attrs: {
	                  attributeName: "height",
	                  values:
	                    _vm.minHeights[i] +
	                    ";" +
	                    _vm.heights[i] +
	                    ";" +
	                    _vm.minHeights[i],
	                  dur: _vm.randoms[i] + "s",
	                  keyTimes: "0;0.5;1",
	                  calcMode: "spline",
	                  keySplines: "0.42,0,0.58,1;0.42,0,0.58,1",
	                  begin: "0s",
	                  repeatCount: "indefinite"
	                }
	              })
	            ]
	          )
	        })
	      ],
	      2
	    )
	  ])
	};
	var __vue_staticRenderFns__$b = [];
	__vue_render__$b._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$b = function (inject) {
	    if (!inject) return
	    inject("data-v-3537cf00_0", { source: ".dv-decoration-6 {\n  width: 100%;\n  height: 100%;\n}\n.dv-decoration-6 svg {\n  transform-origin: left top;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,WAAW;EACX,YAAY;AACd;AACA;EACE,0BAA0B;AAC5B","file":"main.vue","sourcesContent":[".dv-decoration-6 {\n  width: 100%;\n  height: 100%;\n}\n.dv-decoration-6 svg {\n  transform-origin: left top;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$b = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$b = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$b = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$b = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$b, staticRenderFns: __vue_staticRenderFns__$b },
	    __vue_inject_styles__$b,
	    __vue_script__$b,
	    __vue_scope_id__$b,
	    __vue_is_functional_template__$b,
	    __vue_module_identifier__$b,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration6 (Vue) {
	  Vue.component(__vue_component__$b.name, __vue_component__$b);
	}

	//
	var script$a = {
	  name: 'DvDecoration7',
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    }
	  },

	  data() {
	    return {
	      defaultColor: ['#1dc1f5', '#1dc1f5'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$a = script$a;

	/* template */
	var __vue_render__$a = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    { staticClass: "dv-decoration-7" },
	    [
	      _c("svg", { attrs: { width: "21px", height: "20px" } }, [
	        _c("polyline", {
	          attrs: {
	            "stroke-width": "4",
	            fill: "transparent",
	            stroke: _vm.mergedColor[0],
	            points: "10, 0 19, 10 10, 20"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            "stroke-width": "2",
	            fill: "transparent",
	            stroke: _vm.mergedColor[1],
	            points: "2, 0 11, 10 2, 20"
	          }
	        })
	      ]),
	      _vm._v(" "),
	      _vm._t("default"),
	      _vm._v(" "),
	      _c("svg", { attrs: { width: "21px", height: "20px" } }, [
	        _c("polyline", {
	          attrs: {
	            "stroke-width": "4",
	            fill: "transparent",
	            stroke: _vm.mergedColor[0],
	            points: "11, 0 2, 10 11, 20"
	          }
	        }),
	        _vm._v(" "),
	        _c("polyline", {
	          attrs: {
	            "stroke-width": "2",
	            fill: "transparent",
	            stroke: _vm.mergedColor[1],
	            points: "19, 0 10, 10 19, 20"
	          }
	        })
	      ])
	    ],
	    2
	  )
	};
	var __vue_staticRenderFns__$a = [];
	__vue_render__$a._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$a = function (inject) {
	    if (!inject) return
	    inject("data-v-19d6f1fa_0", { source: ".dv-decoration-7 {\n  display: flex;\n  width: 100%;\n  height: 100%;\n  justify-content: center;\n  align-items: center;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,aAAa;EACb,WAAW;EACX,YAAY;EACZ,uBAAuB;EACvB,mBAAmB;AACrB","file":"main.vue","sourcesContent":[".dv-decoration-7 {\n  display: flex;\n  width: 100%;\n  height: 100%;\n  justify-content: center;\n  align-items: center;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$a = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$a = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$a = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$a = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$a, staticRenderFns: __vue_staticRenderFns__$a },
	    __vue_inject_styles__$a,
	    __vue_script__$a,
	    __vue_scope_id__$a,
	    __vue_is_functional_template__$a,
	    __vue_module_identifier__$a,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration7 (Vue) {
	  Vue.component(__vue_component__$a.name, __vue_component__$a);
	}

	//
	var script$9 = {
	  name: 'DvDecoration8',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    reverse: {
	      type: Boolean,
	      default: false
	    }
	  },

	  data() {
	    return {
	      ref: 'decoration-8',
	      defaultColor: ['#3f96a5', '#3f96a5'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    xPos(pos) {
	      const {
	        reverse,
	        width
	      } = this;
	      if (!reverse) return pos;
	      return width - pos;
	    },

	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$9 = script$9;

	/* template */
	var __vue_render__$9 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-8" }, [
	    _c("svg", { attrs: { width: _vm.width, height: _vm.height } }, [
	      _c("polyline", {
	        attrs: {
	          stroke: _vm.mergedColor[0],
	          "stroke-width": "2",
	          fill: "transparent",
	          points: _vm.xPos(0) + ", 0 " + _vm.xPos(30) + ", " + _vm.height / 2
	        }
	      }),
	      _vm._v(" "),
	      _c("polyline", {
	        attrs: {
	          stroke: _vm.mergedColor[0],
	          "stroke-width": "2",
	          fill: "transparent",
	          points:
	            _vm.xPos(20) +
	            ", 0 " +
	            _vm.xPos(50) +
	            ", " +
	            _vm.height / 2 +
	            " " +
	            _vm.xPos(_vm.width) +
	            ", " +
	            _vm.height / 2
	        }
	      }),
	      _vm._v(" "),
	      _c("polyline", {
	        attrs: {
	          stroke: _vm.mergedColor[1],
	          fill: "transparent",
	          "stroke-width": "3",
	          points:
	            _vm.xPos(0) +
	            ", " +
	            (_vm.height - 3) +
	            ", " +
	            _vm.xPos(200) +
	            ", " +
	            (_vm.height - 3)
	        }
	      })
	    ])
	  ])
	};
	var __vue_staticRenderFns__$9 = [];
	__vue_render__$9._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$9 = function (inject) {
	    if (!inject) return
	    inject("data-v-bc3f680c_0", { source: ".dv-decoration-8 {\n  display: flex;\n  width: 100%;\n  height: 100%;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,aAAa;EACb,WAAW;EACX,YAAY;AACd","file":"main.vue","sourcesContent":[".dv-decoration-8 {\n  display: flex;\n  width: 100%;\n  height: 100%;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$9 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$9 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$9 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$9 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$9, staticRenderFns: __vue_staticRenderFns__$9 },
	    __vue_inject_styles__$9,
	    __vue_script__$9,
	    __vue_scope_id__$9,
	    __vue_is_functional_template__$9,
	    __vue_module_identifier__$9,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration8 (Vue) {
	  Vue.component(__vue_component__$9.name, __vue_component__$9);
	}

	//
	var script$8 = {
	  name: 'DvDecoration9',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },
	    dur: {
	      type: Number,
	      default: 3
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'decoration-9',
	      polygonId: `decoration-9-polygon-${id}`,
	      svgWH: [100, 100],
	      svgScale: [1, 1],
	      defaultColor: ['rgba(3, 166, 224, 0.8)', 'rgba(3, 166, 224, 0.5)'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcScale
	      } = this;
	      calcScale();
	    },

	    calcScale() {
	      const {
	        width,
	        height,
	        svgWH
	      } = this;
	      const [w, h] = svgWH;
	      this.svgScale = [width / w, height / h];
	    },

	    onResize() {
	      const {
	        calcScale
	      } = this;
	      calcScale();
	    },

	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    },

	    fade
	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$8 = script$8;

	/* template */
	var __vue_render__$8 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    { ref: _vm.ref, staticClass: "dv-decoration-9" },
	    [
	      _c(
	        "svg",
	        {
	          style:
	            "transform:scale(" + _vm.svgScale[0] + "," + _vm.svgScale[1] + ");",
	          attrs: { width: _vm.svgWH[0] + "px", height: _vm.svgWH[1] + "px" }
	        },
	        [
	          _c("defs", [
	            _c("polygon", {
	              attrs: {
	                id: _vm.polygonId,
	                points: "15, 46.5, 21, 47.5, 21, 52.5, 15, 53.5"
	              }
	            })
	          ]),
	          _vm._v(" "),
	          _c(
	            "circle",
	            {
	              attrs: {
	                cx: "50",
	                cy: "50",
	                r: "45",
	                fill: "transparent",
	                stroke: _vm.mergedColor[1],
	                "stroke-width": "10",
	                "stroke-dasharray": "80, 100, 30, 100"
	              }
	            },
	            [
	              _c("animateTransform", {
	                attrs: {
	                  attributeName: "transform",
	                  type: "rotate",
	                  values: "0 50 50;360 50 50",
	                  dur: _vm.dur + "s",
	                  repeatCount: "indefinite"
	                }
	              })
	            ],
	            1
	          ),
	          _vm._v(" "),
	          _c(
	            "circle",
	            {
	              attrs: {
	                cx: "50",
	                cy: "50",
	                r: "45",
	                fill: "transparent",
	                stroke: _vm.mergedColor[0],
	                "stroke-width": "6",
	                "stroke-dasharray": "50, 66, 100, 66"
	              }
	            },
	            [
	              _c("animateTransform", {
	                attrs: {
	                  attributeName: "transform",
	                  type: "rotate",
	                  values: "0 50 50;-360 50 50",
	                  dur: _vm.dur + "s",
	                  repeatCount: "indefinite"
	                }
	              })
	            ],
	            1
	          ),
	          _vm._v(" "),
	          _c("circle", {
	            attrs: {
	              cx: "50",
	              cy: "50",
	              r: "38",
	              fill: "transparent",
	              stroke: _vm.fade(_vm.mergedColor[1] || _vm.defaultColor[1], 30),
	              "stroke-width": "1",
	              "stroke-dasharray": "5, 1"
	            }
	          }),
	          _vm._v(" "),
	          _vm._l(new Array(20).fill(0), function(foo, i) {
	            return _c(
	              "use",
	              {
	                key: i,
	                attrs: {
	                  "xlink:href": "#" + _vm.polygonId,
	                  stroke: _vm.mergedColor[1],
	                  fill: Math.random() > 0.4 ? "transparent" : _vm.mergedColor[0]
	                }
	              },
	              [
	                _c("animateTransform", {
	                  attrs: {
	                    attributeName: "transform",
	                    type: "rotate",
	                    values: "0 50 50;360 50 50",
	                    dur: _vm.dur + "s",
	                    begin: (i * _vm.dur) / 20 + "s",
	                    repeatCount: "indefinite"
	                  }
	                })
	              ],
	              1
	            )
	          }),
	          _vm._v(" "),
	          _c("circle", {
	            attrs: {
	              cx: "50",
	              cy: "50",
	              r: "26",
	              fill: "transparent",
	              stroke: _vm.fade(_vm.mergedColor[1] || _vm.defaultColor[1], 30),
	              "stroke-width": "1",
	              "stroke-dasharray": "5, 1"
	            }
	          })
	        ],
	        2
	      ),
	      _vm._v(" "),
	      _vm._t("default")
	    ],
	    2
	  )
	};
	var __vue_staticRenderFns__$8 = [];
	__vue_render__$8._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$8 = function (inject) {
	    if (!inject) return
	    inject("data-v-0857c2a2_0", { source: ".dv-decoration-9 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n}\n.dv-decoration-9 svg {\n  position: absolute;\n  left: 0px;\n  top: 0px;\n  transform-origin: left top;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,aAAa;EACb,mBAAmB;EACnB,uBAAuB;AACzB;AACA;EACE,kBAAkB;EAClB,SAAS;EACT,QAAQ;EACR,0BAA0B;AAC5B","file":"main.vue","sourcesContent":[".dv-decoration-9 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n}\n.dv-decoration-9 svg {\n  position: absolute;\n  left: 0px;\n  top: 0px;\n  transform-origin: left top;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$8 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$8 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$8 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$8 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$8, staticRenderFns: __vue_staticRenderFns__$8 },
	    __vue_inject_styles__$8,
	    __vue_script__$8,
	    __vue_scope_id__$8,
	    __vue_is_functional_template__$8,
	    __vue_module_identifier__$8,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration9 (Vue) {
	  Vue.component(__vue_component__$8.name, __vue_component__$8);
	}

	//
	var script$7 = {
	  name: 'DvDecoration10',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'decoration-10',
	      animationId1: `d10ani1${id}`,
	      animationId2: `d10ani2${id}`,
	      animationId3: `d10ani3${id}`,
	      animationId4: `d10ani4${id}`,
	      animationId5: `d10ani5${id}`,
	      animationId6: `d10ani6${id}`,
	      animationId7: `d10ani7${id}`,
	      defaultColor: ['#00c2ff', 'rgba(0, 194, 255, 0.3)'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    }

	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$7 = script$7;

	/* template */
	var __vue_render__$7 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-10" }, [
	    _c("svg", { attrs: { width: _vm.width, height: _vm.height } }, [
	      _c("polyline", {
	        attrs: {
	          stroke: _vm.mergedColor[1],
	          "stroke-width": "2",
	          points:
	            "0, " + _vm.height / 2 + " " + _vm.width + ", " + _vm.height / 2
	        }
	      }),
	      _vm._v(" "),
	      _c(
	        "polyline",
	        {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            "stroke-width": "2",
	            points:
	              "5, " +
	              _vm.height / 2 +
	              " " +
	              (_vm.width * 0.2 - 3) +
	              ", " +
	              _vm.height / 2,
	            "stroke-dasharray": "0, " + _vm.width * 0.2,
	            fill: "freeze"
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              id: _vm.animationId2,
	              attributeName: "stroke-dasharray",
	              values: "0, " + _vm.width * 0.2 + ";" + _vm.width * 0.2 + ", 0;",
	              dur: "3s",
	              begin: _vm.animationId1 + ".end",
	              fill: "freeze"
	            }
	          }),
	          _vm._v(" "),
	          _c("animate", {
	            attrs: {
	              attributeName: "stroke-dasharray",
	              values: _vm.width * 0.2 + ", 0;0, " + _vm.width * 0.2,
	              dur: "0.01s",
	              begin: _vm.animationId7 + ".end",
	              fill: "freeze"
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _c(
	        "polyline",
	        {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            "stroke-width": "2",
	            points:
	              _vm.width * 0.2 +
	              3 +
	              ", " +
	              _vm.height / 2 +
	              " " +
	              (_vm.width * 0.8 - 3) +
	              ", " +
	              _vm.height / 2,
	            "stroke-dasharray": "0, " + _vm.width * 0.6
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              id: _vm.animationId4,
	              attributeName: "stroke-dasharray",
	              values: "0, " + _vm.width * 0.6 + ";" + _vm.width * 0.6 + ", 0",
	              dur: "3s",
	              begin: _vm.animationId3 + ".end + 1s",
	              fill: "freeze"
	            }
	          }),
	          _vm._v(" "),
	          _c("animate", {
	            attrs: {
	              attributeName: "stroke-dasharray",
	              values: _vm.width * 0.6 + ", 0;0, " + _vm.width * 0.6,
	              dur: "0.01s",
	              begin: _vm.animationId7 + ".end",
	              fill: "freeze"
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _c(
	        "polyline",
	        {
	          attrs: {
	            stroke: _vm.mergedColor[0],
	            "stroke-width": "2",
	            points:
	              _vm.width * 0.8 +
	              3 +
	              ", " +
	              _vm.height / 2 +
	              " " +
	              (_vm.width - 5) +
	              ", " +
	              _vm.height / 2,
	            "stroke-dasharray": "0, " + _vm.width * 0.2
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              id: _vm.animationId6,
	              attributeName: "stroke-dasharray",
	              values: "0, " + _vm.width * 0.2 + ";" + _vm.width * 0.2 + ", 0",
	              dur: "3s",
	              begin: _vm.animationId5 + ".end + 1s",
	              fill: "freeze"
	            }
	          }),
	          _vm._v(" "),
	          _c("animate", {
	            attrs: {
	              attributeName: "stroke-dasharray",
	              values: _vm.width * 0.2 + ", 0;0, " + _vm.width * 0.3,
	              dur: "0.01s",
	              begin: _vm.animationId7 + ".end",
	              fill: "freeze"
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _c(
	        "circle",
	        {
	          attrs: {
	            cx: "2",
	            cy: _vm.height / 2,
	            r: "2",
	            fill: _vm.mergedColor[1]
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              id: _vm.animationId1,
	              attributeName: "fill",
	              values: _vm.mergedColor[1] + ";" + _vm.mergedColor[0],
	              begin: "0s;" + _vm.animationId7 + ".end",
	              dur: "0.3s",
	              fill: "freeze"
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _c(
	        "circle",
	        {
	          attrs: {
	            cx: _vm.width * 0.2,
	            cy: _vm.height / 2,
	            r: "2",
	            fill: _vm.mergedColor[1]
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              id: _vm.animationId3,
	              attributeName: "fill",
	              values: _vm.mergedColor[1] + ";" + _vm.mergedColor[0],
	              begin: _vm.animationId2 + ".end",
	              dur: "0.3s",
	              fill: "freeze"
	            }
	          }),
	          _vm._v(" "),
	          _c("animate", {
	            attrs: {
	              attributeName: "fill",
	              values: _vm.mergedColor[1] + ";" + _vm.mergedColor[1],
	              dur: "0.01s",
	              begin: _vm.animationId7 + ".end",
	              fill: "freeze"
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _c(
	        "circle",
	        {
	          attrs: {
	            cx: _vm.width * 0.8,
	            cy: _vm.height / 2,
	            r: "2",
	            fill: _vm.mergedColor[1]
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              id: _vm.animationId5,
	              attributeName: "fill",
	              values: _vm.mergedColor[1] + ";" + _vm.mergedColor[0],
	              begin: _vm.animationId4 + ".end",
	              dur: "0.3s",
	              fill: "freeze"
	            }
	          }),
	          _vm._v(" "),
	          _c("animate", {
	            attrs: {
	              attributeName: "fill",
	              values: _vm.mergedColor[1] + ";" + _vm.mergedColor[1],
	              dur: "0.01s",
	              begin: _vm.animationId7 + ".end",
	              fill: "freeze"
	            }
	          })
	        ]
	      ),
	      _vm._v(" "),
	      _c(
	        "circle",
	        {
	          attrs: {
	            cx: _vm.width - 2,
	            cy: _vm.height / 2,
	            r: "2",
	            fill: _vm.mergedColor[1]
	          }
	        },
	        [
	          _c("animate", {
	            attrs: {
	              id: _vm.animationId7,
	              attributeName: "fill",
	              values: _vm.mergedColor[1] + ";" + _vm.mergedColor[0],
	              begin: _vm.animationId6 + ".end",
	              dur: "0.3s",
	              fill: "freeze"
	            }
	          }),
	          _vm._v(" "),
	          _c("animate", {
	            attrs: {
	              attributeName: "fill",
	              values: _vm.mergedColor[1] + ";" + _vm.mergedColor[1],
	              dur: "0.01s",
	              begin: _vm.animationId7 + ".end",
	              fill: "freeze"
	            }
	          })
	        ]
	      )
	    ])
	  ])
	};
	var __vue_staticRenderFns__$7 = [];
	__vue_render__$7._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$7 = function (inject) {
	    if (!inject) return
	    inject("data-v-1da36c04_0", { source: ".dv-decoration-10 {\n  width: 100%;\n  height: 100%;\n  display: flex;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,WAAW;EACX,YAAY;EACZ,aAAa;AACf","file":"main.vue","sourcesContent":[".dv-decoration-10 {\n  width: 100%;\n  height: 100%;\n  display: flex;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$7 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$7 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$7 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$7 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$7, staticRenderFns: __vue_staticRenderFns__$7 },
	    __vue_inject_styles__$7,
	    __vue_script__$7,
	    __vue_scope_id__$7,
	    __vue_is_functional_template__$7,
	    __vue_module_identifier__$7,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration10 (Vue) {
	  Vue.component(__vue_component__$7.name, __vue_component__$7);
	}

	//
	var script$6 = {
	  name: 'DvDecoration11',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    }
	  },

	  data() {
	    return {
	      ref: 'decoration-11',
	      defaultColor: ['#1a98fc', '#2cf7fe'],
	      mergedColor: []
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  methods: {
	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    },

	    fade
	  },

	  mounted() {
	    const {
	      mergeColor
	    } = this;
	    mergeColor();
	  }

	};

	/* script */
	const __vue_script__$6 = script$6;

	/* template */
	var __vue_render__$6 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-11" }, [
	    _c("svg", { attrs: { width: _vm.width, height: _vm.height } }, [
	      _c("polygon", {
	        attrs: {
	          fill: _vm.fade(_vm.mergedColor[1] || _vm.defaultColor[1], 10),
	          stroke: _vm.mergedColor[1],
	          points: "20 10, 25 4, 55 4 60 10"
	        }
	      }),
	      _vm._v(" "),
	      _c("polygon", {
	        attrs: {
	          fill: _vm.fade(_vm.mergedColor[1] || _vm.defaultColor[1], 10),
	          stroke: _vm.mergedColor[1],
	          points:
	            "20 " +
	            (_vm.height - 10) +
	            ", 25 " +
	            (_vm.height - 4) +
	            ", 55 " +
	            (_vm.height - 4) +
	            " 60 " +
	            (_vm.height - 10)
	        }
	      }),
	      _vm._v(" "),
	      _c("polygon", {
	        attrs: {
	          fill: _vm.fade(_vm.mergedColor[1] || _vm.defaultColor[1], 10),
	          stroke: _vm.mergedColor[1],
	          points:
	            _vm.width -
	            20 +
	            " 10, " +
	            (_vm.width - 25) +
	            " 4, " +
	            (_vm.width - 55) +
	            " 4 " +
	            (_vm.width - 60) +
	            " 10"
	        }
	      }),
	      _vm._v(" "),
	      _c("polygon", {
	        attrs: {
	          fill: _vm.fade(_vm.mergedColor[1] || _vm.defaultColor[1], 10),
	          stroke: _vm.mergedColor[1],
	          points:
	            _vm.width -
	            20 +
	            " " +
	            (_vm.height - 10) +
	            ", " +
	            (_vm.width - 25) +
	            " " +
	            (_vm.height - 4) +
	            ", " +
	            (_vm.width - 55) +
	            " " +
	            (_vm.height - 4) +
	            " " +
	            (_vm.width - 60) +
	            " " +
	            (_vm.height - 10)
	        }
	      }),
	      _vm._v(" "),
	      _c("polygon", {
	        attrs: {
	          fill: _vm.fade(_vm.mergedColor[0] || _vm.defaultColor[0], 20),
	          stroke: _vm.mergedColor[0],
	          points:
	            "\n        20 10, 5 " +
	            _vm.height / 2 +
	            " 20 " +
	            (_vm.height - 10) +
	            "\n        " +
	            (_vm.width - 20) +
	            " " +
	            (_vm.height - 10) +
	            " " +
	            (_vm.width - 5) +
	            " " +
	            _vm.height / 2 +
	            " " +
	            (_vm.width - 20) +
	            " 10\n      "
	        }
	      }),
	      _vm._v(" "),
	      _c("polyline", {
	        attrs: {
	          fill: "transparent",
	          stroke: _vm.fade(_vm.mergedColor[0] || _vm.defaultColor[0], 70),
	          points: "25 18, 15 " + _vm.height / 2 + " 25 " + (_vm.height - 18)
	        }
	      }),
	      _vm._v(" "),
	      _c("polyline", {
	        attrs: {
	          fill: "transparent",
	          stroke: _vm.fade(_vm.mergedColor[0] || _vm.defaultColor[0], 70),
	          points:
	            _vm.width -
	            25 +
	            " 18, " +
	            (_vm.width - 15) +
	            " " +
	            _vm.height / 2 +
	            " " +
	            (_vm.width - 25) +
	            " " +
	            (_vm.height - 18)
	        }
	      })
	    ]),
	    _vm._v(" "),
	    _c("div", { staticClass: "decoration-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$6 = [];
	__vue_render__$6._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$6 = function (inject) {
	    if (!inject) return
	    inject("data-v-f6466864_0", { source: ".dv-decoration-11 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  display: flex;\n}\n.dv-decoration-11 .decoration-content {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,aAAa;AACf;AACA;EACE,kBAAkB;EAClB,QAAQ;EACR,SAAS;EACT,WAAW;EACX,YAAY;EACZ,aAAa;EACb,mBAAmB;EACnB,uBAAuB;AACzB","file":"main.vue","sourcesContent":[".dv-decoration-11 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  display: flex;\n}\n.dv-decoration-11 .decoration-content {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$6 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$6 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$6 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$6 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$6, staticRenderFns: __vue_staticRenderFns__$6 },
	    __vue_inject_styles__$6,
	    __vue_script__$6,
	    __vue_scope_id__$6,
	    __vue_is_functional_template__$6,
	    __vue_module_identifier__$6,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration11 (Vue) {
	  Vue.component(__vue_component__$6.name, __vue_component__$6);
	}

	//
	var script$5 = {
	  name: 'DvDecoration12',
	  mixins: [autoResize],
	  props: {
	    color: {
	      type: Array,
	      default: () => []
	    },

	    /**
	     * @description Scan animation dur
	     */
	    scanDur: {
	      type: Number,
	      default: 3
	    },

	    /**
	     * @description Halo animation dur
	     */
	    haloDur: {
	      type: Number,
	      default: 2
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'decoration-12',
	      gId: `decoration-12-g-${id}`,
	      gradientId: `decoration-12-gradient-${id}`,
	      defaultColor: ['#2783ce', '#2cf7fe'],
	      mergedColor: [],
	      pathD: [],
	      pathColor: [],
	      circleR: [],
	      splitLinePoints: [],
	      arcD: [],
	      segment: 30,
	      sectorAngle: Math.PI / 3,
	      ringNum: 3,
	      ringWidth: 1,
	      showSplitLine: true
	    };
	  },

	  watch: {
	    color() {
	      const {
	        mergeColor
	      } = this;
	      mergeColor();
	    }

	  },
	  computed: {
	    x() {
	      const {
	        width
	      } = this;
	      return width / 2;
	    },

	    y() {
	      const {
	        height
	      } = this;
	      return height / 2;
	    }

	  },
	  methods: {
	    init() {
	      const {
	        mergeColor,
	        calcPathD,
	        calcPathColor,
	        calcCircleR,
	        calcSplitLinePoints,
	        calcArcD
	      } = this;
	      mergeColor();
	      calcPathD();
	      calcPathColor();
	      calcCircleR();
	      calcSplitLinePoints();
	      calcArcD();
	    },

	    mergeColor() {
	      const {
	        color,
	        defaultColor
	      } = this;
	      this.mergedColor = deepMerge(deepClone(defaultColor, true), color || []);
	    },

	    calcPathD() {
	      const {
	        x,
	        y,
	        width,
	        segment,
	        sectorAngle
	      } = this;
	      const startAngle = -Math.PI / 2;
	      const angleGap = sectorAngle / segment;
	      const r = width / 4;
	      let lastEndPoints = getCircleRadianPoint(x, y, r, startAngle);
	      this.pathD = new Array(segment).fill('').map((_, i) => {
	        const endPoints = getCircleRadianPoint(x, y, r, startAngle - (i + 1) * angleGap).map(_ => _.toFixed(5));
	        const d = `M${lastEndPoints.join(',')} A${r}, ${r} 0 0 0 ${endPoints.join(',')}`;
	        lastEndPoints = endPoints;
	        return d;
	      });
	    },

	    calcPathColor() {
	      const {
	        mergedColor: [color],
	        segment
	      } = this;
	      const colorGap = 100 / (segment - 1);
	      this.pathColor = new Array(segment).fill(color).map((_, i) => fade(color, 100 - i * colorGap));
	    },

	    calcCircleR() {
	      const {
	        segment,
	        ringNum,
	        width,
	        ringWidth
	      } = this;
	      const radiusGap = (width / 2 - ringWidth / 2) / ringNum;
	      this.circleR = new Array(ringNum).fill(0).map((_, i) => radiusGap * (i + 1));
	    },

	    calcSplitLinePoints() {
	      const {
	        x,
	        y,
	        width
	      } = this;
	      const angleGap = Math.PI / 6;
	      const r = width / 2;
	      this.splitLinePoints = new Array(6).fill('').map((_, i) => {
	        const startAngle = angleGap * (i + 1);
	        const endAngle = startAngle + Math.PI;
	        const startPoint = getCircleRadianPoint(x, y, r, startAngle);
	        const endPoint = getCircleRadianPoint(x, y, r, endAngle);
	        return `${startPoint.join(',')} ${endPoint.join(',')}`;
	      });
	    },

	    calcArcD() {
	      const {
	        x,
	        y,
	        width
	      } = this;
	      const angleGap = Math.PI / 6;
	      const r = width / 2 - 1;
	      this.arcD = new Array(4).fill('').map((_, i) => {
	        const startAngle = angleGap * (3 * i + 1);
	        const endAngle = startAngle + angleGap;
	        const startPoint = getCircleRadianPoint(x, y, r, startAngle);
	        const endPoint = getCircleRadianPoint(x, y, r, endAngle);
	        return `M${startPoint.join(',')} A${x}, ${y} 0 0 1 ${endPoint.join(',')}`;
	      });
	    },

	    afterAutoResizeMixinInit() {
	      const {
	        init
	      } = this;
	      init();
	    },

	    fade
	  }
	};

	/* script */
	const __vue_script__$5 = script$5;

	/* template */
	var __vue_render__$5 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-decoration-12" }, [
	    _c(
	      "svg",
	      { attrs: { width: _vm.width, height: _vm.height } },
	      [
	        _c(
	          "defs",
	          [
	            _c(
	              "g",
	              { attrs: { id: _vm.gId } },
	              _vm._l(_vm.pathD, function(d, i) {
	                return _c("path", {
	                  key: d,
	                  attrs: {
	                    stroke: _vm.pathColor[i],
	                    "stroke-width": _vm.width / 2,
	                    fill: "transparent",
	                    d: d
	                  }
	                })
	              }),
	              0
	            ),
	            _vm._v(" "),
	            _c(
	              "radialGradient",
	              { attrs: { id: _vm.gradientId, cx: "50%", cy: "50%", r: "50%" } },
	              [
	                _c("stop", {
	                  attrs: {
	                    offset: "0%",
	                    "stop-color": "transparent",
	                    "stop-opacity": "1"
	                  }
	                }),
	                _vm._v(" "),
	                _c("stop", {
	                  attrs: {
	                    offset: "100%",
	                    "stop-color": _vm.fade(
	                      _vm.mergedColor[1] || _vm.defaultColor[1],
	                      30
	                    ),
	                    "stop-opacity": "1"
	                  }
	                })
	              ],
	              1
	            )
	          ],
	          1
	        ),
	        _vm._v(" "),
	        _vm._l(_vm.circleR, function(r) {
	          return _c("circle", {
	            key: r,
	            attrs: {
	              r: r,
	              cx: _vm.x,
	              cy: _vm.y,
	              stroke: _vm.mergedColor[1],
	              "stroke-width": 0.5,
	              fill: "transparent"
	            }
	          })
	        }),
	        _vm._v(" "),
	        _c(
	          "circle",
	          {
	            attrs: {
	              r: "1",
	              cx: _vm.x,
	              cy: _vm.y,
	              stroke: "transparent",
	              fill: "url(#" + _vm.gradientId + ")"
	            }
	          },
	          [
	            _c("animate", {
	              attrs: {
	                attributeName: "r",
	                values: "1;" + _vm.width / 2,
	                dur: _vm.haloDur + "s",
	                repeatCount: "indefinite"
	              }
	            }),
	            _vm._v(" "),
	            _c("animate", {
	              attrs: {
	                attributeName: "opacity",
	                values: "1;0",
	                dur: _vm.haloDur + "s",
	                repeatCount: "indefinite"
	              }
	            })
	          ]
	        ),
	        _vm._v(" "),
	        _c("circle", {
	          attrs: { r: "2", cx: _vm.x, cy: _vm.y, fill: _vm.mergedColor[1] }
	        }),
	        _vm._v(" "),
	        _vm.showSplitLine
	          ? _c(
	              "g",
	              _vm._l(_vm.splitLinePoints, function(p) {
	                return _c("polyline", {
	                  key: p,
	                  attrs: {
	                    points: p,
	                    stroke: _vm.mergedColor[1],
	                    "stroke-width": 0.5,
	                    opacity: "0.5"
	                  }
	                })
	              }),
	              0
	            )
	          : _vm._e(),
	        _vm._v(" "),
	        _vm._l(_vm.arcD, function(d) {
	          return _c("path", {
	            key: d,
	            attrs: {
	              d: d,
	              stroke: _vm.mergedColor[1],
	              "stroke-width": "2",
	              fill: "transparent"
	            }
	          })
	        }),
	        _vm._v(" "),
	        _c(
	          "use",
	          { attrs: { "xlink:href": "#" + _vm.gId } },
	          [
	            _c("animateTransform", {
	              attrs: {
	                attributeName: "transform",
	                type: "rotate",
	                values:
	                  "0, " + _vm.x + " " + _vm.y + ";360, " + _vm.x + " " + _vm.y,
	                dur: _vm.scanDur + "s",
	                repeatCount: "indefinite"
	              }
	            })
	          ],
	          1
	        )
	      ],
	      2
	    ),
	    _vm._v(" "),
	    _c("div", { staticClass: "decoration-content" }, [_vm._t("default")], 2)
	  ])
	};
	var __vue_staticRenderFns__$5 = [];
	__vue_render__$5._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$5 = function (inject) {
	    if (!inject) return
	    inject("data-v-651ce2f8_0", { source: ".dv-decoration-12 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  display: flex;\n}\n.dv-decoration-12 .decoration-content {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,aAAa;AACf;AACA;EACE,kBAAkB;EAClB,QAAQ;EACR,SAAS;EACT,WAAW;EACX,YAAY;EACZ,aAAa;EACb,mBAAmB;EACnB,uBAAuB;AACzB","file":"main.vue","sourcesContent":[".dv-decoration-12 {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  display: flex;\n}\n.dv-decoration-12 .decoration-content {\n  position: absolute;\n  top: 0px;\n  left: 0px;\n  width: 100%;\n  height: 100%;\n  display: flex;\n  align-items: center;\n  justify-content: center;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$5 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$5 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$5 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$5 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$5, staticRenderFns: __vue_staticRenderFns__$5 },
	    __vue_inject_styles__$5,
	    __vue_script__$5,
	    __vue_scope_id__$5,
	    __vue_is_functional_template__$5,
	    __vue_module_identifier__$5,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function decoration12 (Vue) {
	  Vue.component(__vue_component__$5.name, __vue_component__$5);
	}

	//
	var script$4 = {
	  name: 'DvPercentPond',
	  props: {
	    config: {
	      type: Object,
	      default: () => ({})
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      gradientId1: `percent-pond-gradientId1-${id}`,
	      gradientId2: `percent-pond-gradientId2-${id}`,
	      width: 0,
	      height: 0,
	      defaultConfig: {
	        /**
	         * @description Value
	         * @type {Number}
	         * @default value = 0
	         */
	        value: 0,

	        /**
	         * @description Colors (hex|rgb|rgba|color keywords)
	         * @type {Array<String>}
	         * @default colors = ['#00BAFF', '#3DE7C9']
	         * @example colors = ['#000', 'rgb(0, 0, 0)', 'rgba(0, 0, 0, 1)', 'red']
	         */
	        colors: ['#3DE7C9', '#00BAFF'],

	        /**
	         * @description Border width
	         * @type {Number}
	         * @default borderWidth = 3
	         */
	        borderWidth: 3,

	        /**
	         * @description Gap between border and pond
	         * @type {Number}
	         * @default borderGap = 3
	         */
	        borderGap: 3,

	        /**
	         * @description Line dash
	         * @type {Array<Number>}
	         * @default lineDash = [5, 1]
	         */
	        lineDash: [5, 1],

	        /**
	         * @description Text color
	         * @type {String}
	         * @default textColor = '#fff'
	         */
	        textColor: '#fff',

	        /**
	         * @description Border radius
	         * @type {Number}
	         * @default borderRadius = 5
	         */
	        borderRadius: 5,

	        /**
	         * @description Local Gradient
	         * @type {Boolean}
	         * @default localGradient = false
	         * @example localGradient = false | true
	         */
	        localGradient: false,

	        /**
	         * @description Formatter
	         * @type {String}
	         * @default formatter = '{value}%'
	         */
	        formatter: '{value}%'
	      },
	      mergedConfig: null
	    };
	  },

	  computed: {
	    rectWidth() {
	      const {
	        mergedConfig,
	        width
	      } = this;
	      if (!mergedConfig) return 0;
	      const {
	        borderWidth
	      } = mergedConfig;
	      return width - borderWidth;
	    },

	    rectHeight() {
	      const {
	        mergedConfig,
	        height
	      } = this;
	      if (!mergedConfig) return 0;
	      const {
	        borderWidth
	      } = mergedConfig;
	      return height - borderWidth;
	    },

	    points() {
	      const {
	        mergedConfig,
	        width,
	        height
	      } = this;
	      const halfHeight = height / 2;
	      if (!mergedConfig) return `0, ${halfHeight} 0, ${halfHeight}`;
	      const {
	        borderWidth,
	        borderGap,
	        value
	      } = mergedConfig;
	      const polylineLength = (width - (borderWidth + borderGap) * 2) / 100 * value;
	      return `
        ${borderWidth + borderGap}, ${halfHeight}
        ${borderWidth + borderGap + polylineLength}, ${halfHeight + 0.001}
      `;
	    },

	    polylineWidth() {
	      const {
	        mergedConfig,
	        height
	      } = this;
	      if (!mergedConfig) return 0;
	      const {
	        borderWidth,
	        borderGap
	      } = mergedConfig;
	      return height - (borderWidth + borderGap) * 2;
	    },

	    linearGradient() {
	      const {
	        mergedConfig
	      } = this;
	      if (!mergedConfig) return [];
	      const {
	        colors
	      } = mergedConfig;
	      const colorNum = colors.length;
	      const colorOffsetGap = 100 / (colorNum - 1);
	      return colors.map((c, i) => [colorOffsetGap * i, c]);
	    },

	    polylineGradient() {
	      const {
	        gradientId1,
	        gradientId2,
	        mergedConfig
	      } = this;
	      if (!mergedConfig) return gradientId2;
	      if (mergedConfig.localGradient) return gradientId1;
	      return gradientId2;
	    },

	    gradient2XPos() {
	      const {
	        mergedConfig
	      } = this;
	      if (!mergedConfig) return '100%';
	      const {
	        value
	      } = mergedConfig;
	      return `${200 - value}%`;
	    },

	    details() {
	      const {
	        mergedConfig
	      } = this;
	      if (!mergedConfig) return '';
	      const {
	        value,
	        formatter
	      } = mergedConfig;
	      return formatter.replace('{value}', value);
	    }

	  },
	  watch: {
	    config() {
	      const {
	        mergeConfig
	      } = this;
	      mergeConfig();
	    }

	  },
	  methods: {
	    async init() {
	      const {
	        initWH,
	        config,
	        mergeConfig
	      } = this;
	      await initWH();
	      if (!config) return;
	      mergeConfig();
	    },

	    async initWH() {
	      const {
	        $nextTick,
	        $refs
	      } = this;
	      await $nextTick();
	      const {
	        clientWidth,
	        clientHeight
	      } = $refs['percent-pond'];
	      this.width = clientWidth;
	      this.height = clientHeight;
	    },

	    mergeConfig() {
	      const {
	        config,
	        defaultConfig
	      } = this;
	      this.mergedConfig = deepMerge(deepClone(defaultConfig, true), config || {});
	    }

	  },

	  mounted() {
	    const {
	      init
	    } = this;
	    init();
	  }

	};

	/* script */
	const __vue_script__$4 = script$4;

	/* template */
	var __vue_render__$4 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: "percent-pond", staticClass: "dv-percent-pond" }, [
	    _c("svg", [
	      _c(
	        "defs",
	        [
	          _c(
	            "linearGradient",
	            {
	              attrs: {
	                id: _vm.gradientId1,
	                x1: "0%",
	                y1: "0%",
	                x2: "100%",
	                y2: "0%"
	              }
	            },
	            _vm._l(_vm.linearGradient, function(lc) {
	              return _c("stop", {
	                key: lc[0],
	                attrs: { offset: lc[0] + "%", "stop-color": lc[1] }
	              })
	            }),
	            1
	          ),
	          _vm._v(" "),
	          _c(
	            "linearGradient",
	            {
	              attrs: {
	                id: _vm.gradientId2,
	                x1: "0%",
	                y1: "0%",
	                x2: _vm.gradient2XPos,
	                y2: "0%"
	              }
	            },
	            _vm._l(_vm.linearGradient, function(lc) {
	              return _c("stop", {
	                key: lc[0],
	                attrs: { offset: lc[0] + "%", "stop-color": lc[1] }
	              })
	            }),
	            1
	          )
	        ],
	        1
	      ),
	      _vm._v(" "),
	      _c("rect", {
	        attrs: {
	          x: _vm.mergedConfig ? _vm.mergedConfig.borderWidth / 2 : "0",
	          y: _vm.mergedConfig ? _vm.mergedConfig.borderWidth / 2 : "0",
	          rx: _vm.mergedConfig ? _vm.mergedConfig.borderRadius : "0",
	          ry: _vm.mergedConfig ? _vm.mergedConfig.borderRadius : "0",
	          fill: "transparent",
	          "stroke-width": _vm.mergedConfig ? _vm.mergedConfig.borderWidth : "0",
	          stroke: "url(#" + _vm.gradientId1 + ")",
	          width: _vm.rectWidth > 0 ? _vm.rectWidth : 0,
	          height: _vm.rectHeight > 0 ? _vm.rectHeight : 0
	        }
	      }),
	      _vm._v(" "),
	      _c("polyline", {
	        attrs: {
	          "stroke-width": _vm.polylineWidth,
	          "stroke-dasharray": _vm.mergedConfig
	            ? _vm.mergedConfig.lineDash.join(",")
	            : "0",
	          stroke: "url(#" + _vm.polylineGradient + ")",
	          points: _vm.points
	        }
	      }),
	      _vm._v(" "),
	      _c(
	        "text",
	        {
	          attrs: {
	            stroke: _vm.mergedConfig ? _vm.mergedConfig.textColor : "#fff",
	            fill: _vm.mergedConfig ? _vm.mergedConfig.textColor : "#fff",
	            x: _vm.width / 2,
	            y: _vm.height / 2
	          }
	        },
	        [_vm._v("\n      " + _vm._s(_vm.details) + "\n    ")]
	      )
	    ])
	  ])
	};
	var __vue_staticRenderFns__$4 = [];
	__vue_render__$4._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$4 = function (inject) {
	    if (!inject) return
	    inject("data-v-869f31f8_0", { source: ".dv-percent-pond {\n  position: relative;\n  display: flex;\n  flex-direction: column;\n}\n.dv-percent-pond svg {\n  position: absolute;\n  left: 0px;\n  top: 0px;\n  width: 100%;\n  height: 100%;\n}\n.dv-percent-pond polyline {\n  transition: all 0.3s;\n}\n.dv-percent-pond text {\n  font-size: 25px;\n  font-weight: bold;\n  text-anchor: middle;\n  dominant-baseline: middle;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,aAAa;EACb,sBAAsB;AACxB;AACA;EACE,kBAAkB;EAClB,SAAS;EACT,QAAQ;EACR,WAAW;EACX,YAAY;AACd;AACA;EACE,oBAAoB;AACtB;AACA;EACE,eAAe;EACf,iBAAiB;EACjB,mBAAmB;EACnB,yBAAyB;AAC3B","file":"main.vue","sourcesContent":[".dv-percent-pond {\n  position: relative;\n  display: flex;\n  flex-direction: column;\n}\n.dv-percent-pond svg {\n  position: absolute;\n  left: 0px;\n  top: 0px;\n  width: 100%;\n  height: 100%;\n}\n.dv-percent-pond polyline {\n  transition: all 0.3s;\n}\n.dv-percent-pond text {\n  font-size: 25px;\n  font-weight: bold;\n  text-anchor: middle;\n  dominant-baseline: middle;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$4 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$4 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$4 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$4 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$4, staticRenderFns: __vue_staticRenderFns__$4 },
	    __vue_inject_styles__$4,
	    __vue_script__$4,
	    __vue_scope_id__$4,
	    __vue_is_functional_template__$4,
	    __vue_module_identifier__$4,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function percentPond (Vue) {
	  Vue.component(__vue_component__$4.name, __vue_component__$4);
	}

	//
	var script$3 = {
	  name: 'DvFlylineChart',
	  mixins: [autoResize],
	  props: {
	    config: {
	      type: Object,
	      default: () => ({})
	    },
	    dev: {
	      type: Boolean,
	      default: false
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'dv-flyline-chart',
	      unique: Math.random(),
	      maskId: `flyline-mask-id-${id}`,
	      maskCircleId: `mask-circle-id-${id}`,
	      gradientId: `gradient-id-${id}`,
	      gradient2Id: `gradient2-id-${id}`,
	      defaultConfig: {
	        /**
	         * @description Flyline chart center point
	         * @type {Array<Number>}
	         * @default centerPoint = [0, 0]
	         */
	        centerPoint: [0, 0],

	        /**
	         * @description Flyline start points
	         * @type {Array<Array<Number>>}
	         * @default points = []
	         * @example points = [[10, 10], [100, 100]]
	         */
	        points: [],

	        /**
	         * @description Flyline width
	         * @type {Number}
	         * @default lineWidth = 1
	         */
	        lineWidth: 1,

	        /**
	         * @description Orbit color
	         * @type {String}
	         * @default orbitColor = 'rgba(103, 224, 227, .2)'
	         */
	        orbitColor: 'rgba(103, 224, 227, .2)',

	        /**
	         * @description Flyline color
	         * @type {String}
	         * @default orbitColor = '#ffde93'
	         */
	        flylineColor: '#ffde93',

	        /**
	         * @description K value
	         * @type {Number}
	         * @default k = -0.5
	         * @example k = -1 ~ 1
	         */
	        k: -0.5,

	        /**
	         * @description Flyline curvature
	         * @type {Number}
	         * @default curvature = 5
	         */
	        curvature: 5,

	        /**
	         * @description Flyline radius
	         * @type {Number}
	         * @default flylineRadius = 100
	         */
	        flylineRadius: 100,

	        /**
	         * @description Flyline animation duration
	         * @type {Array<Number>}
	         * @default duration = [20, 30]
	         */
	        duration: [20, 30],

	        /**
	         * @description Relative points position
	         * @type {Boolean}
	         * @default relative = true
	         */
	        relative: true,

	        /**
	         * @description Back ground image url
	         * @type {String}
	         * @default bgImgUrl = ''
	         * @example bgImgUrl = './img/bg.jpg'
	         */
	        bgImgUrl: '',

	        /**
	         * @description Text configuration
	         * @type {Object}
	         */
	        text: {
	          /**
	           * @description Text offset
	           * @type {Array<Number>}
	           * @default offset = [0, 15]
	           */
	          offset: [0, 15],

	          /**
	           * @description Text color
	           * @type {String}
	           * @default color = '#ffdb5c'
	           */
	          color: '#ffdb5c',

	          /**
	           * @description Text font size
	           * @type {Number}
	           * @default fontSize = 12
	           */
	          fontSize: 12
	        },

	        /**
	         * @description Halo configuration
	         * @type {Object}
	         */
	        halo: {
	          /**
	           * @description Weather to show halo
	           * @type {Boolean}
	           * @default show = true
	           * @example show = true | false
	           */
	          show: true,

	          /**
	           * @description Halo animation duration (10 = 1s)
	           * @type {Number}
	           * @default duration = 30
	           */
	          duration: 30,

	          /**
	           * @description Halo color
	           * @type {String}
	           * @default color = '#fb7293'
	           */
	          color: '#fb7293',

	          /**
	           * @description Halo max radius
	           * @type {Number}
	           * @default radius = 120
	           */
	          radius: 120
	        },

	        /**
	         * @description Center point img configuration
	         * @type {Object}
	         */
	        centerPointImg: {
	          /**
	           * @description Center point img width
	           * @type {Number}
	           * @default width = 40
	           */
	          width: 40,

	          /**
	           * @description Center point img height
	           * @type {Number}
	           * @default height = 40
	           */
	          height: 40,

	          /**
	           * @description Center point img url
	           * @type {String}
	           * @default url = ''
	           */
	          url: ''
	        },

	        /**
	         * @description Points img configuration
	         * @type {Object}
	         * @default radius = 120
	         */
	        pointsImg: {
	          /**
	           * @description Points img width
	           * @type {Number}
	           * @default width = 15
	           */
	          width: 15,

	          /**
	           * @description Points img height
	           * @type {Number}
	           * @default height = 15
	           */
	          height: 15,

	          /**
	           * @description Points img url
	           * @type {String}
	           * @default url = ''
	           */
	          url: ''
	        }
	      },
	      mergedConfig: null,
	      paths: [],
	      lengths: [],
	      times: [],
	      texts: []
	    };
	  },

	  watch: {
	    config() {
	      const {
	        calcData
	      } = this;
	      calcData();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcData
	      } = this;
	      calcData();
	    },

	    onResize() {
	      const {
	        calcData
	      } = this;
	      calcData();
	    },

	    async calcData() {
	      const {
	        mergeConfig,
	        createFlylinePaths,
	        calcLineLengths
	      } = this;
	      mergeConfig();
	      createFlylinePaths();
	      await calcLineLengths();
	      const {
	        calcTimes,
	        calcTexts
	      } = this;
	      calcTimes();
	      calcTexts();
	    },

	    mergeConfig() {
	      let {
	        config,
	        defaultConfig
	      } = this;
	      const mergedConfig = deepMerge(deepClone(defaultConfig, true), config || {});
	      const {
	        points
	      } = mergedConfig;
	      mergedConfig.points = points.map(item => {
	        if (item instanceof Array) {
	          return {
	            position: item,
	            text: ''
	          };
	        }

	        return item;
	      });
	      this.mergedConfig = mergedConfig;
	    },

	    createFlylinePaths() {
	      const {
	        getPath,
	        mergedConfig,
	        width,
	        height
	      } = this;
	      let {
	        centerPoint,
	        points,
	        relative
	      } = mergedConfig;
	      points = points.map(({
	        position
	      }) => position);

	      if (relative) {
	        centerPoint = [width * centerPoint[0], height * centerPoint[1]];
	        points = points.map(([x, y]) => [width * x, height * y]);
	      }

	      this.paths = points.map(point => getPath(centerPoint, point));
	    },

	    getPath(center, point) {
	      const {
	        getControlPoint
	      } = this;
	      const controlPoint = getControlPoint(center, point);
	      return [point, controlPoint, center];
	    },

	    getControlPoint([sx, sy], [ex, ey]) {
	      const {
	        getKLinePointByx,
	        mergedConfig
	      } = this;
	      const {
	        curvature,
	        k
	      } = mergedConfig;
	      const [mx, my] = [(sx + ex) / 2, (sy + ey) / 2];
	      const distance = getPointDistance([sx, sy], [ex, ey]);
	      const targetLength = distance / curvature;
	      const disDived = targetLength / 2;
	      let [dx, dy] = [mx, my];

	      do {
	        dx += disDived;
	        dy = getKLinePointByx(k, [mx, my], dx)[1];
	      } while (getPointDistance([mx, my], [dx, dy]) < targetLength);

	      return [dx, dy];
	    },

	    getKLinePointByx(k, [lx, ly], x) {
	      const y = ly - k * lx + k * x;
	      return [x, y];
	    },

	    async calcLineLengths() {
	      const {
	        $nextTick,
	        paths,
	        $refs
	      } = this;
	      await $nextTick();
	      this.lengths = paths.map((foo, i) => $refs[`path${i}`][0].getTotalLength());
	    },

	    calcTimes() {
	      const {
	        duration,
	        points
	      } = this.mergedConfig;
	      this.times = points.map(foo => randomExtend(...duration) / 10);
	    },

	    calcTexts() {
	      const {
	        points
	      } = this.mergedConfig;
	      this.texts = points.map(({
	        text
	      }) => text);
	    },

	    consoleClickPos({
	      offsetX,
	      offsetY
	    }) {
	      const {
	        width,
	        height,
	        dev
	      } = this;
	      if (!dev) return;
	      const relativeX = (offsetX / width).toFixed(2);
	      const relativeY = (offsetY / height).toFixed(2);
	      console.warn(`dv-flyline-chart DEV: \n Click Position is [${offsetX}, ${offsetY}] \n Relative Position is [${relativeX}, ${relativeY}]`);
	    }

	  }
	};

	/* script */
	const __vue_script__$3 = script$3;

	/* template */
	var __vue_render__$3 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    {
	      ref: "dv-flyline-chart",
	      staticClass: "dv-flyline-chart",
	      style:
	        "background-image: url(" +
	        (_vm.mergedConfig ? _vm.mergedConfig.bgImgUrl : "") +
	        ")",
	      on: { click: _vm.consoleClickPos }
	    },
	    [
	      _vm.mergedConfig
	        ? _c(
	            "svg",
	            { attrs: { width: _vm.width, height: _vm.height } },
	            [
	              _c(
	                "defs",
	                [
	                  _c(
	                    "radialGradient",
	                    {
	                      attrs: {
	                        id: _vm.gradientId,
	                        cx: "50%",
	                        cy: "50%",
	                        r: "50%"
	                      }
	                    },
	                    [
	                      _c("stop", {
	                        attrs: {
	                          offset: "0%",
	                          "stop-color": "#fff",
	                          "stop-opacity": "1"
	                        }
	                      }),
	                      _vm._v(" "),
	                      _c("stop", {
	                        attrs: {
	                          offset: "100%",
	                          "stop-color": "#fff",
	                          "stop-opacity": "0"
	                        }
	                      })
	                    ],
	                    1
	                  ),
	                  _vm._v(" "),
	                  _c(
	                    "radialGradient",
	                    {
	                      attrs: {
	                        id: _vm.gradient2Id,
	                        cx: "50%",
	                        cy: "50%",
	                        r: "50%"
	                      }
	                    },
	                    [
	                      _c("stop", {
	                        attrs: {
	                          offset: "0%",
	                          "stop-color": "#fff",
	                          "stop-opacity": "0"
	                        }
	                      }),
	                      _vm._v(" "),
	                      _c("stop", {
	                        attrs: {
	                          offset: "100%",
	                          "stop-color": "#fff",
	                          "stop-opacity": "1"
	                        }
	                      })
	                    ],
	                    1
	                  ),
	                  _vm._v(" "),
	                  _vm.paths[0]
	                    ? _c(
	                        "circle",
	                        {
	                          attrs: {
	                            id: "circle" + _vm.paths[0].toString(),
	                            cx: _vm.paths[0][2][0],
	                            cy: _vm.paths[0][2][1]
	                          }
	                        },
	                        [
	                          _c("animate", {
	                            attrs: {
	                              attributeName: "r",
	                              values: "1;" + _vm.mergedConfig.halo.radius,
	                              dur: _vm.mergedConfig.halo.duration / 10 + "s",
	                              repeatCount: "indefinite"
	                            }
	                          }),
	                          _vm._v(" "),
	                          _c("animate", {
	                            attrs: {
	                              attributeName: "opacity",
	                              values: "1;0",
	                              dur: _vm.mergedConfig.halo.duration / 10 + "s",
	                              repeatCount: "indefinite"
	                            }
	                          })
	                        ]
	                      )
	                    : _vm._e()
	                ],
	                1
	              ),
	              _vm._v(" "),
	              _vm.paths[0]
	                ? _c("image", {
	                    attrs: {
	                      "xlink:href": _vm.mergedConfig.centerPointImg.url,
	                      width: _vm.mergedConfig.centerPointImg.width,
	                      height: _vm.mergedConfig.centerPointImg.height,
	                      x:
	                        _vm.paths[0][2][0] -
	                        _vm.mergedConfig.centerPointImg.width / 2,
	                      y:
	                        _vm.paths[0][2][1] -
	                        _vm.mergedConfig.centerPointImg.height / 2
	                    }
	                  })
	                : _vm._e(),
	              _vm._v(" "),
	              _c(
	                "mask",
	                { attrs: { id: "maskhalo" + _vm.paths[0].toString() } },
	                [
	                  _vm.paths[0]
	                    ? _c("use", {
	                        attrs: {
	                          "xlink:href": "#circle" + _vm.paths[0].toString(),
	                          fill: "url(#" + _vm.gradient2Id + ")"
	                        }
	                      })
	                    : _vm._e()
	                ]
	              ),
	              _vm._v(" "),
	              _vm.paths[0] && _vm.mergedConfig.halo.show
	                ? _c("use", {
	                    attrs: {
	                      "xlink:href": "#circle" + _vm.paths[0].toString(),
	                      fill: _vm.mergedConfig.halo.color,
	                      mask: "url(#maskhalo" + _vm.paths[0].toString() + ")"
	                    }
	                  })
	                : _vm._e(),
	              _vm._v(" "),
	              _vm._l(_vm.paths, function(path, i) {
	                return _c("g", { key: i }, [
	                  _c("defs", [
	                    _c("path", {
	                      ref: "path" + i,
	                      refInFor: true,
	                      attrs: {
	                        id: "path" + path.toString(),
	                        d:
	                          "M" +
	                          path[0].toString() +
	                          " Q" +
	                          path[1].toString() +
	                          " " +
	                          path[2].toString(),
	                        fill: "transparent"
	                      }
	                    })
	                  ]),
	                  _vm._v(" "),
	                  _c("use", {
	                    attrs: {
	                      "xlink:href": "#path" + path.toString(),
	                      "stroke-width": _vm.mergedConfig.lineWidth,
	                      stroke: _vm.mergedConfig.orbitColor
	                    }
	                  }),
	                  _vm._v(" "),
	                  _vm.lengths[i]
	                    ? _c(
	                        "use",
	                        {
	                          attrs: {
	                            "xlink:href": "#path" + path.toString(),
	                            "stroke-width": _vm.mergedConfig.lineWidth,
	                            stroke: _vm.mergedConfig.flylineColor,
	                            mask:
	                              "url(#mask" + _vm.unique + path.toString() + ")"
	                          }
	                        },
	                        [
	                          _c("animate", {
	                            attrs: {
	                              attributeName: "stroke-dasharray",
	                              from: "0, " + _vm.lengths[i],
	                              to: _vm.lengths[i] + ", 0",
	                              dur: _vm.times[i] || 0,
	                              repeatCount: "indefinite"
	                            }
	                          })
	                        ]
	                      )
	                    : _vm._e(),
	                  _vm._v(" "),
	                  _c(
	                    "mask",
	                    { attrs: { id: "mask" + _vm.unique + path.toString() } },
	                    [
	                      _c(
	                        "circle",
	                        {
	                          attrs: {
	                            cx: "0",
	                            cy: "0",
	                            r: _vm.mergedConfig.flylineRadius,
	                            fill: "url(#" + _vm.gradientId + ")"
	                          }
	                        },
	                        [
	                          _c("animateMotion", {
	                            attrs: {
	                              dur: _vm.times[i] || 0,
	                              path:
	                                "M" +
	                                path[0].toString() +
	                                " Q" +
	                                path[1].toString() +
	                                " " +
	                                path[2].toString(),
	                              rotate: "auto",
	                              repeatCount: "indefinite"
	                            }
	                          })
	                        ],
	                        1
	                      )
	                    ]
	                  ),
	                  _vm._v(" "),
	                  _c("image", {
	                    attrs: {
	                      "xlink:href": _vm.mergedConfig.pointsImg.url,
	                      width: _vm.mergedConfig.pointsImg.width,
	                      height: _vm.mergedConfig.pointsImg.height,
	                      x: path[0][0] - _vm.mergedConfig.pointsImg.width / 2,
	                      y: path[0][1] - _vm.mergedConfig.pointsImg.height / 2
	                    }
	                  }),
	                  _vm._v(" "),
	                  _c(
	                    "text",
	                    {
	                      style:
	                        "fontSize:" + _vm.mergedConfig.text.fontSize + "px;",
	                      attrs: {
	                        fill: _vm.mergedConfig.text.color,
	                        x: path[0][0] + _vm.mergedConfig.text.offset[0],
	                        y: path[0][1] + _vm.mergedConfig.text.offset[1]
	                      }
	                    },
	                    [_vm._v("\n        " + _vm._s(_vm.texts[i]) + "\n      ")]
	                  )
	                ])
	              })
	            ],
	            2
	          )
	        : _vm._e()
	    ]
	  )
	};
	var __vue_staticRenderFns__$3 = [];
	__vue_render__$3._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$3 = function (inject) {
	    if (!inject) return
	    inject("data-v-f77adb6c_0", { source: ".dv-flyline-chart {\n  display: flex;\n  flex-direction: column;\n  background-size: 100% 100%;\n}\n.dv-flyline-chart polyline {\n  transition: all 0.3s;\n}\n.dv-flyline-chart text {\n  text-anchor: middle;\n  dominant-baseline: middle;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,aAAa;EACb,sBAAsB;EACtB,0BAA0B;AAC5B;AACA;EACE,oBAAoB;AACtB;AACA;EACE,mBAAmB;EACnB,yBAAyB;AAC3B","file":"main.vue","sourcesContent":[".dv-flyline-chart {\n  display: flex;\n  flex-direction: column;\n  background-size: 100% 100%;\n}\n.dv-flyline-chart polyline {\n  transition: all 0.3s;\n}\n.dv-flyline-chart text {\n  text-anchor: middle;\n  dominant-baseline: middle;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$3 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$3 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$3 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$3 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$3, staticRenderFns: __vue_staticRenderFns__$3 },
	    __vue_inject_styles__$3,
	    __vue_script__$3,
	    __vue_scope_id__$3,
	    __vue_is_functional_template__$3,
	    __vue_module_identifier__$3,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function flylineChart (Vue) {
	  Vue.component(__vue_component__$3.name, __vue_component__$3);
	}

	//
	var script$2 = {
	  name: 'DvFlylineChartEnhanced',
	  mixins: [autoResize],
	  props: {
	    config: {
	      type: Object,
	      default: () => ({})
	    },
	    dev: {
	      type: Boolean,
	      default: false
	    }
	  },

	  data() {
	    const id = uuid();
	    return {
	      ref: 'dv-flyline-chart-enhanced',
	      unique: Math.random(),
	      flylineGradientId: `flyline-gradient-id-${id}`,
	      haloGradientId: `halo-gradient-id-${id}`,

	      /**
	       * @description Type Declaration
	       *
	       * interface Halo {
	       *    show?: boolean
	       *    duration?: [number, number]
	       *    color?: string
	       *    radius?: number
	       * }
	       *
	       * interface Text {
	       *    show?: boolean
	       *    offset?: [number, number]
	       *    color?: string
	       *    fontSize?: number
	       * }
	       *
	       * interface Icon {
	       *    show?: boolean
	       *    src?: string
	       *    width?: number
	       *    height?: number
	       * }
	       *
	       * interface Point {
	       *    name: string
	       *    coordinate: [number, number]
	       *    halo?: Halo
	       *    text?: Text
	       *    icon?: Icon
	       * }
	       *
	       * interface Line {
	       *    width?: number
	       *    color?: string
	       *    orbitColor?: string
	       *    duration?: [number, number]
	       *    radius?: string
	       * }
	       *
	       * interface Flyline extends Line {
	       *    source: string
	       *    target: string
	       * }
	       *
	       * interface FlylineWithPath extends Flyline {
	       *    d: string
	       *    path: [[number, number], [number, number], [number, number]]
	       *    key: string
	       * }
	       */
	      defaultConfig: {
	        /**
	         * @description Flyline chart points
	         * @type {Point[]}
	         * @default points = []
	         */
	        points: [],

	        /**
	         * @description Lines
	         * @type {Flyline[]}
	         * @default lines = []
	         */
	        lines: [],

	        /**
	         * @description Global halo configuration
	         * @type {Halo}
	         */
	        halo: {
	          /**
	           * @description Whether to show halo
	           * @type {Boolean}
	           * @default show = false
	           */
	          show: false,

	          /**
	           * @description Halo animation duration (1s = 10)
	           * @type {[number, number]}
	           */
	          duration: [20, 30],

	          /**
	           * @description Halo color
	           * @type {String}
	           * @default color = '#fb7293'
	           */
	          color: '#fb7293',

	          /**
	           * @description Halo radius
	           * @type {Number}
	           * @default radius = 120
	           */
	          radius: 120
	        },

	        /**
	         * @description Global text configuration
	         * @type {Text}
	         */
	        text: {
	          /**
	           * @description Whether to show text
	           * @type {Boolean}
	           * @default show = false
	           */
	          show: false,

	          /**
	           * @description Text offset
	           * @type {[number, number]}
	           * @default offset = [0, 15]
	           */
	          offset: [0, 15],

	          /**
	           * @description Text color
	           * @type {String}
	           * @default color = '#ffdb5c'
	           */
	          color: '#ffdb5c',

	          /**
	           * @description Text font size
	           * @type {Number}
	           * @default fontSize = 12
	           */
	          fontSize: 12
	        },

	        /**
	         * @description Global icon configuration
	         * @type {Icon}
	         */
	        icon: {
	          /**
	           * @description Whether to show icon
	           * @type {Boolean}
	           * @default show = false
	           */
	          show: false,

	          /**
	           * @description Icon src
	           * @type {String}
	           * @default src = ''
	           */
	          src: '',

	          /**
	           * @description Icon width
	           * @type {Number}
	           * @default width = 15
	           */
	          width: 15,

	          /**
	           * @description Icon height
	           * @type {Number}
	           * @default width = 15
	           */
	          height: 15
	        },

	        /**
	         * @description Global line configuration
	         * @type {Line}
	         */
	        line: {
	          /**
	           * @description Line width
	           * @type {Number}
	           * @default width = 1
	           */
	          width: 1,

	          /**
	           * @description Flyline color
	           * @type {String}
	           * @default color = '#ffde93'
	           */
	          color: '#ffde93',

	          /**
	           * @description Orbit color
	           * @type {String}
	           * @default orbitColor = 'rgba(103, 224, 227, .2)'
	           */
	          orbitColor: 'rgba(103, 224, 227, .2)',

	          /**
	           * @description Flyline animation duration
	           * @type {[number, number]}
	           * @default duration = [20, 30]
	           */
	          duration: [20, 30],

	          /**
	           * @description Flyline radius
	           * @type {Number}
	           * @default radius = 100
	           */
	          radius: 100
	        },

	        /**
	         * @description Back ground image url
	         * @type {String}
	         * @default bgImgSrc = ''
	         */
	        bgImgSrc: '',

	        /**
	         * @description K value
	         * @type {Number}
	         * @default k = -0.5
	         * @example k = -1 ~ 1
	         */
	        k: -0.5,

	        /**
	         * @description Flyline curvature
	         * @type {Number}
	         * @default curvature = 5
	         */
	        curvature: 5,

	        /**
	         * @description Relative points position
	         * @type {Boolean}
	         * @default relative = true
	         */
	        relative: true
	      },

	      /**
	       * @description Fly line data
	       * @type {FlylineWithPath[]}
	       * @default flylines = []
	       */
	      flylines: [],

	      /**
	       * @description Fly line lengths
	       * @type {Number[]}
	       * @default flylineLengths = []
	       */
	      flylineLengths: [],

	      /**
	       * @description Fly line points
	       * @default flylinePoints = []
	       */
	      flylinePoints: [],
	      mergedConfig: null
	    };
	  },

	  watch: {
	    config() {
	      const {
	        calcData
	      } = this;
	      calcData();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcData
	      } = this;
	      calcData();
	    },

	    onResize() {
	      const {
	        calcData
	      } = this;
	      calcData();
	    },

	    async calcData() {
	      const {
	        mergeConfig,
	        calcflylinePoints,
	        calcLinePaths
	      } = this;
	      mergeConfig();
	      calcflylinePoints();
	      calcLinePaths();
	      const {
	        calcLineLengths
	      } = this;
	      await calcLineLengths();
	    },

	    mergeConfig() {
	      let {
	        config,
	        defaultConfig
	      } = this;
	      const mergedConfig = deepMerge(deepClone(defaultConfig, true), config || {});
	      const {
	        points,
	        lines,
	        halo,
	        text,
	        icon,
	        line
	      } = mergedConfig;
	      mergedConfig.points = points.map(item => {
	        item.halo = deepMerge(deepClone(halo, true), item.halo || {});
	        item.text = deepMerge(deepClone(text, true), item.text || {});
	        item.icon = deepMerge(deepClone(icon, true), item.icon || {});
	        return item;
	      });
	      mergedConfig.lines = lines.map(item => {
	        return deepMerge(deepClone(line, true), item);
	      });
	      this.mergedConfig = mergedConfig;
	    },

	    calcflylinePoints() {
	      const {
	        mergedConfig,
	        width,
	        height
	      } = this;
	      const {
	        relative,
	        points
	      } = mergedConfig;
	      this.flylinePoints = points.map((item, i) => {
	        const {
	          coordinate: [x, y],
	          halo,
	          icon,
	          text
	        } = item;
	        if (relative) item.coordinate = [x * width, y * height];
	        item.halo.time = randomExtend(...halo.duration) / 10;
	        const {
	          width: iw,
	          height: ih
	        } = icon;
	        item.icon.x = item.coordinate[0] - iw / 2;
	        item.icon.y = item.coordinate[1] - ih / 2;
	        const [ox, oy] = text.offset;
	        item.text.x = item.coordinate[0] + ox;
	        item.text.y = item.coordinate[1] + oy;
	        item.key = `${item.coordinate.toString()}${i}`;
	        return item;
	      });
	    },

	    calcLinePaths() {
	      const {
	        getPath,
	        mergedConfig
	      } = this;
	      const {
	        points,
	        lines
	      } = mergedConfig;
	      this.flylines = lines.map(item => {
	        const {
	          source,
	          target,
	          duration
	        } = item;
	        const sourcePoint = points.find(({
	          name
	        }) => name === source).coordinate;
	        const targetPoint = points.find(({
	          name
	        }) => name === target).coordinate;
	        const path = getPath(sourcePoint, targetPoint).map(item => item.map(v => parseFloat(v.toFixed(10))));
	        const d = `M${path[0].toString()} Q${path[1].toString()} ${path[2].toString()}`;
	        const key = `path${path.toString()}`;
	        const time = randomExtend(...duration) / 10;
	        return { ...item,
	          path,
	          key,
	          d,
	          time
	        };
	      });
	    },

	    getPath(start, end) {
	      const {
	        getControlPoint
	      } = this;
	      const controlPoint = getControlPoint(start, end);
	      return [start, controlPoint, end];
	    },

	    getControlPoint([sx, sy], [ex, ey]) {
	      const {
	        getKLinePointByx,
	        mergedConfig
	      } = this;
	      const {
	        curvature,
	        k
	      } = mergedConfig;
	      const [mx, my] = [(sx + ex) / 2, (sy + ey) / 2];
	      const distance = getPointDistance([sx, sy], [ex, ey]);
	      const targetLength = distance / curvature;
	      const disDived = targetLength / 2;
	      let [dx, dy] = [mx, my];

	      do {
	        dx += disDived;
	        dy = getKLinePointByx(k, [mx, my], dx)[1];
	      } while (getPointDistance([mx, my], [dx, dy]) < targetLength);

	      return [dx, dy];
	    },

	    getKLinePointByx(k, [lx, ly], x) {
	      const y = ly - k * lx + k * x;
	      return [x, y];
	    },

	    async calcLineLengths() {
	      const {
	        $nextTick,
	        flylines,
	        $refs
	      } = this;
	      await $nextTick();
	      this.flylineLengths = flylines.map(({
	        key
	      }) => $refs[key][0].getTotalLength());
	    },

	    consoleClickPos({
	      offsetX,
	      offsetY
	    }) {
	      const {
	        width,
	        height,
	        dev
	      } = this;
	      if (!dev) return;
	      const relativeX = (offsetX / width).toFixed(2);
	      const relativeY = (offsetY / height).toFixed(2);
	      console.warn(`dv-flyline-chart-enhanced DEV: \n Click Position is [${offsetX}, ${offsetY}] \n Relative Position is [${relativeX}, ${relativeY}]`);
	    }

	  }
	};

	/* script */
	const __vue_script__$2 = script$2;

	/* template */
	var __vue_render__$2 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    {
	      ref: _vm.ref,
	      staticClass: "dv-flyline-chart-enhanced",
	      style:
	        "background-image: url(" +
	        (_vm.mergedConfig ? _vm.mergedConfig.bgImgSrc : "") +
	        ")",
	      on: { click: _vm.consoleClickPos }
	    },
	    [
	      _vm.flylines.length
	        ? _c(
	            "svg",
	            { attrs: { width: _vm.width, height: _vm.height } },
	            [
	              _c(
	                "defs",
	                [
	                  _c(
	                    "radialGradient",
	                    {
	                      attrs: {
	                        id: _vm.flylineGradientId,
	                        cx: "50%",
	                        cy: "50%",
	                        r: "50%"
	                      }
	                    },
	                    [
	                      _c("stop", {
	                        attrs: {
	                          offset: "0%",
	                          "stop-color": "#fff",
	                          "stop-opacity": "1"
	                        }
	                      }),
	                      _vm._v(" "),
	                      _c("stop", {
	                        attrs: {
	                          offset: "100%",
	                          "stop-color": "#fff",
	                          "stop-opacity": "0"
	                        }
	                      })
	                    ],
	                    1
	                  ),
	                  _vm._v(" "),
	                  _c(
	                    "radialGradient",
	                    {
	                      attrs: {
	                        id: _vm.haloGradientId,
	                        cx: "50%",
	                        cy: "50%",
	                        r: "50%"
	                      }
	                    },
	                    [
	                      _c("stop", {
	                        attrs: {
	                          offset: "0%",
	                          "stop-color": "#fff",
	                          "stop-opacity": "0"
	                        }
	                      }),
	                      _vm._v(" "),
	                      _c("stop", {
	                        attrs: {
	                          offset: "100%",
	                          "stop-color": "#fff",
	                          "stop-opacity": "1"
	                        }
	                      })
	                    ],
	                    1
	                  )
	                ],
	                1
	              ),
	              _vm._v(" "),
	              _vm._l(_vm.flylinePoints, function(point) {
	                return _c("g", { key: point.key + Math.random() }, [
	                  _c("defs", [
	                    point.halo.show
	                      ? _c(
	                          "circle",
	                          {
	                            attrs: {
	                              id: "halo" + _vm.unique + point.key,
	                              cx: point.coordinate[0],
	                              cy: point.coordinate[1]
	                            }
	                          },
	                          [
	                            _c("animate", {
	                              attrs: {
	                                attributeName: "r",
	                                values: "1;" + point.halo.radius,
	                                dur: point.halo.time + "s",
	                                repeatCount: "indefinite"
	                              }
	                            }),
	                            _vm._v(" "),
	                            _c("animate", {
	                              attrs: {
	                                attributeName: "opacity",
	                                values: "1;0",
	                                dur: point.halo.time + "s",
	                                repeatCount: "indefinite"
	                              }
	                            })
	                          ]
	                        )
	                      : _vm._e()
	                  ]),
	                  _vm._v(" "),
	                  _c(
	                    "mask",
	                    { attrs: { id: "mask" + _vm.unique + point.key } },
	                    [
	                      point.halo.show
	                        ? _c("use", {
	                            attrs: {
	                              "xlink:href": "#halo" + _vm.unique + point.key,
	                              fill: "url(#" + _vm.haloGradientId + ")"
	                            }
	                          })
	                        : _vm._e()
	                    ]
	                  ),
	                  _vm._v(" "),
	                  point.halo.show
	                    ? _c("use", {
	                        attrs: {
	                          "xlink:href": "#halo" + _vm.unique + point.key,
	                          fill: point.halo.color,
	                          mask: "url(#mask" + _vm.unique + point.key + ")"
	                        }
	                      })
	                    : _vm._e(),
	                  _vm._v(" "),
	                  point.icon.show
	                    ? _c("image", {
	                        attrs: {
	                          "xlink:href": point.icon.src,
	                          width: point.icon.width,
	                          height: point.icon.height,
	                          x: point.icon.x,
	                          y: point.icon.y
	                        }
	                      })
	                    : _vm._e(),
	                  _vm._v(" "),
	                  point.text.show
	                    ? _c(
	                        "text",
	                        {
	                          style:
	                            "fontSize:" +
	                            point.text.fontSize +
	                            "px;color:" +
	                            point.text.color,
	                          attrs: {
	                            fill: point.text.color,
	                            x: point.text.x,
	                            y: point.text.y
	                          }
	                        },
	                        [_vm._v("\n        " + _vm._s(point.name) + "\n      ")]
	                      )
	                    : _vm._e()
	                ])
	              }),
	              _vm._v(" "),
	              _vm._l(_vm.flylines, function(line, i) {
	                return _c("g", { key: line.key + Math.random() }, [
	                  _c("defs", [
	                    _c("path", {
	                      ref: line.key,
	                      refInFor: true,
	                      attrs: { id: line.key, d: line.d, fill: "transparent" }
	                    })
	                  ]),
	                  _vm._v(" "),
	                  _c("use", {
	                    attrs: {
	                      "xlink:href": "#" + line.key,
	                      "stroke-width": line.width,
	                      stroke: line.orbitColor
	                    }
	                  }),
	                  _vm._v(" "),
	                  _c(
	                    "mask",
	                    { attrs: { id: "mask" + _vm.unique + line.key } },
	                    [
	                      _c(
	                        "circle",
	                        {
	                          attrs: {
	                            cx: "0",
	                            cy: "0",
	                            r: line.radius,
	                            fill: "url(#" + _vm.flylineGradientId + ")"
	                          }
	                        },
	                        [
	                          _c("animateMotion", {
	                            attrs: {
	                              dur: line.time,
	                              path: line.d,
	                              rotate: "auto",
	                              repeatCount: "indefinite"
	                            }
	                          })
	                        ],
	                        1
	                      )
	                    ]
	                  ),
	                  _vm._v(" "),
	                  _vm.flylineLengths[i]
	                    ? _c(
	                        "use",
	                        {
	                          attrs: {
	                            "xlink:href": "#" + line.key,
	                            "stroke-width": line.width,
	                            stroke: line.color,
	                            mask: "url(#mask" + _vm.unique + line.key + ")"
	                          }
	                        },
	                        [
	                          _c("animate", {
	                            attrs: {
	                              attributeName: "stroke-dasharray",
	                              from: "0, " + _vm.flylineLengths[i],
	                              to: _vm.flylineLengths[i] + ", 0",
	                              dur: line.time,
	                              repeatCount: "indefinite"
	                            }
	                          })
	                        ]
	                      )
	                    : _vm._e()
	                ])
	              })
	            ],
	            2
	          )
	        : _vm._e()
	    ]
	  )
	};
	var __vue_staticRenderFns__$2 = [];
	__vue_render__$2._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$2 = function (inject) {
	    if (!inject) return
	    inject("data-v-16f1f13e_0", { source: ".dv-flyline-chart-enhanced {\n  display: flex;\n  flex-direction: column;\n  background-size: 100% 100%;\n}\n.dv-flyline-chart-enhanced text {\n  text-anchor: middle;\n  dominant-baseline: middle;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,aAAa;EACb,sBAAsB;EACtB,0BAA0B;AAC5B;AACA;EACE,mBAAmB;EACnB,yBAAyB;AAC3B","file":"main.vue","sourcesContent":[".dv-flyline-chart-enhanced {\n  display: flex;\n  flex-direction: column;\n  background-size: 100% 100%;\n}\n.dv-flyline-chart-enhanced text {\n  text-anchor: middle;\n  dominant-baseline: middle;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$2 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$2 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$2 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$2 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$2, staticRenderFns: __vue_staticRenderFns__$2 },
	    __vue_inject_styles__$2,
	    __vue_script__$2,
	    __vue_scope_id__$2,
	    __vue_is_functional_template__$2,
	    __vue_module_identifier__$2,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function flylineChartEnhanced (Vue) {
	  Vue.component(__vue_component__$2.name, __vue_component__$2);
	}

	//
	var script$1 = {
	  name: 'DvScrollBoard',
	  mixins: [autoResize],
	  props: {
	    config: {
	      type: Object,
	      default: () => ({})
	    }
	  },

	  data() {
	    return {
	      ref: 'scroll-board',
	      defaultConfig: {
	        /**
	         * @description Board header
	         * @type {Array<String>}
	         * @default header = []
	         * @example header = ['column1', 'column2', 'column3']
	         */
	        header: [],

	        /**
	         * @description Board data
	         * @type {Array<Array>}
	         * @default data = []
	         */
	        data: [],

	        /**
	         * @description Row num
	         * @type {Number}
	         * @default rowNum = 5
	         */
	        rowNum: 5,

	        /**
	         * @description Header background color
	         * @type {String}
	         * @default headerBGC = '#00BAFF'
	         */
	        headerBGC: '#00BAFF',

	        /**
	         * @description Odd row background color
	         * @type {String}
	         * @default oddRowBGC = '#003B51'
	         */
	        oddRowBGC: '#003B51',

	        /**
	         * @description Even row background color
	         * @type {String}
	         * @default evenRowBGC = '#003B51'
	         */
	        evenRowBGC: '#0A2732',

	        /**
	         * @description Scroll wait time
	         * @type {Number}
	         * @default waitTime = 2000
	         */
	        waitTime: 2000,

	        /**
	         * @description Header height
	         * @type {Number}
	         * @default headerHeight = 35
	         */
	        headerHeight: 35,

	        /**
	         * @description Column width
	         * @type {Array<Number>}
	         * @default columnWidth = []
	         */
	        columnWidth: [],

	        /**
	         * @description Column align
	         * @type {Array<String>}
	         * @default align = []
	         * @example align = ['left', 'center', 'right']
	         */
	        align: [],

	        /**
	         * @description Show index
	         * @type {Boolean}
	         * @default index = false
	         */
	        index: false,

	        /**
	         * @description index Header
	         * @type {String}
	         * @default indexHeader = '#'
	         */
	        indexHeader: '#',

	        /**
	         * @description Carousel type
	         * @type {String}
	         * @default carousel = 'single'
	         * @example carousel = 'single' | 'page'
	         */
	        carousel: 'single',

	        /**
	         * @description Pause scroll when mouse hovered
	         * @type {Boolean}
	         * @default hoverPause = true
	         * @example hoverPause = true | false
	         */
	        hoverPause: true
	      },
	      mergedConfig: null,
	      header: [],
	      rowsData: [],
	      rows: [],
	      widths: [],
	      heights: [],
	      avgHeight: 0,
	      aligns: [],
	      animationIndex: 0,
	      animationHandler: '',
	      updater: 0,
	      needCalc: false
	    };
	  },

	  watch: {
	    config() {
	      const {
	        stopAnimation,
	        calcData
	      } = this;
	      stopAnimation();
	      this.animationIndex = 0;
	      calcData();
	    }

	  },
	  methods: {
	    handleHover(enter, ri, ci, row, ceil) {
	      const {
	        mergedConfig,
	        emitEvent,
	        stopAnimation,
	        animation
	      } = this;
	      if (enter) emitEvent('mouseover', ri, ci, row, ceil);
	      if (!mergedConfig.hoverPause) return;

	      if (enter) {
	        stopAnimation();
	      } else {
	        animation(true);
	      }
	    },

	    afterAutoResizeMixinInit() {
	      const {
	        calcData
	      } = this;
	      calcData();
	    },

	    onResize() {
	      const {
	        mergedConfig,
	        calcWidths,
	        calcHeights
	      } = this;
	      if (!mergedConfig) return;
	      calcWidths();
	      calcHeights();
	    },

	    calcData() {
	      const {
	        mergeConfig,
	        calcHeaderData,
	        calcRowsData
	      } = this;
	      mergeConfig();
	      calcHeaderData();
	      calcRowsData();
	      const {
	        calcWidths,
	        calcHeights,
	        calcAligns
	      } = this;
	      calcWidths();
	      calcHeights();
	      calcAligns();
	      const {
	        animation
	      } = this;
	      animation(true);
	    },

	    mergeConfig() {
	      let {
	        config,
	        defaultConfig
	      } = this;
	      this.mergedConfig = deepMerge(deepClone(defaultConfig, true), config || {});
	    },

	    calcHeaderData() {
	      let {
	        header,
	        index,
	        indexHeader
	      } = this.mergedConfig;

	      if (!header.length) {
	        this.header = [];
	        return;
	      }

	      header = [...header];
	      if (index) header.unshift(indexHeader);
	      this.header = header;
	    },

	    calcRowsData() {
	      let {
	        data,
	        index,
	        headerBGC,
	        rowNum
	      } = this.mergedConfig;

	      if (index) {
	        data = data.map((row, i) => {
	          row = [...row];
	          const indexTag = `<span class="index" style="background-color: ${headerBGC};">${i + 1}</span>`;
	          row.unshift(indexTag);
	          return row;
	        });
	      }

	      data = data.map((ceils, i) => ({
	        ceils,
	        rowIndex: i
	      }));
	      const rowLength = data.length;

	      if (rowLength > rowNum && rowLength < 2 * rowNum) {
	        data = [...data, ...data];
	      }

	      data = data.map((d, i) => ({ ...d,
	        scroll: i
	      }));
	      this.rowsData = data;
	      this.rows = data;
	    },

	    calcWidths() {
	      const {
	        width,
	        mergedConfig,
	        rowsData
	      } = this;
	      const {
	        columnWidth,
	        header
	      } = mergedConfig;
	      const usedWidth = columnWidth.reduce((all, w) => all + w, 0);
	      let columnNum = 0;

	      if (rowsData[0]) {
	        columnNum = rowsData[0].ceils.length;
	      } else if (header.length) {
	        columnNum = header.length;
	      }

	      const avgWidth = (width - usedWidth) / (columnNum - columnWidth.length);
	      const widths = new Array(columnNum).fill(avgWidth);
	      this.widths = deepMerge(widths, columnWidth);
	    },

	    calcHeights(onresize = false) {
	      const {
	        height,
	        mergedConfig,
	        header
	      } = this;
	      const {
	        headerHeight,
	        rowNum,
	        data
	      } = mergedConfig;
	      let allHeight = height;
	      if (header.length) allHeight -= headerHeight;
	      const avgHeight = allHeight / rowNum;
	      this.avgHeight = avgHeight;
	      if (!onresize) this.heights = new Array(data.length).fill(avgHeight);
	    },

	    calcAligns() {
	      const {
	        header,
	        mergedConfig
	      } = this;
	      const columnNum = header.length;
	      let aligns = new Array(columnNum).fill('left');
	      const {
	        align
	      } = mergedConfig;
	      this.aligns = deepMerge(aligns, align);
	    },

	    async animation(start = false) {
	      const {
	        needCalc,
	        calcHeights,
	        calcRowsData
	      } = this;

	      if (needCalc) {
	        calcRowsData();
	        calcHeights();
	        this.needCalc = false;
	      }

	      let {
	        avgHeight,
	        animationIndex,
	        mergedConfig,
	        rowsData,
	        animation,
	        updater
	      } = this;
	      const {
	        waitTime,
	        carousel,
	        rowNum
	      } = mergedConfig;
	      const rowLength = rowsData.length;
	      if (rowNum >= rowLength) return;

	      if (start) {
	        await new Promise(resolve => setTimeout(resolve, waitTime));
	        if (updater !== this.updater) return;
	      }

	      const animationNum = carousel === 'single' ? 1 : rowNum;
	      let rows = rowsData.slice(animationIndex);
	      rows.push(...rowsData.slice(0, animationIndex));
	      this.rows = rows.slice(0, carousel === 'page' ? rowNum * 2 : rowNum + 1);
	      this.heights = new Array(rowLength).fill(avgHeight);
	      await new Promise(resolve => setTimeout(resolve, 300));
	      if (updater !== this.updater) return;
	      this.heights.splice(0, animationNum, ...new Array(animationNum).fill(0));
	      animationIndex += animationNum;
	      const back = animationIndex - rowLength;
	      if (back >= 0) animationIndex = back;
	      this.animationIndex = animationIndex;
	      this.animationHandler = setTimeout(animation, waitTime - 300);
	    },

	    stopAnimation() {
	      const {
	        animationHandler,
	        updater
	      } = this;
	      this.updater = (updater + 1) % 999999;
	      if (!animationHandler) return;
	      clearTimeout(animationHandler);
	    },

	    emitEvent(type, ri, ci, row, ceil) {
	      const {
	        ceils,
	        rowIndex
	      } = row;
	      this.$emit(type, {
	        row: ceils,
	        ceil,
	        rowIndex,
	        columnIndex: ci
	      });
	    },

	    updateRows(rows, animationIndex) {
	      const {
	        mergedConfig,
	        animationHandler,
	        animation
	      } = this;
	      this.mergedConfig = { ...mergedConfig,
	        data: [...rows]
	      };
	      this.needCalc = true;
	      if (typeof animationIndex === 'number') this.animationIndex = animationIndex;
	      if (!animationHandler) animation(true);
	    }

	  },

	  destroyed() {
	    const {
	      stopAnimation
	    } = this;
	    stopAnimation();
	  }

	};

	/* script */
	const __vue_script__$1 = script$1;

	/* template */
	var __vue_render__$1 = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c("div", { ref: _vm.ref, staticClass: "dv-scroll-board" }, [
	    _vm.header.length && _vm.mergedConfig
	      ? _c(
	          "div",
	          {
	            staticClass: "header",
	            style: "background-color: " + _vm.mergedConfig.headerBGC + ";"
	          },
	          _vm._l(_vm.header, function(headerItem, i) {
	            return _c("div", {
	              key: "" + headerItem + i,
	              staticClass: "header-item",
	              style:
	                "\n        height: " +
	                _vm.mergedConfig.headerHeight +
	                "px;\n        line-height: " +
	                _vm.mergedConfig.headerHeight +
	                "px;\n        width: " +
	                _vm.widths[i] +
	                "px;\n      ",
	              attrs: { align: _vm.aligns[i] },
	              domProps: { innerHTML: _vm._s(headerItem) }
	            })
	          }),
	          0
	        )
	      : _vm._e(),
	    _vm._v(" "),
	    _vm.mergedConfig
	      ? _c(
	          "div",
	          {
	            staticClass: "rows",
	            style:
	              "height: " +
	              (_vm.height -
	                (_vm.header.length ? _vm.mergedConfig.headerHeight : 0)) +
	              "px;"
	          },
	          _vm._l(_vm.rows, function(row, ri) {
	            return _c(
	              "div",
	              {
	                key: "" + row.toString() + row.scroll,
	                staticClass: "row-item",
	                style:
	                  "\n        height: " +
	                  _vm.heights[ri] +
	                  "px;\n        line-height: " +
	                  _vm.heights[ri] +
	                  "px;\n        background-color: " +
	                  _vm.mergedConfig[
	                    row.rowIndex % 2 === 0 ? "evenRowBGC" : "oddRowBGC"
	                  ] +
	                  ";\n      "
	              },
	              _vm._l(row.ceils, function(ceil, ci) {
	                return _c("div", {
	                  key: "" + ceil + ri + ci,
	                  staticClass: "ceil",
	                  style: "width: " + _vm.widths[ci] + "px;",
	                  attrs: { align: _vm.aligns[ci] },
	                  domProps: { innerHTML: _vm._s(ceil) },
	                  on: {
	                    click: function($event) {
	                      return _vm.emitEvent("click", ri, ci, row, ceil)
	                    },
	                    mouseenter: function($event) {
	                      return _vm.handleHover(true, ri, ci, row, ceil)
	                    },
	                    mouseleave: function($event) {
	                      return _vm.handleHover(false)
	                    }
	                  }
	                })
	              }),
	              0
	            )
	          }),
	          0
	        )
	      : _vm._e()
	  ])
	};
	var __vue_staticRenderFns__$1 = [];
	__vue_render__$1._withStripped = true;

	  /* style */
	  const __vue_inject_styles__$1 = function (inject) {
	    if (!inject) return
	    inject("data-v-6171d3dd_0", { source: ".dv-scroll-board {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  color: #fff;\n}\n.dv-scroll-board .text {\n  padding: 0 10px;\n  box-sizing: border-box;\n  white-space: nowrap;\n  overflow: hidden;\n  text-overflow: ellipsis;\n}\n.dv-scroll-board .header {\n  display: flex;\n  flex-direction: row;\n  font-size: 15px;\n}\n.dv-scroll-board .header .header-item {\n  padding: 0 10px;\n  box-sizing: border-box;\n  white-space: nowrap;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  transition: all 0.3s;\n}\n.dv-scroll-board .rows {\n  overflow: hidden;\n}\n.dv-scroll-board .rows .row-item {\n  display: flex;\n  font-size: 14px;\n  transition: all 0.3s;\n}\n.dv-scroll-board .rows .ceil {\n  padding: 0 10px;\n  box-sizing: border-box;\n  white-space: nowrap;\n  overflow: hidden;\n  text-overflow: ellipsis;\n}\n.dv-scroll-board .rows .index {\n  border-radius: 3px;\n  padding: 0px 3px;\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,kBAAkB;EAClB,WAAW;EACX,YAAY;EACZ,WAAW;AACb;AACA;EACE,eAAe;EACf,sBAAsB;EACtB,mBAAmB;EACnB,gBAAgB;EAChB,uBAAuB;AACzB;AACA;EACE,aAAa;EACb,mBAAmB;EACnB,eAAe;AACjB;AACA;EACE,eAAe;EACf,sBAAsB;EACtB,mBAAmB;EACnB,gBAAgB;EAChB,uBAAuB;EACvB,oBAAoB;AACtB;AACA;EACE,gBAAgB;AAClB;AACA;EACE,aAAa;EACb,eAAe;EACf,oBAAoB;AACtB;AACA;EACE,eAAe;EACf,sBAAsB;EACtB,mBAAmB;EACnB,gBAAgB;EAChB,uBAAuB;AACzB;AACA;EACE,kBAAkB;EAClB,gBAAgB;AAClB","file":"main.vue","sourcesContent":[".dv-scroll-board {\n  position: relative;\n  width: 100%;\n  height: 100%;\n  color: #fff;\n}\n.dv-scroll-board .text {\n  padding: 0 10px;\n  box-sizing: border-box;\n  white-space: nowrap;\n  overflow: hidden;\n  text-overflow: ellipsis;\n}\n.dv-scroll-board .header {\n  display: flex;\n  flex-direction: row;\n  font-size: 15px;\n}\n.dv-scroll-board .header .header-item {\n  padding: 0 10px;\n  box-sizing: border-box;\n  white-space: nowrap;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  transition: all 0.3s;\n}\n.dv-scroll-board .rows {\n  overflow: hidden;\n}\n.dv-scroll-board .rows .row-item {\n  display: flex;\n  font-size: 14px;\n  transition: all 0.3s;\n}\n.dv-scroll-board .rows .ceil {\n  padding: 0 10px;\n  box-sizing: border-box;\n  white-space: nowrap;\n  overflow: hidden;\n  text-overflow: ellipsis;\n}\n.dv-scroll-board .rows .index {\n  border-radius: 3px;\n  padding: 0px 3px;\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__$1 = undefined;
	  /* module identifier */
	  const __vue_module_identifier__$1 = undefined;
	  /* functional template */
	  const __vue_is_functional_template__$1 = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__$1 = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__$1, staticRenderFns: __vue_staticRenderFns__$1 },
	    __vue_inject_styles__$1,
	    __vue_script__$1,
	    __vue_scope_id__$1,
	    __vue_is_functional_template__$1,
	    __vue_module_identifier__$1,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function scrollBoard (Vue) {
	  Vue.component(__vue_component__$1.name, __vue_component__$1);
	}

	//
	var script = {
	  name: 'DvScrollRankingBoard',
	  mixins: [autoResize],
	  props: {
	    config: {
	      type: Object,
	      default: () => ({})
	    }
	  },

	  data() {
	    return {
	      ref: 'scroll-ranking-board',
	      defaultConfig: {
	        /**
	         * @description Board data
	         * @type {Array<Object>}
	         * @default data = []
	         */
	        data: [],

	        /**
	         * @description Row num
	         * @type {Number}
	         * @default rowNum = 5
	         */
	        rowNum: 5,

	        /**
	         * @description Scroll wait time
	         * @type {Number}
	         * @default waitTime = 2000
	         */
	        waitTime: 2000,

	        /**
	         * @description Carousel type
	         * @type {String}
	         * @default carousel = 'single'
	         * @example carousel = 'single' | 'page'
	         */
	        carousel: 'single',

	        /**
	         * @description Value unit
	         * @type {String}
	         * @default unit = ''
	         * @example unit = 'ton'
	         */
	        unit: '',

	        /**
	         * @description Auto sort by value
	         * @type {Boolean}
	         * @default sort = true
	         */
	        sort: true,

	        /**
	         * @description Value formatter
	         * @type {Function}
	         * @default valueFormatter = null
	         */
	        valueFormatter: null
	      },
	      mergedConfig: null,
	      rowsData: [],
	      rows: [],
	      heights: [],
	      animationIndex: 0,
	      animationHandler: '',
	      updater: 0
	    };
	  },

	  watch: {
	    config() {
	      const {
	        stopAnimation,
	        calcData
	      } = this;
	      stopAnimation();
	      calcData();
	    }

	  },
	  methods: {
	    afterAutoResizeMixinInit() {
	      const {
	        calcData
	      } = this;
	      calcData();
	    },

	    onResize() {
	      const {
	        mergedConfig,
	        calcHeights
	      } = this;
	      if (!mergedConfig) return;
	      calcHeights(true);
	    },

	    calcData() {
	      const {
	        mergeConfig,
	        calcRowsData
	      } = this;
	      mergeConfig();
	      calcRowsData();
	      const {
	        calcHeights
	      } = this;
	      calcHeights();
	      const {
	        animation
	      } = this;
	      animation(true);
	    },

	    mergeConfig() {
	      let {
	        config,
	        defaultConfig
	      } = this;
	      this.mergedConfig = deepMerge(deepClone(defaultConfig, true), config || {});
	    },

	    calcRowsData() {
	      let {
	        data,
	        rowNum,
	        sort
	      } = this.mergedConfig;
	      sort && data.sort(({
	        value: a
	      }, {
	        value: b
	      }) => {
	        if (a > b) return -1;
	        if (a < b) return 1;
	        if (a === b) return 0;
	      });
	      const value = data.map(({
	        value
	      }) => value);
	      const min = Math.min(...value) || 0; // abs of min

	      const minAbs = Math.abs(min);
	      const max = Math.max(...value) || 0; // abs of max
	      const total = max + minAbs;
	      data = data.map((row, i) => ({ ...row,
	        ranking: i + 1,
	        percent: (row.value + minAbs) / total * 100
	      }));
	      const rowLength = data.length;

	      if (rowLength > rowNum && rowLength < 2 * rowNum) {
	        data = [...data, ...data];
	      }

	      data = data.map((d, i) => ({ ...d,
	        scroll: i
	      }));
	      this.rowsData = data;
	      this.rows = data;
	    },

	    calcHeights(onresize = false) {
	      const {
	        height,
	        mergedConfig
	      } = this;
	      const {
	        rowNum,
	        data
	      } = mergedConfig;
	      const avgHeight = height / rowNum;
	      this.avgHeight = avgHeight;
	      if (!onresize) this.heights = new Array(data.length).fill(avgHeight);
	    },

	    async animation(start = false) {
	      let {
	        avgHeight,
	        animationIndex,
	        mergedConfig,
	        rowsData,
	        animation,
	        updater
	      } = this;
	      const {
	        waitTime,
	        carousel,
	        rowNum
	      } = mergedConfig;
	      const rowLength = rowsData.length;
	      if (rowNum >= rowLength) return;

	      if (start) {
	        await new Promise(resolve => setTimeout(resolve, waitTime));
	        if (updater !== this.updater) return;
	      }

	      const animationNum = carousel === 'single' ? 1 : rowNum;
	      let rows = rowsData.slice(animationIndex);
	      rows.push(...rowsData.slice(0, animationIndex));
	      this.rows = rows.slice(0, rowNum + 1);
	      this.heights = new Array(rowLength).fill(avgHeight);
	      await new Promise(resolve => setTimeout(resolve, 300));
	      if (updater !== this.updater) return;
	      this.heights.splice(0, animationNum, ...new Array(animationNum).fill(0));
	      animationIndex += animationNum;
	      const back = animationIndex - rowLength;
	      if (back >= 0) animationIndex = back;
	      this.animationIndex = animationIndex;
	      this.animationHandler = setTimeout(animation, waitTime - 300);
	    },

	    stopAnimation() {
	      const {
	        animationHandler,
	        updater
	      } = this;
	      this.updater = (updater + 1) % 999999;
	      if (!animationHandler) return;
	      clearTimeout(animationHandler);
	    }

	  },

	  destroyed() {
	    const {
	      stopAnimation
	    } = this;
	    stopAnimation();
	  }

	};

	/* script */
	const __vue_script__ = script;

	/* template */
	var __vue_render__ = function() {
	  var _vm = this;
	  var _h = _vm.$createElement;
	  var _c = _vm._self._c || _h;
	  return _c(
	    "div",
	    { ref: _vm.ref, staticClass: "dv-scroll-ranking-board" },
	    _vm._l(_vm.rows, function(item, i) {
	      return _c(
	        "div",
	        {
	          key: item.toString() + item.scroll,
	          staticClass: "row-item",
	          style: "height: " + _vm.heights[i] + "px;"
	        },
	        [
	          _c("div", { staticClass: "ranking-info" }, [
	            _c("div", { staticClass: "rank" }, [
	              _vm._v("No." + _vm._s(item.ranking))
	            ]),
	            _vm._v(" "),
	            _c("div", {
	              staticClass: "info-name",
	              domProps: { innerHTML: _vm._s(item.name) }
	            }),
	            _vm._v(" "),
	            _c("div", { staticClass: "ranking-value" }, [
	              _vm._v(
	                _vm._s(
	                  _vm.mergedConfig.valueFormatter
	                    ? _vm.mergedConfig.valueFormatter(item)
	                    : item.value + _vm.mergedConfig.unit
	                )
	              )
	            ])
	          ]),
	          _vm._v(" "),
	          _c("div", { staticClass: "ranking-column" }, [
	            _c(
	              "div",
	              {
	                staticClass: "inside-column",
	                style: "width: " + item.percent + "%;"
	              },
	              [_c("div", { staticClass: "shine" })]
	            )
	          ])
	        ]
	      )
	    }),
	    0
	  )
	};
	var __vue_staticRenderFns__ = [];
	__vue_render__._withStripped = true;

	  /* style */
	  const __vue_inject_styles__ = function (inject) {
	    if (!inject) return
	    inject("data-v-722956d4_0", { source: ".dv-scroll-ranking-board {\n  width: 100%;\n  height: 100%;\n  color: #fff;\n  overflow: hidden;\n}\n.dv-scroll-ranking-board .row-item {\n  transition: all 0.3s;\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  overflow: hidden;\n}\n.dv-scroll-ranking-board .ranking-info {\n  display: flex;\n  width: 100%;\n  font-size: 13px;\n}\n.dv-scroll-ranking-board .ranking-info .rank {\n  width: 40px;\n  color: #1370fb;\n}\n.dv-scroll-ranking-board .ranking-info .info-name {\n  flex: 1;\n}\n.dv-scroll-ranking-board .ranking-column {\n  border-bottom: 2px solid rgba(19, 112, 251, 0.5);\n  margin-top: 5px;\n}\n.dv-scroll-ranking-board .ranking-column .inside-column {\n  position: relative;\n  height: 6px;\n  background-color: #1370fb;\n  margin-bottom: 2px;\n  border-radius: 1px;\n  overflow: hidden;\n}\n.dv-scroll-ranking-board .ranking-column .shine {\n  position: absolute;\n  left: 0%;\n  top: 2px;\n  height: 2px;\n  width: 50px;\n  transform: translateX(-100%);\n  background: radial-gradient(#28f8ff 5%, transparent 80%);\n  animation: shine 3s ease-in-out infinite alternate;\n}\n@keyframes shine {\n80% {\n    left: 0%;\n    transform: translateX(-100%);\n}\n100% {\n    left: 100%;\n    transform: translateX(0%);\n}\n}\n", map: {"version":3,"sources":["main.vue"],"names":[],"mappings":"AAAA;EACE,WAAW;EACX,YAAY;EACZ,WAAW;EACX,gBAAgB;AAClB;AACA;EACE,oBAAoB;EACpB,aAAa;EACb,sBAAsB;EACtB,uBAAuB;EACvB,gBAAgB;AAClB;AACA;EACE,aAAa;EACb,WAAW;EACX,eAAe;AACjB;AACA;EACE,WAAW;EACX,cAAc;AAChB;AACA;EACE,OAAO;AACT;AACA;EACE,gDAAgD;EAChD,eAAe;AACjB;AACA;EACE,kBAAkB;EAClB,WAAW;EACX,yBAAyB;EACzB,kBAAkB;EAClB,kBAAkB;EAClB,gBAAgB;AAClB;AACA;EACE,kBAAkB;EAClB,QAAQ;EACR,QAAQ;EACR,WAAW;EACX,WAAW;EACX,4BAA4B;EAC5B,wDAAwD;EACxD,kDAAkD;AACpD;AACA;AACE;IACE,QAAQ;IACR,4BAA4B;AAC9B;AACA;IACE,UAAU;IACV,yBAAyB;AAC3B;AACF","file":"main.vue","sourcesContent":[".dv-scroll-ranking-board {\n  width: 100%;\n  height: 100%;\n  color: #fff;\n  overflow: hidden;\n}\n.dv-scroll-ranking-board .row-item {\n  transition: all 0.3s;\n  display: flex;\n  flex-direction: column;\n  justify-content: center;\n  overflow: hidden;\n}\n.dv-scroll-ranking-board .ranking-info {\n  display: flex;\n  width: 100%;\n  font-size: 13px;\n}\n.dv-scroll-ranking-board .ranking-info .rank {\n  width: 40px;\n  color: #1370fb;\n}\n.dv-scroll-ranking-board .ranking-info .info-name {\n  flex: 1;\n}\n.dv-scroll-ranking-board .ranking-column {\n  border-bottom: 2px solid rgba(19, 112, 251, 0.5);\n  margin-top: 5px;\n}\n.dv-scroll-ranking-board .ranking-column .inside-column {\n  position: relative;\n  height: 6px;\n  background-color: #1370fb;\n  margin-bottom: 2px;\n  border-radius: 1px;\n  overflow: hidden;\n}\n.dv-scroll-ranking-board .ranking-column .shine {\n  position: absolute;\n  left: 0%;\n  top: 2px;\n  height: 2px;\n  width: 50px;\n  transform: translateX(-100%);\n  background: radial-gradient(#28f8ff 5%, transparent 80%);\n  animation: shine 3s ease-in-out infinite alternate;\n}\n@keyframes shine {\n  80% {\n    left: 0%;\n    transform: translateX(-100%);\n  }\n  100% {\n    left: 100%;\n    transform: translateX(0%);\n  }\n}\n"]}, media: undefined });

	  };
	  /* scoped */
	  const __vue_scope_id__ = undefined;
	  /* module identifier */
	  const __vue_module_identifier__ = undefined;
	  /* functional template */
	  const __vue_is_functional_template__ = false;
	  /* style inject SSR */
	  
	  /* style inject shadow dom */
	  

	  
	  const __vue_component__ = /*#__PURE__*/normalizeComponent(
	    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
	    __vue_inject_styles__,
	    __vue_script__,
	    __vue_scope_id__,
	    __vue_is_functional_template__,
	    __vue_module_identifier__,
	    false,
	    createInjector,
	    undefined,
	    undefined
	  );

	function scrollRankingBoard (Vue) {
	  Vue.component(__vue_component__.name, __vue_component__);
	}

	/**
	 * IMPORT COMPONENTS
	 */
	/**
	 * USE COMPONENTS
	 */

	function datav (Vue) {
	  Vue.use(fullScreenContainer);
	  Vue.use(loading); // border box

	  Vue.use(borderBox1);
	  Vue.use(borderBox2);
	  Vue.use(borderBox3);
	  Vue.use(borderBox4);
	  Vue.use(borderBox5);
	  Vue.use(borderBox6);
	  Vue.use(borderBox7);
	  Vue.use(borderBox8);
	  Vue.use(borderBox9);
	  Vue.use(borderBox10);
	  Vue.use(borderBox11);
	  Vue.use(borderBox12);
	  Vue.use(borderBox13); // decoration

	  Vue.use(decoration1);
	  Vue.use(decoration2);
	  Vue.use(decoration3);
	  Vue.use(decoration4);
	  Vue.use(decoration5);
	  Vue.use(decoration6);
	  Vue.use(decoration7);
	  Vue.use(decoration8);
	  Vue.use(decoration9);
	  Vue.use(decoration10);
	  Vue.use(decoration11);
	  Vue.use(decoration12); // charts

	  Vue.use(charts);
	  Vue.use(activeRingChart);
	  Vue.use(capsuleChart);
	  Vue.use(waterLevelPond);
	  Vue.use(percentPond);
	  Vue.use(flylineChart);
	  Vue.use(flylineChartEnhanced);
	  Vue.use(conicalColumnChart);
	  Vue.use(digitalFlop);
	  Vue.use(scrollBoard);
	  Vue.use(scrollRankingBoard);
	}

	Vue__default['default'].use(datav);

})));
